/**
 * WlanMap 0.9
 *
 * Copyright 2013      Konrad Enzensberger <e.konrad@mpegcode.com>
 * All rights reserved.
 *
 * WlanMap is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WlanMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.*
 *
 * You should have received a copy of the GNU General Public License
 * along with wlanapi library.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>

#include "ControlView.h"
#include "apmp.h"

QT_BEGIN_NAMESPACE
class QAction;
class QMenu;
class QTextEdit;
QT_END_NAMESPACE

namespace Ui {
    class qSettings;
}

namespace Marble
{

//! [class definition with macro]
class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow();
	~MainWindow();
	void init();
	void setDataPath(QString qsPath){m_qsMapDataPath=qsPath;}
protected:
    void closeEvent(QCloseEvent *event);

private slots:
  
    void open();
    bool save();
    bool saveAs();
    void about();
	void aboutMarblewidget();
	void help();

	//Wlan
	void slotWlanDetection();
	void slotWlanMeasure();
	void slotSettings();
	//Wlan settings
	void slotTimer(int);
	void slotContinous(bool);
	void slotSingle(bool);
	//access point settings
	void slotSelectedAP(bool);
	void slotStrongestAP(bool);
	void slotAllAP(bool);
	//Measure points
	void slotMPgps(bool);
	void slotMPmanual(bool);
	//Map
	void slotLoadOsmMap();
	void slotLoadGoogleMap();
	void SlotSetGlobeProj();
	void SlotSetMercProj();
	void SlotSetEquiProj();
	void SlotAddImageLayer();

	//GPS
	void slotGps();
	void receiveGpsCoordinates( GeoDataCoordinates, qreal );
	void slotHomeChanged( GeoDataCoordinates);

	//Tree
	void slotSendEvent(long type,long size,void* pData,long intID);
	void slotSelectedItem(long overlayID,long layerID,int id);

	//marble
	void  showPosition( const QString& position);
    void  showDistance( const QString& position);
	void  showElevationValue(qreal value);
	void  slotCoordinateFormat(qreal,qreal,long);
	//info
	void showWlanSettings();

	//heatmap
	void slotCreateHeatmap();

private:
 
	void deleteMapDirectory(QString&);

    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();

	void setProjection(int iProj);
   
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName);

	void setMessPoint( GeoDataCoordinates pos);
	void scanSelectedAP();
	void scanStrongestAP();
	void scanAllAP();

	void updateStatusBar();
	void setupStatusBar();

	void setLayerVisible(long id,long visible,QString&);

private:
    QString curFile;
    bool isUntitled;
	QString m_qsMapDataPath;
	QString m_lastFileOpenPath;

    QMenu *fileMenu;
    QMenu *wlanMenu;
	QMenu *mapMenu;
	QMenu *basemapMenu;
	QMenu *projMenu;
    QMenu *helpMenu;
	QMenu *settingsMenu;

    QToolBar *fileToolBar;
    QToolBar *wlanToolBar;
    
    QAction *openAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *exitAct;

	QAction *wlanDetectAct;
	QAction *wlanMeasureAct;
	QAction *settingsAct;
	QAction *heatmapAct;

	QAction *osmAct;
	QAction *googleAct;
	QAction *globeAct;
	QAction *mercatorAct;
	QAction *flatAct;
	QAction *importAct;
   
    QAction *aboutAct;
    QAction *aboutQtAct;
	QAction *aboutMarbleAct;
	QAction *manualAct;

	//Status bar
	QString  m_position;
    QString  m_distance;
	QString  m_elevation;
	QString  m_info;
	QLabel  *m_positionLabel;
	QLabel  *m_distanceLabel;   
	QLabel  *m_elevationLabel;
	QLabel  *m_infoLabel;

	//Map
	ControlView*	m_controlView;
	QString			m_qsThemeId;
	qreal			m_dCenterLon;
	qreal			m_dCenterLat;
	int				m_iProjection;

	//GPS
	qreal			m_dGpsLon;
	qreal			m_dGpsLat;
	bool			m_bGPS;
	int				m_iZoom;

	//WLAN
	QHash <QString,long> wlanlay;
	QHash <QString,long> shadinglay;
	QHash <QString,long> qlAdapter;
	QHash <long,QApMp*> m_qhMP;

	Ui::qSettings * m_uiWlanSettings;
	bool	m_bContinous;
	bool	m_bMP;
	int	    m_iAP;
	int		m_iTimer;
	long	m_lOverlay;
	long	m_lLayer;
	long	m_lActiveLayer;
	long	m_lAdapterOverlay;
	long	m_lShadingOverlay;
	long	m_lShadingLayer;
	int		m_iCount;
};

}

#endif
