/*******************************************************************************
* @file			EsriRunnerPlugin.cpp
* @copyright	&copy;2011-2012 Rohde&Schwarz, Munich
* @version		$Header: \ISAR_dev\userinterface\MapViewControl\marblewidget\EsriRunnerPlugin.h
*
* @author		Konrad Enzensberger
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "EsriRunnerPlugin.h"

namespace Marble
{

class EsriRunnerPlugin::Private
{
public:
    QStringList m_supportedCelestialBodies;

    bool m_canWorkOffline;

    Private();
};

EsriRunnerPlugin::Private::Private()
    : m_canWorkOffline( true )
{
    // nothing to do
}

EsriRunnerPlugin::EsriRunnerPlugin( QObject* parent ) :
    QObject( parent ),
    d( new Private )
{
}

EsriRunnerPlugin::~EsriRunnerPlugin()
{
    delete d;
}

QIcon EsriRunnerPlugin::icon() const
{
    return QIcon();
}

bool EsriRunnerPlugin::supportsCelestialBody( const QString &celestialBodyId ) const
{
    if ( d->m_supportedCelestialBodies.isEmpty() ) {
        return true;
    }

    return d->m_supportedCelestialBodies.contains( celestialBodyId );
}

void EsriRunnerPlugin::setSupportedCelestialBodies( const QStringList &celestialBodies )
{
    d->m_supportedCelestialBodies = celestialBodies;
}

void EsriRunnerPlugin::setCanWorkOffline( bool canWorkOffline )
{
    d->m_canWorkOffline = canWorkOffline;
}

bool EsriRunnerPlugin::canWorkOffline() const
{
    return d->m_canWorkOffline;
}

bool EsriRunnerPlugin::canWork() const
{
    return true;
}

}

#include "moc_EsriRunnerPlugin.cpp"
