/*******************************************************************************
* @file			IconViewWidget.cpp
* @copyright	2011-2013 Rohde&Schwarz, Munich
* @version		$Header: IconViewWidget.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "IconViewWidget.h"

#include "MarbleWidget.h"
#include "MarbleDirs.h"

#include <QtGui>
#include <QtCore/QMimeData>
#include <QtGui/QDragEnterEvent>
#include <QGridLayout>
#include <QDir>
#include <QMenu>
#include <QtGui/QIcon>
#include <QtGui/QPixmap>

namespace Marble
{
	class IconViewWidgetPrivate
	{
		public:
		//	Ui::Form m_ui;

			MarbleWidget* m_marbleWidget;

			IconViewWidgetPrivate();

	};

	IconViewWidgetPrivate::IconViewWidgetPrivate() :
        m_marbleWidget( 0 )
	{


	}

IconViewWidget::IconViewWidget(QWidget *parent)
     : QFrame(parent), d( new IconViewWidgetPrivate )
 {
     setMinimumSize(100,80);
     setFrameStyle(QFrame::Sunken | QFrame::StyledPanel);
  //   setAcceptDrops(true);

	 setWindowTitle("Icon View");

//	 tagCloudScrollArea = new QScrollArea(this); 
//	 tagCloudScrollArea->setWidget(this);
	
	 m_pGrid=NULL;
	 m_iMvSym=0;
 }

IconViewWidget::~IconViewWidget()
{
	m_iconTab.clear();
	if(m_pGrid!=NULL)
		delete m_pGrid;
	delete d;
}

void IconViewWidget::initializeIconGrid(QString& SymDir,bool path)
{
	if(m_pGrid!=NULL)
	{
		int iIco = m_iconTab.size();
		for(int c=0;c<iIco;c++)
		{
			QLabel* pIcon = (QLabel*)m_iconTab.value(c);
			if(pIcon!=NULL)
			{
				m_pGrid->removeWidget(pIcon);
				delete pIcon;
			}
		}

		m_iconTab.clear();
		delete m_pGrid;
		m_pGrid=NULL;
	}

	if(m_pGrid==NULL)
	{
		m_pGrid = new QGridLayout( this );
		//m_pGrid->setSpacing(32);
	}

	QLabel* pLabel=NULL;
	QString qsIcon;

	if(path)
	{
		qsIcon = SymDir;
	}
	else if(!m_iMvSym)
		qsIcon = MarbleDirs::path(SymDir);
	else
	{
		qsIcon = m_qsRamonData + "/" + SymDir;
	}
	int y=0;
	int x=0;
	m_iMvSym=0;

	QDir dir;
	//dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	dir.setSorting(QDir::Size | QDir::Reversed);

	QString qsDir = qsIcon;
	dir.setPath(qsDir);
	if (!dir.exists())
	  return;

    QString qsFile;
	QStringList qList = dir.entryList();
	int iC = qList.count();
	int iPos=0;

/*	 QRect rect = frameRect();
	 int iHeight = rect.height();
	 iPos = (iC-2)/5 + 1;
	 iPos = iPos*15;
	 if(iPos>iHeight)
	 {
		 iPos = iPos-iHeight;
		 rect.setTop(rect.top()+iPos);
		 setFrameRect(rect);
	 }
	 if(iPos<iHeight)
	 {
		 iPos = iHeight-iPos;
		 rect.setTop(rect.top()-iPos);
		 setFrameRect(rect);
	 }

	 iPos=0;*/
	for(int i=0;i<iC;i++)
	{
	    qsIcon.clear();
		qsFile = qList.at(i);
		if(qsFile!=".." && qsFile!=".")
		{
			if(qsFile.contains('.'))
			{
				qsIcon = qsDir + "/" + qsFile;
				pLabel = new QLabel(this);
				pLabel->setPixmap(QPixmap(qsIcon) );
		//	pLabel->move(x*40+20, y*40+20);
		//	pLabel->show();
				pLabel->setAttribute(Qt::WA_DeleteOnClose);
				pLabel->setAccessibleName(qsIcon);

				m_pGrid->addWidget(pLabel,y,x,Qt::AlignCenter);
				m_iconTab.insert(iPos++,pLabel);

				x++;
				if(x>3)
				{
					x=0;
					y++;
				}
			}
		}

	}


/*	QRect qRect = geometry();

	QSize qs = size();
	//int iH = height();
	int iMin = (iC/4)*30;
	qRect.setTop(qRect.bottom()+iMin);
//	setGeometry(qRect);
//	updateGeometry();
	repaint(qRect);

//	resize(qRect.width(),qRect.height());

//	resize(qs.width(),iMin);*/

	int iH = height();
	int iMin = (iC/4)*40;
	if(iH < iMin)
	{
	//	setMinimumHeight(iMin);
		setMaximumHeight(iMin);
	}
	else if(iH > iMin)
	{
	//	setMinimumHeight(iMin);
		if(iMin<80)iMin=80;
		setMaximumHeight(iMin);
	}

}

void IconViewWidget::addMarbleWidget(MarbleWidget *marbleWidget,QString& ramonData)
{
	d->m_marbleWidget = marbleWidget;
	if(ramonData!="no")
		m_qsRamonData = ramonData;
	else m_qsRamonData.clear();
}

 void IconViewWidget::dragEnterEvent(QDragEnterEvent *event)
 {
     if (event->mimeData()->hasFormat("application/x-dnditemdata")) {
         if (event->source() == this) {
             event->setDropAction(Qt::MoveAction);
             event->accept();
         } else {
             event->acceptProposedAction();
         }
     } else {
         event->ignore();
     }
 }

 void IconViewWidget::dragMoveEvent(QDragMoveEvent *event)
 {
     if (event->mimeData()->hasFormat("application/x-dnditemdata")) {
         if (event->source() == this) {
             event->setDropAction(Qt::MoveAction);
             event->accept();
         } else {
             event->acceptProposedAction();
         }
     } else {
         event->ignore();
     }
 }

 void IconViewWidget::dropEvent(QDropEvent *event)
 {
     if (event->mimeData()->hasFormat("application/x-dnditemdata")) {
         QByteArray itemData = event->mimeData()->data("application/x-dnditemdata");
         QDataStream dataStream(&itemData, QIODevice::ReadOnly);

         QPixmap pixmap;
         QPoint offset;
         dataStream >> pixmap >> offset;

         QLabel *newIcon = new QLabel(this);
         newIcon->setPixmap(pixmap);
         newIcon->move(event->pos() - offset);
         newIcon->show();
         newIcon->setAttribute(Qt::WA_DeleteOnClose);

         if (event->source() == this) {
             event->setDropAction(Qt::MoveAction);
             event->accept();
         } else {
             event->acceptProposedAction();
         }
     } else {
         event->ignore();
     }
 }

 void IconViewWidget::mousePressEvent(QMouseEvent *event)
 {
	 if ( event->button() == Qt::LeftButton)	
	 {
		 QLabel *child = static_cast<QLabel*>(childAt(event->pos()));
		 if (!child)
			 return;

		 QPixmap pixmap = *child->pixmap();

		 QByteArray itemData;
		 QDataStream dataStream(&itemData, QIODevice::WriteOnly);
		 dataStream << pixmap << QPoint(event->pos() - child->pos());

		 QMimeData *mimeData = new QMimeData;
		 mimeData->setText(child->accessibleName());
		 mimeData->setData("application/x-dnditemdata", itemData);

		 QDrag *drag = new QDrag(this);
		 drag->setMimeData(mimeData);
		 drag->setPixmap(pixmap);
		 drag->setHotSpot(event->pos() - child->pos());

		 QPixmap tempPixmap = pixmap;
		 QPainter painter;
		 painter.begin(&tempPixmap);
		 painter.fillRect(pixmap.rect(), QColor(127, 127, 127, 127));
		 painter.end();

		 child->setPixmap(tempPixmap);

		 if (drag->exec(Qt::CopyAction | Qt::MoveAction, Qt::CopyAction) == Qt::MoveAction)
			 child->close();
		 else {
			 child->show();
			 child->setPixmap(pixmap);
		 }
	 }
	 if ( event->button() == Qt::RightButton)
	 {
		 QDir dir;
		 //dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
		 dir.setSorting(QDir::Size | QDir::Reversed);

		 QString qsDir = MarbleDirs::path("Symbols");
		 dir.setPath(qsDir);
		 if (!dir.exists())
			 return;

		 m_qsList.clear();
		 m_qsList = dir.entryList();
		 int iC = m_qsList.count();
		 if(iC<=0)
			 return;

		 bool use=false;
		 int iRC=0;
		 m_iMvSym=iC;
		 if(!m_qsRamonData.isEmpty())
		 {
			qsDir.clear();
			qsDir = m_qsRamonData + "/Symbols";
			dir.setPath(qsDir);
			if (dir.exists())
			{
				 m_qsList += dir.entryList();
				 iRC =  m_qsList.count() - iC;
			}

		 }

		 QString qsMenu;
		 QAction* pAction=NULL;
		 QIcon qIco;

		 QMenu* qMenu = new QMenu(this);
		 connect(qMenu, SIGNAL( triggered(QAction*) ),this, SLOT( slotSelectActionMenu(QAction*) ) );
		 for(int i=0;i<iC+iRC;i++)
		 {
		    if(i==iC)
			{
				qMenu->addSeparator();
				use=true;
			}
			qsMenu = m_qsList.at(i);
		    if(qsMenu != ".." && qsMenu != ".")
			{
				qIco = getIcon(qsMenu,use);
				pAction = new QAction(qsMenu, this );
				if(!qIco.isNull())
					pAction->setIcon(qIco);
				pAction->setData(i);
				qMenu->addAction(pAction );
			}

		 }

		 QPoint pt;
		 pt.setX( event->pos().x());
		 pt.setY( event->pos().y());
		// qMenu->popup(pt);
		 qMenu->popup(mapToGlobal( pt ) );

	 }

 }

 QIcon IconViewWidget::getIcon(QString& qsDir,bool use)
 {
	 QIcon qIco;
	 QPixmap qPix;
	 QString qsIco;
	 QString qsIconDir;

	 qsIco = "Symbols/" + qsDir;
	 
	 if(!use)
		qsIconDir = MarbleDirs::path(qsIco);
	 else if(!m_qsRamonData.isEmpty())
	 {
		qsIconDir = m_qsRamonData + "/" + qsIco;
	 }

	 if(qsIconDir.isEmpty())
		 return qIco;

	 qsIco.clear();

	QDir dir;
	//dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
	dir.setSorting(QDir::Size | QDir::Reversed);
	dir.setPath(qsIconDir);
	if (!dir.exists())
		return qIco;

	QStringList qList = dir.entryList();
	int iC = qList.count();
	if(iC<1)
		return qIco;

	for(int i=0;i<iC;i++)
	{
		qsIco = qList.at(i);
		if(qsIco != ".." && qsIco != ".")
		{
			if(qsDir=="milsym")
			{
				if(qsIco=="milsym7.png")
					i=iC;
			}
			else if(qsDir=="mapsym")
			{
				if(qsIco=="9.png")
					i=iC;
			}
			else if(qsDir=="citysym")
			{
				if(qsIco=="B4_WC.png")
					i=iC;
			}
			else i=iC;
		}
	}
	if(qsIco.isEmpty())
		return qIco;

	qsIconDir +="/";
	qsIconDir += qsIco;
	qPix.load(qsIconDir);
	qIco = QIcon( qPix );

	return qIco;

 }

 void IconViewWidget::slotSelectActionMenu(QAction* pAction)
 {
	 if(pAction==NULL)return;
	 if(m_qsList.count()<=0)return;

	 int iC = m_qsList.count();
	 
	 QVariant qVar = pAction->data();
	 int iVal = qVar.toInt();
	 if(iVal<0 || iVal>=iC)return;

	  if(iVal<m_iMvSym)
		 m_iMvSym=0;

	 QString qsSym = m_qsList.at(iVal);
	 QString qsDir("Symbols\\");
	 qsDir += qsSym;

	 qsSym.clear();

	 if(m_iMvSym==0)
		  qsSym = MarbleDirs::path(qsDir);
	 else qsSym = m_qsRamonData + '\\' + qsDir;

	 initializeIconGrid(qsDir,false);
	 emit signalIconSelectionChanged(qsSym);

 }

} // namespace Marble

#include "moc_IconViewWidget.cpp"
