//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2012      Dennis Nienhüser <earthwings@gentoo.org>
//

#include "GeoImageGraphicsItem.h"

#include "GeoGraphicsItem_p.h"
#include "GeoPainter.h"
#include "ViewportParams.h"

namespace Marble
{

void GeoImageGraphicsItem::setLatLonBox( const GeoDataLatLonBox &box )
{
    p()->m_latLonAltBox = box;
}

GeoDataLatLonBox& GeoImageGraphicsItem::latLonBox() const
{
    return p()->m_latLonAltBox;
}

void GeoImageGraphicsItem::setImage( const QImage &image )
{
    m_image = image;
}

QImage GeoImageGraphicsItem::image() const
{
    return m_image;
}

void GeoImageGraphicsItem::setImageFile( const QString &filename )
{
    m_imageFile = filename;
}

QString GeoImageGraphicsItem::imageFile() const
{
    return m_imageFile;
}

void GeoImageGraphicsItem::paint( GeoPainter* painter, const ViewportParams* viewport )
{
    bool unloadImage = true;
    if ( viewport->projection() != Spherical ) {
        qreal x1(0.0), x2(0.0), y1( 0.0 ), y2( 0.0 );
        viewport->screenCoordinates( p()->m_latLonAltBox.west(), p()->m_latLonAltBox.north(), x1, y1 );
        viewport->screenCoordinates( p()->m_latLonAltBox.east(), p()->m_latLonAltBox.south(), x2, y2 );
        QRectF const screen( QPointF( 0, 0), viewport->size() );
        QRectF const position( x1, y1, x2-x1, y2-y1 );
        if ( !(screen & position).isEmpty() ) {
            if ( m_image.isNull() && !m_imageFile.isEmpty() ) {
                /** @todo: Load in a thread */
                m_image = QImage( m_imageFile );
            }
            unloadImage = false;
			//heading enz
			painter->translate( viewport->width()/2, viewport->height()/2 ); 
			painter->rotate( viewport->heading() * RAD2DEG ); 
			painter->translate( -viewport->width()/2, -viewport->height()/2 );
			//enz
            painter->drawImage( position, m_image );
        }
    } else 
	{
        /** @todo: Implement for spherical projection, possibly reusing code
                   from SphericalScanlineTextureMapper */
		/** enz - modification made in SphericalProjection::screenCoordinates(...)	*/

		//	p()->m_latLonAltBox.setRotation(viewport->heading());// * RAD2DEG);
		
		qreal x1(1111.0), x2(1111.0), y1( 1111.0 ), y2( 1111.0 ); 
        viewport->screenCoordinates( p()->m_latLonAltBox.west(), p()->m_latLonAltBox.north(), x1, y1 );
        viewport->screenCoordinates( p()->m_latLonAltBox.east(), p()->m_latLonAltBox.south(), x2, y2 );
		painter->setCompositionMode(QPainter::RasterOp_SourceAndDestination);
    //  painter->drawImage( QRectF( x1, y1, x2-x1, y2-y1 ), m_image );
		const int radius = (int)(1.05 * (qreal)(viewport->radius()));
		const qreal  inverseRadius = 1.0 / (qreal)(viewport->radius());

		QRect rect( viewport->width() / 2 - radius, viewport->height() / 2 - radius,2 * radius, 2 * radius);

		QRect const screen( QPoint( 0, 0), viewport->size() );
        QRect const position( x1, y1, x2-x1, y2-y1 );
        if ( !(screen & position).isEmpty() ) {
            if ( m_image.isNull() && !m_imageFile.isEmpty() ) {
                /** @todo: Load in a thread */
                m_image = QImage( m_imageFile );
            }
            unloadImage = false;
			//heading enz
			painter->translate( viewport->width()/2, viewport->height()/2 ); 
			painter->rotate(viewport->heading() * RAD2DEG ); 
			painter->translate( -viewport->width()/2, -viewport->height()/2 );
		/*	QMatrix rm;
			rm.rotate(viewport->heading() * RAD2DEG );
			QImage img = m_image.transformed(rm);*/
		/*	QTransform rotate_disc;
			rotate_disc.translate(m_image.width()/2.0 , m_image.height()/2.0);
			rotate_disc.rotate(viewport->heading() * RAD2DEG);
			rotate_disc.translate(-m_image.width()/2.0 , -m_image.height()/2.0);

			//Transform coordinate system.
			painter->setTransform( rotate_disc );*/

			//Load image.

			//Notice: As coordinate system is set at center of window, we have to center the image also... so the minus offset to come to center of image.
			//painter.drawPixmap( -loadingDisk.width()/2, -loadingDisk.height()/2, loadingDisk );

			//enz
			
/*			QTransform rot;
			rot.rotate(viewport->heading() * RAD2DEG);
			QImage image = m_image.transformed(rot);*/
	/*		QTransform transform;
            transform.translate( -m_image.width() / 2, -m_image.height() / 2 );
            transform.rotateRadians(viewport->heading());// * RAD2DEG);
            transform.translate( m_image.width() / 2, m_image.height() / 2 );

			rect = rect.intersect( position );
			painter->drawImage( rect,m_image.transformed( transform,Qt::SmoothTransformation ) );*/
            painter->drawImage(position,m_image );
		}
    }
    /** @todo: Respect x-repeat */

    if ( unloadImage && !m_imageFile.isEmpty() ) {
        // No unloading if no path is known
        m_image = QImage();
    }
}
}
