/*******************************************************************************
* @file			KmlGroundOverlayTagWriter.cpp
* @copyright	&copy;2011-2012 Rohde&Schwarz, Munich
* @version		$Header: \ISAR_dev\userinterface\MapViewControl\marblewidget\Geodata\writers\kml\KmlGroundOverlayTagWriter.h
*
* @author		Konrad Enzensberger
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "KmlGroundOverlayTagWriter.h"

#include "GeoDataOverlay.h"
#include "GeoDataGroundOverlay.h"
#include "GeoDataTypes.h"
#include "GeoWriter.h"
#include "KmlElementDictionary.h"

namespace Marble
{

static GeoTagWriterRegistrar s_writerPoint( GeoTagWriter::QualifiedName(GeoDataTypes::GeoDataGroundOverlayType,
                                                                            kml::kmlTag_nameSpace22),
                                               new KmlGroundOverlayTagWriter() );


bool KmlGroundOverlayTagWriter::write( const GeoNode *node,
                               GeoWriter& writer ) const
{
    const GeoDataGroundOverlay *overlay = static_cast<const GeoDataGroundOverlay*>( node );

	if(overlay==NULL)return false;
	
	const GeoDataLatLonBox box = overlay->latLonBox();
	const QString icon = overlay->iconFile();
	QString north = QString::number( box.north( GeoDataCoordinates::Degree ), 'f', 10 );
	QString south = QString::number( box.south( GeoDataCoordinates::Degree ), 'f', 10 );
	QString east  = QString::number( box.east( GeoDataCoordinates::Degree ), 'f', 10 );
	QString west  = QString::number( box.west( GeoDataCoordinates::Degree ), 'f', 10 );

	writer.writeStartElement( kml::kmlTag_GroundOverlay );

	//Icon
	writer.writeStartElement( kml::kmlTag_Icon );
	writer.writeStartElement( kml::kmlTag_href );
	writer.writeCharacters( icon );
	writer.writeEndElement();
	writer.writeEndElement();

	//LatLonBox
	writer.writeStartElement( "LatLonBox" );
	
	writer.writeStartElement( "north" );
	writer.writeCharacters(north );
	writer.writeEndElement();

	writer.writeStartElement( "south" );
	writer.writeCharacters(south );
	writer.writeEndElement();

	writer.writeStartElement( "east" );
	writer.writeCharacters(east );
	writer.writeEndElement();

	writer.writeStartElement( "west" );
	writer.writeCharacters(west );
	writer.writeEndElement();
	 
	writer.writeEndElement();

	writer.writeEndElement();

    return true;
}

}
