/*******************************************************************************
* @file			MvComboBox.cpp
* @copyright	2011-2012 Rohde&Schwarz, Munich
* @version		$Header: MvComboBox.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/
#include "MvComboBox.h"

#include "MarbleDirs.h"

#include <QList>
#include <QPair>

#include <QPainter>
#include <QPen>

namespace Marble
{

MvPenStyleComboBox::MvPenStyleComboBox( QWidget* parent,int width )
    : QComboBox( parent )
{
  QList < QPair<Qt::PenStyle, QString> > styles;
  styles << qMakePair( Qt::SolidLine, tr( "  Solid Line" ) )
  << qMakePair( Qt::DashLine, tr( "  Dash Line" ) )
  << qMakePair( Qt::DotLine, tr( "  Dot Line" ) )
  << qMakePair( Qt::DashDotLine, tr( "  Dash Dot Line" ) )
  << qMakePair( Qt::DashDotDotLine, tr( "  Dash Dot Dot Line" ) )
  << qMakePair( Qt::NoPen, tr( "  No Pen" ) );

  setIconSize( QSize( 40, 12 ) );

  for ( int i = 0; i < styles.count(); i++ )
  {
    Qt::PenStyle style = styles.at( i ).first;
    QString name = styles.at( i ).second;
    addItem( iconForPen( style ), name, QVariant( style ) );
  }
}

Qt::PenStyle MvPenStyleComboBox::penStyle() const
{
  return ( Qt::PenStyle ) itemData( currentIndex() ).toInt();
}

void MvPenStyleComboBox::setPenStyle( Qt::PenStyle style )
{
  int idx = findData( QVariant( style ) );
  setCurrentIndex( idx == -1 ? 0 : idx );
}

QIcon MvPenStyleComboBox::iconForPen( Qt::PenStyle style )
{
  QPixmap pix( iconSize() );
  QPainter p;
  pix.fill( Qt::transparent );

  p.begin( &pix );
  QPen pen( style );
  pen.setWidth( 2 );
  p.setPen( pen );
  double mid = iconSize().height() / 2.0;
  p.drawLine( 0, mid, iconSize().width(), mid );
  p.end();

  return QIcon( pix );
}


/////////
// join

MvPenJoinStyleComboBox::MvPenJoinStyleComboBox( QWidget* parent )
    : QComboBox( parent )
{
  QString path = ":bitmaps";//QgsApplication::defaultThemePath(); cb_line.png
  addItem( QIcon( path + "/join_bevel.png" ), tr( "Bevel" ), QVariant( Qt::BevelJoin ) );
  addItem( QIcon( path + "/join_miter.png" ), tr( "Miter" ), QVariant( Qt::MiterJoin ) );
  addItem( QIcon( path + "/join_round.png" ), tr( "Round" ), QVariant( Qt::RoundJoin ) );
}

Qt::PenJoinStyle MvPenJoinStyleComboBox::penJoinStyle() const
{
  return ( Qt::PenJoinStyle ) itemData( currentIndex() ).toInt();
}

void MvPenJoinStyleComboBox::setPenJoinStyle( Qt::PenJoinStyle style )
{
  int idx = findData( QVariant( style ) );
  setCurrentIndex( idx == -1 ? 0 : idx );
}


/////////
// cap

MvPenCapStyleComboBox::MvPenCapStyleComboBox( QWidget* parent )
    : QComboBox( parent )
{
  QString path = ":bitmaps";//QgsApplication::defaultThemePath();
  addItem( QIcon( path + "/cap_square.png" ), tr( "Square" ), QVariant( Qt::SquareCap ) );
  addItem( QIcon( path + "/cap_flat.png" ), tr( "Flat" ), QVariant( Qt::FlatCap ) );
  addItem( QIcon( path + "/cap_round.png" ), tr( "Round" ), QVariant( Qt::RoundCap ) );
}

Qt::PenCapStyle MvPenCapStyleComboBox::penCapStyle() const
{
  return ( Qt::PenCapStyle ) itemData( currentIndex() ).toInt();
}

void MvPenCapStyleComboBox::setPenCapStyle( Qt::PenCapStyle style )
{
  int idx = findData( QVariant( style ) );
  setCurrentIndex( idx == -1 ? 0 : idx );
}

}//namespace marble