/******************************************************************************************************************
 *	@file			TreeViewModel.h
 *	@copyright		2013 Rohde&Schwarz, Munich
 *	@version		$Header: TreeViewModel.h 
 *	5 August 2011 16:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView TreeViewModel.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_TREEVIEWMODEL_H
#define MARBLE_TREEVIEWMODEL_H

#include <QAbstractItemModel>
#include <vector>

#include "GeoDataLineString.h"
#include "MarblePlacemarkModel.h"

#include "Enum.h"
#include "treeviewitem.h"

using namespace GIS;
using namespace Model;

/* FORWARD DECLARATION *********************************************************/
class QObject;
class QVariant;
//class QTimer;
//class TreeNode;

/**
  * A QAbstractItemModel that contains a list of gis object's.
  
  */
namespace Marble
{
class TreeViewModel : public QAbstractItemModel
{
	  Q_OBJECT

public:
    TreeViewModel(QObject *parent = 0);
    ~TreeViewModel();
    void setRootNode(TreeNode *node);
    QModelIndex index(int row, int column,const QModelIndex &parent) const;
    QModelIndex parent(const QModelIndex &child) const;
    int rowCount( const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation,int role) const;
	bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
	Qt::ItemFlags flags ( const QModelIndex & index ) const;
	void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
	//virtual bool hasChildren( const QModelIndex &parent ) const;

public:
	enum TreeViewItemDataRole {
        CoordinateRole = MarblePlacemarkModel::CoordinateRole, // synchronized with MarblePlacemarkModel
        TypeRole = CoordinateRole + 24, // avoid conflict with MarblePlacemarkModel
		DataRole = CoordinateRole + 25,
		InfoRole = CoordinateRole + 26,
		OffXRole = CoordinateRole + 27,
		OffYRole = CoordinateRole + 28,
		FillColorRole = CoordinateRole + 29,
		LineColorRole = CoordinateRole + 30,
		TransRole = CoordinateRole + 31,
		ZValueRole = CoordinateRole + 32,
		ValueRole = CoordinateRole + 33
    };

private: // auxiliary functions

	bool insertRows(int iPosition, int iRows, const QModelIndex& parent);
	bool removeRows(int iPosition, int iRows, const QModelIndex& parent);

	Model::TreeNode* AddRow(QModelIndex& parent=QModelIndex(),GIS::TreeViewItemType type=Overlay);
    TreeNode *nodeFromIndex(const QModelIndex &index) const;
	void clear();
    

public:	
	// adding data to the model
	qint64 AddMapOverlay(QStringList& qsList,GeoDataDocument* pDoc,bool bMap);

	qint64 AddOverlay			(QStringList& qsList,bool bMap);
	qint64 AddLayer				(QStringList& qsList,qint64);
	qint64 AddTakticalLayer		(QStringList& qsList,qint64);
	qint64 AddMapLayer			(QStringList& qsList,qint64,bool);
	qint64 AddGroup				(QStringList& qsList,qint64);

	qint64 AddIcon(QString& qs,long id,qreal lon, qreal lat, long xoffs, long yoffs, long align, long idx);
	qint64 AddMarker(QString& qs,long id,qreal lon, qreal lat, long xoffs, long yoffs, long align);
	qint64 AddPlot(QStringList& qsl,long id,qreal lon, qreal lat, qreal radius, GeoDataLineString& gdl, long flags);
	qint64 AddPlotEx(QStringList& qsl,long id,qreal lon, qreal lat,GeoDataLineString& gdl, long flags);
	qint64 AddAzimuthLine(long id,qreal lon, qreal lat,long azi);
	qint64 MoveAzimuthLine(long id,qreal lon, qreal lat,long azi,GeoDataLineString&);
//	qint64 MoveAzimuthLine(long id,qreal lon, qreal lat,long azi,TCoord * coords,int iCount);

	qint64 AddPolyLine(long id,long color,GeoDataLineString&);
	qint64 AddPolyLineEx(long id,long color,int iGridX,int iGridY,int iSID,GeoDataLineString&);
	qint64 AddPolygon(long id,long color,GeoDataLineString&);
	qint64 AddColorPolygone(QString& qs,long id,long lineCol,long fillCol,qreal lon, qreal lat,int width,GeoDataLineString& gdl);

	qint64 AddRect(long id,double left, double top,double right,double bottom);
	qint64 AddDescription(QString& qs,long id,qreal lon,qreal lat,int xoff,int yoff,int align=0);

	qint64 AddEllipse(QString& qsl,long id,qreal lon, qreal lat,GeoDataLineString& gdl);
	qint64 AddColorEllipse(QString& qsl,long id,long lineCol,long fillCol,qreal lon, qreal lat,GeoDataLineString& gdl);
	qint64 AddArrow(long id,qreal lon, qreal lat,int dir_count1,int dir_count2,long color,GeoDataLineString& gdl);

	qint64 AddSvgObject(long id, QString& key,double lon, double lat, bool file,bool fixed,bool omni,int bts, QPixmap* pPix);
	qint64 AddHotSpot(long id, QString& key,double lon, double lat, bool set, QPixmap* pPix);
	qint64 AddPixmapObject(long id, QString& key,double lon, double lat,int xoffs,int yoffs,int alignment, QPixmap* pPix);

	qint64 AddHyperbola(QString& qsl,long id,qreal lon, qreal lat,GeoDataLineString& gdl);

	qint64 UpdatePolyLine(long id,long color,GeoDataLineString&);

	qint64 DeleteObject(long id, int type);
	qint64 DeleteByName(QStringList& qsList);

	qint64 ModifyObjectText(long id,QString&);

	//MOG objects
	qint64 AddNamedPolyLine(long id,QString& name,qreal lon,qreal lat,long color,GeoDataLineString&);

	//remove map tree entries
	bool RemoveMap();

	TreeNode	*getNodeFromIndex(const QModelIndex &index);
	TreeNode	*getNodeFromIndexId(qint64 &index);
	TreeNode	*getChildNode(TreeNode*);

	QModelIndex  getIndexFromIndexId(qint64 &index);
	void addChild(TreeNode *parent, TreeNode *child);

	//get the icon pixmap
	QPixmap pixmap( QString& key ) const;
	//check pixmap key
	bool checkPixmapKey( QString& key ) const;

	//get line string
	GeoDataLineString* getPoly(const QModelIndex &index);

private slots:
	void slotUpdateMap();

Q_SIGNALS:
	void signalUpdateMap();

private:	
	TreeNode*	m_pRootNode;
//	QTimer*		m_pUpdateTimer;
	QMap<QString, QPixmap> m_pixmapCache;
	QMap<QString, QPixmap> m_pixmapTree;
//	QHash<long,int>		   m_objects;

};




} // namespace Marble

//Q_DECLARE_METATYPE( Marble::TreeViewModel::TreeViewItemType )

#endif
