/******************************************************************************************************************
 *	@file			TreeViewProxyModel.h
 *	@copyright		2013 Rohde&Schwarz, Munich
 *	@version		$Header: TreeViewProxyModel.h   
 *	5 Juli 2011 16:49:35  enzensbe $
 *
 *	@author			Konrad Enzensberger		e.konrad@mpegcode.com
 *	@responsible	KOnrad Enzensberger
 * 
 *	@language		C++ ANSI V3
 *
 *	@description 	This file defines the MapView tree view model proxy.
 *					
 *					
 *****************************************************************************************************************/

#ifndef MARBLE_TREEVIEWPROXYMODEL_H
#define MARBLE_TREEVIEWPROXYMODEL_H

#include <QtGui/QSortFilterProxyModel>

#include "Enum.h"
using namespace GIS;

namespace Marble
{

/**
  * @brief A QAbstractItemModel that contains a list of routing instructions.
  *
  * Can be used in conjunction with RoutingModel to filter out waypoints
  * that don't contain instructions (a list of "important" parts of the route is left)
  *
  * @see RoutingModel
  */
class TreeViewProxyModel : public QSortFilterProxyModel
{
public:
    /** Constructor */
    explicit TreeViewProxyModel( QObject *parent = 0 );

protected:
    /** Reimplemened from QSortFilterProxyModel. Filters waypoints without instructions */
    virtual bool filterAcceptsRow ( int source_row, const QModelIndex &source_parent ) const;

};

} // namespace Marble

#endif
