/*******************************************************************************
* @file			MapViewDimLayer.cpp
* @copyright	2011-2013 Rohde&Schwarz, Munich
* @version		$Header: MapViewDimLayer.h
*
* @author		Konrad Enzensberger		e.konrad@mpegcode.com
* @responsible	KOnrad Enzensberger
* 
* @language		C++ ANSI V3
*
* @history
*
*
********************************************************************************
*******************************************************************************/

/* INCLUDE FILES **************************************************************/

#include "MapViewTSLayer.h"

#include "GeoDataCoordinates.h" 
#include "GeoDataLineString.h"
#include "GeoPainter.h"
#include "GeoSceneDocument.h"
#include "GeoSceneHead.h"
#include "MarblePlacemarkModel.h"
#include "MarbleWidget.h"
#include "MarbleWidgetPopupMenu.h"
#include "TreeViewModel.h"
//#include "gps/PositionTracking.h"
#include "MarbleMap.h"
#include "MarbleModel.h"
#include "MarbleDirs.h"
#include "ViewportParams.h"
//#include "PolygoneSkeleton.h"

#include "GeoDataDocument.h"
#include "GeoDataFolder.h"
#include "GeoDataPlacemark.h"
#include "GeoDataMultiGeometry.h"
#include "GeoDataExtendedData.h"
#include "GeoDataStyle.h"
#include "GeoDataStyleMap.h"
#include "GeoDataLineStyle.h"
#include "GeoDataFeature.h"
#include "GeoWriter.h"

#include <QtCore/QMap>
#include <QtGui/QAbstractProxyModel>
#include <QtGui/QIcon>
#include <QtGui/QItemSelectionModel>
#include <QtGui/QKeyEvent>
#include <QtGui/QMenu>
#include <QtGui/QMouseEvent>
#include <QtGui/QPixmap>
#include <QtGui/QBitmap>
#include <QString>
//#include <QtGui/QComboBox>
#include <QDebug>

namespace Marble
{

class MapViewTSLayerPrivate
{
    template<class T>
    struct PaintRegion {
        T index;
        QRegion region;

        PaintRegion( const T &index_, const QRegion &region_ ) :
                index( index_ ), region( region_ )
        {
            // nothing to do
        }
    };

    typedef PaintRegion<QModelIndex> ModelRegion;
    typedef PaintRegion<int> RequestRegion;

public:
    MapViewTSLayer *q;

	qint64 m_indexID;
	qint64 m_infoID;

    QList<ModelRegion>		m_placemarks;	//GeoObjects like Icons, ...
    QList<RequestRegion>	m_regions;		//interpolation point from selected polygone
	QList<ModelRegion>		m_routeRegion;	//region from polygones
    QList<ModelRegion>		m_selpoint;		//interpolation points of polygones
    QRegion					m_layerRegion; 
	QRegion					m_polyRegion;

	QMap<GIS::SelectType,QString>	m_selTypePixmaps;
	QMap<int,QModelIndex>			m_multiSelect;
	QMap<int,QModelIndex>			m_multiItemSelect;

 //   QAbstractProxyModel *m_proxyModel;

	//the index from selected poly via point
    int m_movingIndex;
	//poly (1) or placemark (2) moved
	int m_movingID;
	//the index of selected poly via point by right mouse click, used for delete via point
    int m_activeMenuIndex;
	//selected item is a poly 
	bool m_bIsPoly;

	//style url for multi geometry already set
	bool m_bMultiGeoStyle;

	bool m_viewportChanged; //recalc objects regions if true
	bool m_bInputEnabled;	//if true, do not take care obout mouse move over objects
	bool m_contentChanged;  //draw layer objects regions if true

    MarbleWidget	*m_marbleWidget;
	QWidget			*m_parentWidget;

    QPixmap m_targetPixmap;

    QRect m_dirtyRect;
	QRect m_annoRect;
	QRect m_legendRect;

    QPoint m_dropStopOver;
    QPoint m_dragStopOver;
	QPoint m_posInfo; 
	QPoint m_mousePos; 

    int m_dragStopOverRightIndex;

    bool m_pointSelection;

    TreeViewModel			*m_treeviewModel;
	TreeNode				*m_pNode;
	QModelIndex				m_selIdx;
	//QModelIndex				m_infoIdx;
	bool					m_bObjId;
	bool					m_update;
	bool					m_bGeoProjected;
	int						m_iLegendAlign;

    MarblePlacemarkModel	*m_placemarkModel;
    QItemSelectionModel		*m_selectionModel;

    bool m_layerDirty;

    QSize m_pixmapSize;

    MarbleWidgetPopupMenu	*m_contextMenu;
	QMenu					*m_selectionMenu;
	bool					m_bMultiSelect;

    QAction *m_removeViaPointAction;
	QAction *m_insertViaPointAction;
	QAction *m_deleteObjectAction;

	//holds the menu items sent by TcpIp
	QActionGroup* pActionGroup;

	//layer properties
	 // Static members
    QFont  m_layerFont;
	QColor	m_lineColor;
	QColor	m_fillColor;
	QColor	m_fontColor;
	QColor	m_fontBackColor;
	int		m_lineWidth;
	Qt::PenStyle m_penStyle;
	int		m_markerSize;
	bool	m_bEdit;
	bool	m_bDelete;
	bool	m_bMoveObj;
	bool	m_bDeleteObj;

	int		m_iDimmer;

	double m_minLon;
	double m_maxLon;
	double m_minLat;
	double m_maxLat;

	qreal m_rotation;

//    QComboBox *m_alternativeRoutesView;

//    AlternativeRoutesModel* m_alternativeRoutesModel;

    /** Constructor */
    explicit MapViewTSLayerPrivate( MapViewTSLayer *parent, MarbleWidget *widget, qint64 index );

    /** Show a context menu at the specified position */
    void showContextMenu( const QPoint &position );

	/** Show a multi select menu at the specified position */
    void showMultiSelectMenu( const QPoint &position );

	/** Show a multi select menu at the specified position - additiona add layer context menu */
    void showMultiSelectContextMenu( const QPoint &position );

    /** Update the cached drag position. Use an empty point to clear it. */
    void storeDragPosition( const QPoint &position );

	 /** Update placemark list */
    void removePlacemark(qint64);

    // The following methods are mostly only called at one place in the code, but often
    // Inlined to avoid the function call overhead. Having functions here is just to
    // keep the code clean

    /**
      * Returns the start or destination position if Ctrl key is among the
      * provided modifiers, the cached insert position otherwise
      */
    inline int viaInsertPosition( Qt::KeyboardModifiers modifiers ) const;

    /** Paint icons for each placemark in the placemark model */
    inline void renderPlacemarks( GeoPainter *painter );

    /** render all geo objects */
    inline void renderLayer( GeoPainter *painter );

	 /** render all non geo objcts */
    inline void renderLegendLayer( GeoPainter *painter );

    /** Paint turn instruction for selected items */
    inline void renderAnnotations( GeoPainter *painter );

	/** Paint icons for trip points etc */
    inline void renderRequest( GeoPainter *painter );

    /** Insert via points or emit position signal, if appropriate */
    inline bool handleMouseButtonRelease( QMouseEvent *e );

    /** Select route instructions points, start dragging trip points */
    inline bool handleMouseButtonPress( QMouseEvent *e );

    /** Dragging trip points, route polygon hovering */
    inline bool handleMouseMove( QMouseEvent *e );

	 /** mouse dbl click */
    inline bool handleMouseDblClick( QMouseEvent *e );

    /** Escape to stop selecting points */
    inline bool handleKeyEvent( QKeyEvent *e );

    /** True if the given point (screen coordinates) is among the route instruction points */
    inline bool isInfoPoint( const QPoint &point );

    /** True if the given point (screen coordinates) is above an alternative route */
    //inline bool isAlternativeRoutePoint( const QPoint &point );

    /** Paint the stopover indicator pixmap at the given position. Also repaints the old position */
    inline void paintStopOver( QRect position );

    /** Removes the stopover indicator pixmap. Also repaints its old position */
    inline void clearStopOver();

	/** synch tree model with layer items */
	inline void synchronizeWithTree(const QModelIndex& idx);

	/** synch extern with tree model and layer items 
	*   select = false: item selected by tree
	*   select = true : item selection command from extern
	*/
	inline void synchronizeWithExtern(const QModelIndex& idx, bool ext);

	/** rotate a polygon */
	bool rotatePoly(qreal angle);
	/** resize a polygon */
	bool resizePoly(qreal angle);
	/** move all group members */
	bool moveObjects(GeoDataCoordinates& moved,TreeNode* pNode,const QPoint&);

	/** add menue entries sent by TcpIp */
	inline void addContextMenu(QString& qsMenu,int iVal);
	/** delete menue entries sent by TcpIp */
	inline void delContextMenu(void);

	/** export layer objects to kml */
	inline void export2kml(QString&);
	/** export layer objects to kml */
	inline void exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder);
	/** add icon to kml */
	inline void addIconToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add label to kml */
	inline void addLabelToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add rect to kml */
	inline void addRectToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add polygone to kml */
	inline void addPolygoneToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add polyline to kml */
	inline void addPolyToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add ellipse to kml */
	inline void addEllipseToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add azimuth to kml */
	inline void addAzimuthToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add plot object to kml */
	inline void addPlotToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);
	/** add plotEx object to kml */
	inline void addPlotExToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder);

	/** add MultiGeometry object to kml */
	inline void addMultiGeometryToKml(TreeNode* pGroup,GeoDataDocument* document,GeoDataFolder* folder);

	/** add Icon object to MultiGeometry */
	void addIconToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add Label object to MultiGeometry */
	void addLabelToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add Rect object to MultiGeometry */
	void addRectToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add Polyline object to MultiGeometry */
	void addPolyToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add polygon object to MultiGeometry */
	void addPolygonToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add ellipse object to MultiGeometry */
	void addEllipseToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add azimuth object to MultiGeometry */
	void addAzimuthToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add Plot object to MultiGeometry */
	void addPlotToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add PlotEx object to MultiGeometry */
	void addPlotExToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);
	/** add Arrow object to MultiGeometry */
	void addArrowToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG);


	/** calc new ellipse coordinates */
	inline bool calcEllipseCoords(qreal, qreal, qreal, qreal,GeoDataLineString&);
	/** calc new plot coordinates */
	inline bool calcPlotCoords(const QModelIndex& idx,const GeoDataCoordinates&,const GeoDataCoordinates&);

	/** helper methode to calculate arrow lines **/
	inline void DrawArrow( GeoPainter *painter, QPoint& pt, double angle, short dir_count,QColor&);
	inline bool CycToPoint(QPoint& center,QPoint& ret, double angle, double relativeDistanceToCenter);
	inline void logging(QString& str1,long val1,long val2,long val3,long val4);

	/** detect all intersection objects */
	int getIntersectionObjects(QModelIndex& idx,QPoint pt,bool flag,int pos);
	/** detect all intersection objects from region*/
	int getIntersectionRegionObjects(QModelIndex& idx,const QRect&,bool flag,int pos);
};

MapViewTSLayerPrivate::MapViewTSLayerPrivate( MapViewTSLayer *parent, MarbleWidget *widget, qint64 index ) :
        q( parent ), m_movingIndex( -1 ),m_movingID(0), m_marbleWidget( widget ),m_bObjId(false),
        m_targetPixmap( ":/bitmaps/routing_pick.png" ), m_dragStopOverRightIndex( -1 ),
        m_pointSelection( false ), m_treeviewModel( 0 ),m_pNode(0), m_placemarkModel( 0 ), m_selectionModel( 0 ),
        m_layerDirty( false ), m_pixmapSize( 22, 22 ), m_activeMenuIndex( -1 ),m_bIsPoly(false),m_bGeoProjected(true),m_iLegendAlign(3),
		m_indexID(index), m_viewportChanged(true),m_contentChanged(true),m_parentWidget(NULL),pActionGroup(NULL),
		m_infoID(0),m_selectionMenu(NULL),m_bMultiGeoStyle(false)
{
    m_contextMenu = new MarbleWidgetPopupMenu( m_marbleWidget, m_marbleWidget->model() );
    m_removeViaPointAction = new QAction( QObject::tr( "&Remove" ), q );
	m_insertViaPointAction = new QAction( QObject::tr( "&Insert" ), q );
	m_deleteObjectAction = new QAction( QObject::tr( "&Delete" ), q );
    QObject::connect( m_removeViaPointAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
	QObject::connect( m_insertViaPointAction, SIGNAL( triggered() ), q, SLOT( insertViaPoint() ) );
	QObject::connect( m_deleteObjectAction, SIGNAL( triggered() ), q, SLOT( deleteObject() ) );
    m_contextMenu->addAction( Qt::RightButton, m_removeViaPointAction );
	m_contextMenu->addAction( Qt::RightButton, m_insertViaPointAction );
	m_contextMenu->addAction( Qt::RightButton, m_deleteObjectAction );
	//m_contextMenu->getRmbMenu()->addSeparator();

	m_bMultiSelect=false;
	m_bInputEnabled=false;

	m_rotation=0;

/*    QAction *exportAction = new QAction( QObject::tr( "&Export route..." ), q );
    QObject::connect( exportAction, SIGNAL( triggered() ), q, SIGNAL( exportRequested() ) );
    m_contextMenu->addAction( Qt::RightButton, exportAction );*/

	m_lineColor = QColor( 0,0,0);//black
	m_fillColor = QColor( 255,0, 0, 0 );//red - transparent
	m_fontColor = QColor( 0,0, 0);//black
	m_fontBackColor = QColor( 255,255,255);//white
	m_lineWidth=2; 
	m_penStyle=Qt::SolidLine;
	m_markerSize=32;
	m_bEdit=true;
	m_bDelete=true;
	m_bMoveObj=true;
	m_bDeleteObj=true;

	m_iDimmer=0;

	m_minLon = 399.0;;
	m_maxLon = -399.0;
	m_minLat = 399.0;
	m_maxLat = -399.0;

	m_legendRect.setLeft(1000);
	m_legendRect.setTop(1000);
	m_legendRect.setRight(0);
	m_legendRect.setBottom(0);

	m_layerFont = QFont("Sans Serif");

}

void  MapViewTSLayerPrivate::addContextMenu(QString& qsMenu,int iVal)
{
	if(pActionGroup==NULL)
	{
		pActionGroup = new QActionGroup(m_marbleWidget);
		if(pActionGroup==NULL)
			return;
		QObject::connect( pActionGroup, SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		m_contextMenu->addAction(Qt::RightButton,m_contextMenu->getRmbMenu()->addSeparator());
	}

	if(qsMenu != "MapView")
	{
		QAction* lcmAction = new QAction(qsMenu, q );
		pActionGroup->addAction(lcmAction);
		lcmAction->setData(iVal);
		lcmAction->setEnabled(iVal<100);//menu id > 100 is disabled
		//QObject::connect( m_lcmAction, SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		m_contextMenu->addAction( Qt::RightButton, lcmAction );
	}
	else //only menu set by TcpIp is used
	{
		if(m_contextMenu!=NULL)
		{
			QMenu* qMenu = m_contextMenu->getRmbMenu();
			if(qMenu!=NULL)
			{
				qMenu->removeAction(m_removeViaPointAction);
				qMenu->removeAction(m_insertViaPointAction);
				qMenu->removeAction(m_deleteObjectAction);
				/*foreach( QAction *action, qMenu->actions() ) 
				{
					QVariant qVar = action->data();
					int iVal = qVar.toInt();
					if ( action->text() == QObject::tr( "&Remove" ) ) 
					{
						qMenu->removeAction( action );
					}
				}*/
			}
		}
	}

}

void  MapViewTSLayerPrivate::delContextMenu()
{
	if(pActionGroup!=NULL)
	{
		if(m_contextMenu!=NULL)
		{
			QMenu* qMenu = m_contextMenu->getRmbMenu();
			if(qMenu!=NULL)
			{
				foreach( QAction *action, qMenu->actions() ) 
				{
					QVariant qVar = action->data();
					int iVal = qVar.toInt();
					if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) ) 
					{
						qMenu->removeAction( action );
					}
				}
			}
		}
		QObject::disconnect(pActionGroup,SIGNAL( triggered(QAction*) ), q, SLOT( slotLayerContextMenu(QAction*) ) );
		delete pActionGroup;
		pActionGroup=NULL;
	}
}

void MapViewTSLayerPrivate::showContextMenu( const QPoint &pos )
{
	m_bIsPoly=false;
    m_contextMenu->showRmbMenu( pos.x(), pos.y() );
}

void MapViewTSLayerPrivate::showMultiSelectContextMenu( const QPoint &position )
{
	int iSize = m_multiSelect.size();
	if(iSize<2)
		return;

	QAction* lcmAction = NULL;
	QModelIndex idx;
	TreeNode* pNode=NULL;
	QString qsMenu;
	QMenu* pMenu=NULL;

	if(m_selectionMenu!=NULL)
	{
		m_selectionMenu->clear();
		delete m_selectionMenu;
		m_selectionMenu=NULL;
	}

	m_selectionMenu = new QMenu(m_marbleWidget);//MarbleWidgetPopupMenu( m_marbleWidget, m_marbleWidget->model() );

	if(m_selectionMenu!=NULL)
	{
		QObject::connect( m_selectionMenu, SIGNAL( hovered(QAction*) ), q, SLOT( slotLayerContextMenuHovered(QAction*) ) );
		m_bMultiSelect=true;
		for(int i=0;i<iSize;i++)
		{
			idx = m_multiSelect.value(i);
			if(idx.isValid())
				pNode = m_treeviewModel->getNodeFromIndex(idx);
			if(pNode!=NULL)
			{
				//qsMenu = pNode->parent->m_qsDesc;
				//lcmAction = new QAction(qsMenu, q );
				//QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
				if(pNode->ItemType == Icon)
				{
					qsMenu = pNode->parent->m_qsDesc;
					if(qsMenu.isEmpty())
						qsMenu = pNode->parent->m_qsName;

					//lcmAction = new QAction(qsMenu, q );
					//QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
					QPixmap qPix = m_treeviewModel->pixmap(pNode->m_qsImage);
					if ( !qPix.isNull())
					{
						QIcon qIco = QIcon( qPix );
						//lcmAction->setIcon(qIco);
						//m_selectionMenu->addAction(lcmAction );
						pMenu = m_selectionMenu->addMenu(qIco,qsMenu);
						if(pMenu!= NULL && m_contextMenu!=NULL)
						{
							QAction* selectAction = new QAction(QObject::tr("Select"), q );
						//	 m_removeViaPointAction = new QAction( QObject::tr( "&Remove" ), q );
							QObject::connect(selectAction, SIGNAL( triggered() ), q, SLOT( slotSelectContextMenu() ) );
							pMenu->addAction(selectAction);

							pMenu->addSeparator();
							QString sT("%1");
							sT = sT.arg(i);
							pMenu->setWhatsThis(sT);
							if(pActionGroup!=NULL)
							{
								if(m_contextMenu!=NULL)
								{
									QMenu* qMenu = m_contextMenu->getRmbMenu();
									if(qMenu!=NULL)
									{
										foreach( QAction *action, qMenu->actions() ) 
										{
											//pMenu->addMenu(m_contextMenu->getRmbMenu());
											if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" ))  )
											{
											//	QString sT("%1");
											//	sT = sT.arg(i);
											//	action->setToolTip(sT);
												pMenu->addAction(action);
											}
										}//foreach
									}//if qMenu
								}//if context menu
							}//if action grop
						}//if pMenu
					}//if qPix
					else
					{
						pMenu = m_selectionMenu->addMenu(qsMenu);
						if(pMenu!= NULL && m_contextMenu!=NULL)
						{
							if(pActionGroup!=NULL)
							{
								if(m_contextMenu!=NULL)
								{
									QMenu* qMenu = m_contextMenu->getRmbMenu();
									if(qMenu!=NULL)
									{
										foreach( QAction *action, qMenu->actions() ) 
										{
											//pMenu->addMenu(m_contextMenu->getRmbMenu());
											if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) )
												pMenu->addAction(action);
										}
									}//qMenu
								}//context menu
							}//pActionGroup
						}//if pMenu

							
					}//else
					//m_selectionMenu->addAction(lcmAction );
				}
				else if(pNode->ItemType == Marker && pNode->MarkerType == Circle)
				{
					QString qsPix = m_selTypePixmaps[ GIS::Unknown ];
					QPixmap qPix(qsPix);
					qsMenu = pNode->parent->m_qsName;
				//	lcmAction = new QAction(qsMenu, q );
				//	QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
					if ( !qPix.isNull())
					{
						QIcon qIco = QIcon( qPix );
					//	lcmAction->setIcon(qIco);
						pMenu = m_selectionMenu->addMenu(qIco,qsMenu);
						if(pMenu!= NULL && m_contextMenu!=NULL)
						{
							if(pActionGroup!=NULL)
							{
								if(m_contextMenu!=NULL)
								{
									QMenu* qMenu = m_contextMenu->getRmbMenu();
									if(qMenu!=NULL)
									{
										foreach( QAction *action, qMenu->actions() ) 
										{
											//pMenu->addMenu(m_contextMenu->getRmbMenu());
											if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) )
												pMenu->addAction(action);
										}
									}//qMenu
								}//context menu
							}//pActionGroup
						}//if pMenu
					}
					else
					{
						pMenu = m_selectionMenu->addMenu(qsMenu);
						if(pMenu!= NULL && m_contextMenu!=NULL)
						{
							if(pActionGroup!=NULL)
							{
								if(m_contextMenu!=NULL)
								{
									QMenu* qMenu = m_contextMenu->getRmbMenu();
									if(qMenu!=NULL)
									{
										foreach( QAction *action, qMenu->actions() ) 
										{
											//pMenu->addMenu(m_contextMenu->getRmbMenu());
											if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) )
												pMenu->addAction(action);
										}
									}//qMenu
								}//context menu
							}//pActionGroup
						}//if pMenu
					}
				}
				else 
				{
					qsMenu = pNode->parent->m_qsDesc;
					//lcmAction = new QAction(qsMenu, q );
					//QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
					//m_selectionMenu->addAction(lcmAction );
					pMenu = m_selectionMenu->addMenu(qsMenu);
					if(pMenu!= NULL && m_contextMenu!=NULL)
					{
						if(pActionGroup!=NULL)
						{
							if(m_contextMenu!=NULL)
							{
								QMenu* qMenu = m_contextMenu->getRmbMenu();
								if(qMenu!=NULL)
								{
									foreach( QAction *action, qMenu->actions() ) 
									{
										//pMenu->addMenu(m_contextMenu->getRmbMenu());
										if ( (action->text() != QObject::tr( "&Remove" )) && (action->text() != QObject::tr( "&Insert" )) && (action->text() != QObject::tr( "&Delete" )) )
											pMenu->addAction(action);
									}
								}//qMenu
							}//context menu
						}//pActionGroup
					}//if pMenu
				}
			}
		}

		m_selectionMenu->popup( m_marbleWidget->mapToGlobal( position ) );
		//m_selectionMenu->showRmbMenu( position.x(), position.y() );
	}

}

void MapViewTSLayerPrivate::showMultiSelectMenu( const QPoint &position )
{
	//show context menu along with layer menu ?
	if(m_contextMenu!=NULL && pActionGroup!=NULL)
	{
		showMultiSelectContextMenu(position);
		return;
	}

	int iSize = m_multiSelect.size();
	if(iSize<2)
		return;

	QAction* lcmAction = NULL;
	QModelIndex idx;
	TreeNode* pNode=NULL;
	QString qsMenu;

	m_selectionMenu = new QMenu(m_marbleWidget);//MarbleWidgetPopupMenu( m_marbleWidget, m_marbleWidget->model() );

	if(m_selectionMenu!=NULL)
	{
		QObject::connect(m_selectionMenu, SIGNAL( triggered(QAction*) ), q, SLOT( slotSelectActionMenu(QAction*) ) );
		for(int i=0;i<iSize;i++)
		{
			idx = m_multiSelect.value(i);
			if(idx.isValid())
				pNode = m_treeviewModel->getNodeFromIndex(idx);
			if(pNode!=NULL)
			{
				//qsMenu = pNode->parent->m_qsDesc;
				//lcmAction = new QAction(qsMenu, q );
				//QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
				if(pNode->ItemType == Icon)
				{
					qsMenu = pNode->parent->m_qsDesc;
					if(qsMenu.isEmpty())
						qsMenu = pNode->parent->m_qsName;

					lcmAction = new QAction(qsMenu, q );
					lcmAction->setData(i);
					//QObject::connect( lcmAction, SIGNAL( triggered() ), q, SLOT( removeViaPoint() ) );
					QPixmap qPix = m_treeviewModel->pixmap(pNode->m_qsImage);
					if ( !qPix.isNull())
					{
						QIcon qIco = QIcon( qPix );
						lcmAction->setIcon(qIco);
					//	m_selectionMenu->addAction(lcmAction );
					}
					m_selectionMenu->addAction(lcmAction );
				}
				else if(pNode->ItemType == Marker && pNode->MarkerType == Circle)
				{
					QString qsPix = m_selTypePixmaps[ GIS::Unknown ];
					QPixmap qPix(qsPix);
					qsMenu = pNode->parent->m_qsName;
					lcmAction = new QAction(qsMenu, q );
					lcmAction->setData(i);
					
					if ( !qPix.isNull())
					{
						QIcon qIco = QIcon( qPix );
						lcmAction->setIcon(qIco);
					}
					m_selectionMenu->addAction(lcmAction );
				}
				else 
				{
					qsMenu = pNode->parent->m_qsDesc;
					if(qsMenu.isEmpty())
						qsMenu = pNode->parent->m_qsName;
					lcmAction = new QAction(qsMenu, q );
					lcmAction->setData(i);
					m_selectionMenu->addAction(lcmAction );
				}
			}
		}

		m_selectionMenu->popup( m_marbleWidget->mapToGlobal( position ) );
		//m_selectionMenu->showRmbMenu( position.x(), position.y() );
	}
}

void  MapViewTSLayerPrivate::synchronizeWithTree(const QModelIndex& idx)
{
	if ( !m_selectionModel || m_selectionModel->selection().isEmpty() ) 
	{
        // nothing to do
		return;
    }

	QRegion rg;

	if(m_routeRegion.size()>0 )
	{
		foreach( const ModelRegion &region, m_routeRegion ) 
		{
			if ( region.index == idx &&  m_selectionModel->isSelected(idx) )
			{
				m_selIdx = idx;
				rg = region.region;
				m_marbleWidget->update(rg);
				//qDebug() << "SynchronizeWith: " << m_selIdx.internalPointer();
				break;
			}
		}
	}
	else if(m_placemarks.size()>0)
	{
		foreach( const ModelRegion &region, m_placemarks ) 
		{
			if ( region.index == idx && m_selectionModel->isSelected(idx) )
			{
				m_selIdx = idx;
				rg = region.region;
				m_marbleWidget->update(rg);
				//qDebug() << "SynchronizeWith: " << m_selIdx.internalPointer();
				break;
			}
		}
	}
			
}

void  MapViewTSLayerPrivate::synchronizeWithExtern(const QModelIndex& idx,bool ext)
{
	if ( !m_selectionModel ) 
	{
        // nothing to do
		return;
    }

	//first being optimistic
	if(idx.isValid())
	{
		m_selIdx = idx;
		QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
		m_selectionModel->select( m_selIdx, command );
		return;
	}

	QRegion rg;
	bool set=false;

	if(m_placemarks.size()>0)
	{
		foreach( const ModelRegion &regionp, m_placemarks ) 
		{
			if ( regionp.index == idx )
			{
				m_selIdx = idx;
				rg = regionp.region;
			//	m_marbleWidget->update(rg);
			//	qDebug() << "SynchronizeWith: " << m_selIdx.internalPointer();
				set=true;
				break;
			}
		}
	}
	else if(m_routeRegion.size()>0 )
	{
		foreach( const ModelRegion &region, m_routeRegion ) 
		{
			if ( region.index == idx )
			{
				m_selIdx = idx;
				rg = region.region;
			//	m_marbleWidget->update(rg);
			//	qDebug() << "SynchronizeWith: " << m_selIdx.internalPointer();
				set=true;
				break;
			}
		}
	}

	if(ext&&set)
	{
		QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
		if (0)// m_selectionModel->isSelected( region.index ) ) 
		{
				command = QItemSelectionModel::Clear;
		}
		m_selectionModel->select( m_selIdx, command );
		m_marbleWidget->update(rg);
		//m_marbleWidget->repaint();
	}

}

int MapViewTSLayerPrivate::viaInsertPosition( Qt::KeyboardModifiers modifiers ) const
{
  /*  if ( modifiers & Qt::ControlModifier ) {
        bool leftHand = m_polySkeleton->size() / 2 >= m_dragStopOverRightIndex;
        if ( leftHand && m_polySkeleton->size() > 2 ) {
            return 0;
        } else {
            return m_polySkeleton->size();
        }
    } else {
        return m_dragStopOverRightIndex;
    }*/

	return 0;
}

void MapViewTSLayerPrivate::renderPlacemarks( GeoPainter *painter )
{
    m_placemarks.clear();
    painter->setPen( QColor( Qt::black ) );
    for ( int i = 0; i < m_placemarkModel->rowCount(); ++i ) {
        QModelIndex index = m_placemarkModel->index( i, 0 );
        QVariant data = index.data( MarblePlacemarkModel::CoordinateRole );
        if ( index.isValid() && !data.isNull() ) {
            GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( data );

            QPixmap pixmap = qVariantValue<QPixmap>( index.data( Qt::DecorationRole ) );
            if ( !pixmap.isNull() && m_selectionModel->isSelected( index ) ) {
                QIcon selected = QIcon( pixmap );
                QPixmap result = selected.pixmap( m_pixmapSize, QIcon::Selected, QIcon::On );
                painter->drawPixmap( pos, result );
            } else {
                painter->drawPixmap( pos, pixmap );
            }

            QRegion region = painter->regionFromRect( pos, m_targetPixmap.width(), m_targetPixmap.height() );
            m_placemarks.push_back( ModelRegion( index, region ) );
        }
    }
}
/*
void RoutingLayerPrivate::renderAlternativeRoutes( GeoPainter *painter )
{
    m_alternativeRouteRegions.clear();

    QPen grayPen( QColor::fromRgb( 136, 138, 133, 200 ) ); // gray, oxygen palette
    grayPen.setWidth( 5 );
    painter->setPen( grayPen );

    for ( int i=0; i<m_alternativeRoutesModel->rowCount(); ++i ) {
        GeoDataDocument* route = m_alternativeRoutesModel->route( i );
        if ( route ) {
            GeoDataLineString* points = AlternativeRoutesModel::waypoints( route );
            painter->drawPolyline( *points );
            QRegion region = painter->regionFromPolyline( *points, 8 );
            m_alternativeRouteRegions.push_back( SkeletonRegion( i, region ) );
        }
    }
}*/

void MapViewTSLayerPrivate::renderLayer( GeoPainter *painter )
{
	if(!m_update)return;
	//if(!m_contentChanged)return;

   // GeoDataLineString waypoints;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);

	//QModelIndex idxOverlay = m_treeviewModel->parent(idxParent);
	QVariant qVar; 
	//= m_treeviewModel->data(idxOverlay,Qt::CheckStateRole);
	//if(!qVar.toBool())return;
	
	//qVar = m_treeviewModel->data(idxParent,Qt::CheckStateRole);
	//if(!qVar.toBool())return;

	/*if(m_minLon != 399.0)
	{
		GeoDataLatLonBox ellipseBox( m_maxLat,m_minLat,m_maxLon,m_minLon, GeoDataCoordinates::Degree );
		if(!m_marbleWidget->viewport()->viewLatLonAltBox().intersects(ellipseBox) )
			return;
	}*/

	GeoDataLatLonAltBox viewBox = m_marbleWidget->viewport()->viewLatLonAltBox();
		
	if(m_viewportChanged)
	{
		m_placemarks.clear();
		m_routeRegion.clear();
		m_selpoint.clear();
	}
	//m_polySkeleton->clear();

//	qVar = m_treeviewModel->data(idxParent,Qt::UserRole);
	long colorRef = -1;// (long)qVar.toLongLong();
	int red      = 0;//colorRef & 255;
	int green    = 0;//( colorRef>>8 ) & 255;
	int blue     = 0;//( colorRef>>16 ) & 255;
	QColor color;// = QColor( red, green, blue );*/

//	QFont defaultFont = m_marbleWidget->defaultFont();
/*	defaultFont.setWeight( 75 );
	const qreal s_labelOutlineWidth = 2.5;
	int fontascent = QFontMetrics( defaultFont ).ascent();
    const QPointF  baseline( s_labelOutlineWidth / 2.0, fontascent );
	QPainterPath   outlinepath;*/

//	QString qN = defaultFont.rawName();
	painter->setFont(m_layerFont);

	//TreeNode* pNode = m_treeviewModel->getNodeFromIndex(idxParent);
	//if(pNode!=NULL && pNode->ItemType == Layer)
	//	pNode->m_bUpdate=false;

	QRegion region;

//	painter->setPen(color);// QColor( 99, 198, 99, 255 ) );
//	painter->setBrush( QColor( 99, 198, 99, 80 ) );
	QPen layerPen(m_lineColor);
    layerPen.setWidth( m_lineWidth );
	layerPen.setStyle(m_penStyle);
	painter->setPen(layerPen);
	painter->setBrush(m_fillColor);

	//group item count
	int iRowCount = m_treeviewModel->rowCount(idxParent);
	int iReg=0;

//	qDebug() << "RenderLayer: " << idxParent.internalId() << iRowCount; 

    for ( int i = 0; i < iRowCount; ++i ) 
	{

        QModelIndex index = m_treeviewModel->index( i, 0,idxParent );

		if ( index.isValid() )
		{
			qVar = m_treeviewModel->data(index,Qt::CheckStateRole);
			if(qVar.toBool())
			{
				//group child count
				int iChildCount = m_treeviewModel->rowCount(index);

				//QVariant qVar;

				GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
				//GIS::TreeViewItemType type = qVariantValue<Model::GIS::TreeViewItemType>( index.data( TreeViewModel::TypeRole ) );
				//int type = (int)index.data( TreeViewModel::TypeRole );
				GIS::TreeViewItemType type = (GIS::TreeViewItemType) qVariantValue<unsigned int>( index.data( TreeViewModel::TypeRole ) );
				if ( type == GIS::Group ) 
				{
					
					if(iChildCount>0)//m_treeviewModel->rowCount(index)>0)
					{
						for ( int ic = 0; ic < iChildCount; ++ic ) 
						{
							QModelIndex idxChild = m_treeviewModel->index( ic, 0,index );
							if(idxChild.isValid())
							{
								qVar = m_treeviewModel->data(idxChild,Qt::CheckStateRole);
								GIS::TreeViewItemType cType = (GIS::TreeViewItemType) qVariantValue<unsigned int>( idxChild.data( TreeViewModel::TypeRole ) );
								
								//check intersection
								if(!viewBox.contains(pos) )
								{
									if ( cType == GIS::Icon || cType == GIS::SvgBts || cType == GIS::Text || 
											cType == GIS::Desc  || cType == GIS::Plot || cType == GIS::PlotEx)
										qVar=false;
								}

								if(qVar.toBool())
								{
									
									if ( cType == GIS::Icon )
									{
							//			qDebug() << "RenderIcon: " << idxChild.internalId() << ic; 
										QPixmap qPix;
										qVar = m_treeviewModel->data(idxChild,Qt::UserRole);
										QString qsImage = qVar.toString();
										if(!qsImage.isEmpty())
										{
											bool bMS=false;
											if(m_multiSelect.size()>1)
											{
												qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
												bMS = qVar.toBool();
											}

											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
											int iXoff = qVar.toInt();
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
											int iYoff = qVar.toInt();

											if(iXoff != 0 || iYoff != 0)
											{
												qreal xpos=0;
												qreal ypos=0;
												m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),xpos,ypos);

												if(1)//iXoff!=0)
													iXoff = (int)(xpos+0.5) + iXoff;
												if(1)//iYoff!=0)
													iYoff = (int)(ypos+0.5) + iYoff;

												m_marbleWidget->geoCoordinates(iXoff,iYoff,xpos,ypos);
												pos.set(xpos,ypos,0,GeoDataCoordinates::Degree);
											}

											qPix = m_treeviewModel->pixmap(qsImage);
											if ( !qPix.isNull() && m_selectionModel->isSelected( idxChild ) ) 
											{
												m_pixmapSize.setHeight(qPix.height());
												m_pixmapSize.setWidth(qPix.width());
										//		QIcon selected = QIcon( qPix );
												//color = QColor( 127,255, 127 );
												//const QBitmap qbm  = qPix.createMaskFromColor(color);
												//QPixmap result //(selected.pixmap( m_pixmapSize, QIcon::Selected, QIcon::On );
												//qPix.setMask(qbm);
												//qPix.fill(Qt::transparent);
												QImage qImg = qPix.toImage();
												qImg.invertPixels();
												
												if(m_viewportChanged && !bMS)
												{
													region = painter->regionFromRect( pos,qPix.width(),qPix.height() );
													if(!region.isEmpty())
													{
														if(m_bEdit || m_bMoveObj)
															m_placemarks.push_front( ModelRegion(idxChild, region ) );
													}
													//painter->drawPixmap(pos,qImg); 
													
											/*		QPixmap tempPixmap = qPix;
													//QPainter painter;
													painter->begin(&tempPixmap);
													painter->fillRect(qPix.rect(), QColor(127, 127, 127, 127));
													painter->end();*/
												}
												painter->drawImage(pos,qImg);
												if(!bMS && m_multiSelect.size()>1)
												{
													//QRect rect(iXoff-qPix.width()/2,iYoff-qPix.height()/2,qPix.width(),qPix.height());
													color = QColor(0,255,255,100);
													painter->setBrush(color);
													if(qPix.width()>=24 && qPix.height()>=24)
														painter->drawEllipse(pos,qPix.width(),qPix.height());
													else painter->drawEllipse(pos,24,24);
													painter->setBrush(m_fillColor);
												}
											}
											else if(!qPix.isNull())
											{
												m_pixmapSize.setHeight(qPix.height());
												m_pixmapSize.setWidth(qPix.width());

												if(m_viewportChanged && !bMS)
												{
													region = painter->regionFromRect( pos,qPix.width(),qPix.height() );
													if(!region.isEmpty())
													{
														if(m_bEdit || m_bMoveObj)
															m_placemarks.push_front( ModelRegion(idxChild, region ) );
													}
												}
												painter->drawPixmap(pos,qPix); 
												if(bMS)
												{
													QRect rect(iXoff-qPix.width()/2,iYoff-qPix.height()/2,qPix.width(),qPix.height());
													color = QColor(210,210,210,100);
													painter->setBrush(color);
													if(qPix.width()<24 || qPix.height()<24)
													{
														rect.setLeft(iXoff-12);
														rect.setTop(iYoff-12);
														rect.setHeight(24);
														rect.setWidth(24);
													}
													painter->drawEllipse(rect);
													painter->setBrush(m_fillColor);
												}
											}

											if(iXoff != 0 || iYoff != 0)
												pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );

										/*	if ( m_viewportChanged ) {
												//region = painter->regionFromEllipse( pos, 28, 28 );
												region = painter->regionFromRect( pos,qPix.width(),qPix.height() );
												m_placemarks.push_front( ModelRegion(idxChild, region ) );
											}*/
										}
									}//if Icon
									else if ( cType == GIS::SvgBts )
									{
							//			qDebug() << "RenderSvg: " << idxChild.internalId() << ic; 
										GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										QPixmap qPix;
										qVar = m_treeviewModel->data(idxChild,Qt::UserRole);
										QString qsImage = qVar.toString();
										if(!qsImage.isEmpty())
										{
											qPix = m_treeviewModel->pixmap(qsImage);
											if(!qPix.isNull())
											{
												m_pixmapSize.setHeight(qPix.height());
												m_pixmapSize.setWidth(qPix.width());

												qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
												int iBTS = qVar.toInt();
												if(iBTS<=0)iBTS=1;
												qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);
												int iFixed = qVar.toInt();
												if(iFixed>0)
												{
													qreal distance = m_marbleWidget->distance()*1000;//meter
													int height = m_marbleWidget->viewport()->height();
													int width = m_marbleWidget->viewport()->width(); 
													distance = distance/(qreal)width;
													distance = (qreal)iBTS/distance;
													if(distance<10)distance=10;
													width=height=(int)distance;
													//qPix = qPix.scaled(width,height,Qt::IgnoreAspectRatio,Qt::FastTransformation);
													qPix = qPix.scaled(width,height,Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
												}

												if(m_viewportChanged)
												{
													region = painter->regionFromRect( pos,qPix.width(),qPix.height() );
													if(!region.isEmpty())
													{
														if(m_bEdit || m_bMoveObj)
															m_placemarks.push_front( ModelRegion(idxChild, region ) );
													}
												}
												painter->drawPixmap(pos,qPix); 
											}
										}
									}
									else if (cType == GIS::Text )
									{
							//			qDebug() << "RenderText: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										QString qsImage = qVar.toString();
										if(!qsImage.isEmpty())
										{
											//if(qsImage.contains('\n'))
											//	qsImage.remove('\n');

											int iW = m_pixmapSize.width();
											int iH = m_pixmapSize.height();

											qreal xpos=0;
											qreal ypos=0;
											m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),xpos,ypos);

											if(m_iDimmer>50)
											{
												if(m_fontColor.red()==0 && m_fontColor.green()==0 && m_fontColor.blue()==0)
													painter->setPen( QColor(255,255,255) );
												else painter->setPen(m_fontColor);
											}
											else painter->setPen(m_fontColor);

											/*QBrush brush;
											brush.setColor(m_fontBackColor);
											brush.setStyle(Qt::
											painter->setBackground(brush);*/
											/*if(m_fontBackColor.red()!=255 && m_fontBackColor.green()!=255 && m_fontBackColor.blue()!=255)
											{
												QRadialGradient radialGrad(QPointF(100, 100), 100);
												radialGrad.setColorAt(0, QColor( 198, 198, 198, 200 ) );
												radialGrad.setColorAt(0.5, QColor( 199, 198, 99, 200  ) );
												radialGrad.setColorAt( 1, Qt::white );
												radialGrad.setSpread( QGradient::ReflectSpread );
												QBrush gradientBrush( radialGrad );
												painter->setBrush(gradientBrush);
											}*/

											//painter->drawText( pos, qsImage );
											//painter->drawText(pos,qsImage,QSize(0,0), 10, 30, 5, 5 );
											QSizeF textSize( painter->fontMetrics().width( qsImage ), painter->fontMetrics().height() );  
											qreal rH = textSize.height();
											qreal rW = textSize.width();

											if(qsImage.contains('\n'))
											{
												int tc=qsImage.count('\n')+1;
												int len=0;
												rH *= tc;
												QString tmp;
												for(int itt=0;itt<tc;itt++)
												{
													tmp = qsImage.section('\n',itt,itt);
													if(len<tmp.length())
													{
														QSizeF tSize( painter->fontMetrics().width( tmp ), painter->fontMetrics().height() );
														rW = tSize.width();
														len=tmp.length();
													}
												}
											}

											//left/top
											xpos = xpos - (iW/2);
											ypos = ypos - (iH/2);
											ypos = ypos - rH;
											xpos = xpos - (rW/2);
											//left/bottom 
											//rW = xpos + iX;
											QRect rect(xpos,ypos,iW+rW,iH+rH+rH);
											//QRect rect(xpos,ypos,textSize.width(),textSize.height());
											QRect bRect;

											xpos = rect.left() + (rect.right()-rect.left())/2;
											xpos = xpos - rW/2 - 5;
											ypos = rect.bottom()-rH;

										/*	QMatrix matrixold=painter->matrix();
											QMatrix matrix;
											matrix.rotate(45.0);
											painter->setMatrix(matrix);*/
											
											QLinearGradient linearGrad(QPointF(xpos,ypos), QPointF(xpos+rW+10,ypos+rH));
											linearGrad.setColorAt(0, Qt::white);
											linearGrad.setColorAt(1, Qt::darkGray);

											if(m_iDimmer<=50)
											{
												linearGrad.setColorAt(0, Qt::white);
												linearGrad.setColorAt(1, Qt::darkGray);
												painter->fillRect(xpos,rect.bottom()-rH,rW+10,rH,QBrush(linearGrad));
											}
											else
											{
												linearGrad.setColorAt(0, Qt::darkGray);
												linearGrad.setColorAt(1, Qt::lightGray);
												painter->fillRect(xpos,rect.bottom()-rH,rW+10,rH,QBrush(linearGrad));
											}

									/*		QPainterPath path;
											path.addRoundedRect(rect,45,45,Qt::RelativeSize);
											QPoint pt;
											pt.setX(rect.left());
											pt.setY(rect.bottom());
											path.addText(pt,m_layerFont,qsImage);
											//painter->setPen(pen);
											painter->setBrush(m_fillColor);
											painter->drawPath(path);*/

										//	painter->translate(
											painter->drawText(rect,Qt::AlignBottom|Qt::AlignHCenter,qsImage,&bRect);
											//painter->setMatrix(matrixold);

										/*	outlinepath.addText( baseline, defaultFont,qsImage);
											painter->drawPath( outlinepath );
											painter->setPen( Qt::NoPen );
											painter->drawPath( outlinepath );*/
											painter->setPen(layerPen);
										}
										else
										{
											m_treeviewModel->DeleteObject(idxChild.internalId(),GIS::Text);
											ic--;//to be able to draw next child
										}
									}//if Text
									else if ( cType == GIS::Desc )
									{
							//			qDebug() << "RenderDesc: " << idxChild.internalId() << ic; 
										GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										QString qsDesc = qVar.toString();
										if(!qsDesc.isEmpty())
										{
										//	painter->setPen( QColor(255, 0,0, 255 ) );
											int iW = m_pixmapSize.width();
											int iH = m_pixmapSize.height();

											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
											int iXoff = qVar.toInt();
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
											int iYoff = qVar.toInt();

											qreal xpos=0;
											qreal ypos=0;
											m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),xpos,ypos);

											if(iXoff!=0)
												xpos += (qreal)iXoff;
											if(iYoff!=0)
												ypos += (qreal)iYoff;

											QSizeF textSize( painter->fontMetrics().width( qsDesc ), painter->fontMetrics().height() );  
											qreal rH = textSize.height();
											qreal rW = textSize.width();

											if(qsDesc.contains('\n'))
											{
												rH *= (qsDesc.count('\n')+1);
											}

											qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);
											int iAlignment = qVar.toInt();
											//left/top
											//xpos = xpos - (iW/2);
											//ypos = ypos - (iH/2);
											if(iAlignment==0)//center
											{
												ypos = ypos + rH;
												xpos = xpos - (rW/2);
											}
											else if(iAlignment==1)//upper-left
											{
												ypos = ypos - iH;
												xpos = xpos - rW;
											}
											else if(iAlignment==2)//upper-right
											{
												ypos = ypos - iH;
												xpos = xpos + iW;
											}
											else if(iAlignment==3)//lower-right
											{
												ypos = ypos + rH;
												xpos = xpos + iW;
											}
											else if(iAlignment==4)//lower-left
											{
												ypos = ypos + rH;
												xpos = xpos - rW;
											}
											else
											{
												ypos = ypos + rH;
												xpos = xpos - (rW/2);
											}
											//left/bottom 
											//rW = xpos + iX;
											QRect rect(xpos,ypos,rW,rH);//iW+rW,iH+rH+rH);

											if(m_iDimmer>50)
											{
												if(m_fontColor.red()==0 && m_fontColor.green()==0 && m_fontColor.blue()==0)
													painter->setPen( QColor(255,255,255) );
												else painter->setPen(m_fontColor);
											}
											else painter->setPen(m_fontColor);

											if(m_viewportChanged)
											{
												m_marbleWidget->geoCoordinates((int)xpos,(int)(ypos+rH),xpos,ypos);
												targetPosition.setLatitude(ypos,GeoDataCoordinates::Degree);
												region = painter->regionFromRect(targetPosition,rect.width(),rect.height() );
												if(!region.isEmpty())
												{
													if(m_bEdit || m_bMoveObj)
														m_placemarks.push_front( ModelRegion(idxChild, region ) );
												}
											}

											//painter->drawText(targetPosition, qsDesc );
											//QFontMetrics metrics = painter->fontMetrics(); // geht NUR in einem QWidget!
											//QRect boundingRect = metrics.boundingRect(qsDesc); // ermittle Flche des Textes
											painter->drawText(rect,Qt::AlignTop|Qt::AlignHCenter,qsDesc);//,&boundingRect);
											painter->setPen(layerPen);
										}
									}//if Desc
									else if ( cType == GIS::Plot )
									{
							//			qDebug() << "RenderPlot: " << idxChild.internalId() << ic; 
										GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										//qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										//QString qsName = qVar.toString();
										if(targetPosition!=pos)
											calcPlotCoords(idxChild,targetPosition,pos);

										if(1/*!qsName.isEmpty()&& (pos.longitude()!=pos.latitude() )*/ )
										{
										/*	qVar = m_treeviewModel->data(idxChild,Qt::UserRole);//Qt::UserRole);
											qreal iRadius = (qreal)qVar.toDouble();
											iRadius/=100;

											painter->setBrush( QColor( 255,255,255,0 ) );
											painter->drawEllipse(pos,iRadius*2,iRadius,true ); 
											painter->setBrush(m_fillColor);*/
											/*if ( m_viewportChanged ) 
											{
												if(m_bEdit || m_bMoveObj)
												{
													region = painter->regionFromEllipse( pos, 28, 28 );
													if(!region.isEmpty())
														m_placemarks.push_front( ModelRegion( index, region ) );
												}
											}*/
											GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
											painter->drawPolyline(*gdl);

											if(m_bEdit || m_bMoveObj)
											{
												if(m_viewportChanged) 
												{
													region = painter->regionFromPolyline(*gdl, 8 );
													//region = painter->regionFromPolygon(poly,Qt::WindingFill);
													if(!region.isEmpty())
													{
														m_routeRegion.push_front( ModelRegion( idxChild, region ) );
													}
												}
											}
										}

									}//if Plot
									else if ( cType == GIS::PlotEx )
									{
							//			qDebug() << "RenderPlotEx: " << idxChild.internalId() << ic; 
									/*	qVar = m_treeviewModel->data(idxChild,Qt::UserRole);//Qt::UserRole);
										qreal iRadius = (qreal)qVar.toDouble();
										if(iRadius!=0 && (pos.longitude()!=pos.latitude() ))
										{
										//	painter->setPen(color);// QColor( 99, 198, 99, 255 ) );
											painter->setBrush( QColor( 255,255,255,0 ) );
											painter->drawEllipse(pos,iRadius*10,iRadius*2); 
										//	painter->setPen(layerPen);
											painter->setBrush(m_fillColor);
											//painter->rotate(45);
											
										}*/
										GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										if(targetPosition!=pos)
											calcPlotCoords(idxChild,targetPosition,pos);

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										painter->drawPolyline(*gdl);

										if(m_bEdit || m_bMoveObj)
										{
											if(m_viewportChanged)
											{
												region = painter->regionFromPolyline(*gdl, 8 );
												//region = painter->regionFromPolygon(poly,Qt::WindingFill);
												if(!region.isEmpty())
												{
													m_routeRegion.push_front( ModelRegion( idxChild, region ) );
												}
											}
										}

									}//if PlotEx
									else if ( cType == GIS::Elipse )
									{
							//			qDebug() << "RenderEllipse: " << idxChild.internalId() << ic; 
									
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									
											QPen lPen(color);
											lPen.setWidth( m_lineWidth );
											lPen.setStyle(m_penStyle);
											painter->setPen(lPen);
										}

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												if(iTrans!=0)
													color = QColor( red, green, blue, iTrans);
												else
												{
													iTrans  = ( colorRef>>24 ) & 255;
													color = QColor( red, green, blue, iTrans);
												}
											}
											painter->setBrush(color);
										}
										else
										{
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
											{
												color = m_fillColor;
												color.setAlpha(iTrans);
												painter->setBrush(color);
											}
										}

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										GeoDataPolygon poly( Tessellate );
										poly.setOuterBoundary( GeoDataLinearRing(*gdl) );
										painter->drawPolygon(poly);
										//painter->drawPolyline(*gdl);
										//GeoDataLineString gdl2;
										if(m_bEdit || m_bMoveObj)
										{
											if(m_viewportChanged)
											{
												int i = gdl->size(); 
												i = i/4;
												for(int ix=0;ix<gdl->size();ix+=i)
												{
												//	gdl2.append(gdl->at(ix));
												//	m_polySkeleton->append(gdl->at(ix));
													region = painter->regionFromEllipse(gdl->at(ix), 10, 10 );
													m_selpoint.push_front( ModelRegion( idxChild, region ) );
												}
												region = painter->regionFromEllipse(pos, 20, 20 );
												m_selpoint.push_front( ModelRegion( idxChild, region ) );
											}

											if(m_selectionModel->isSelected( idxChild ))
											{
												qreal x=0;
												qreal y=0;
												if(m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),x,y))
												{
													const int MarkRadius = 5;

													// Paint the mark, and repeat it if the projection allows it.
													painter->setRenderHint( QPainter::Antialiasing, false );
													//painter->setPen( QColor( Qt::red ) );

													painter->drawLine( (int)x - MarkRadius, (int)y, x + MarkRadius, (int)y );
													painter->drawLine( (int)x, (int)y - MarkRadius, (int)x, (int)y + MarkRadius );

													if(0)//m_viewportChanged)
													{
														region = painter->regionFromEllipse(pos, 20, 20 );
														//m_selpoint.push_front( ModelRegion( idxChild, region ) );
														m_placemarks.push_front( ModelRegion(idxChild, region ) );
													}
												}
											}

											if(m_viewportChanged)
											{
												region = painter->regionFromPolyline(*gdl, 8 );
												//region = painter->regionFromPolygon(poly,Qt::WindingFill);
												if(!region.isEmpty())
												{
													m_routeRegion.push_front( ModelRegion( idxChild, region ) );
												}
											}

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										}

									}//if Ellipse
									else if ( cType == GIS::Marker )
									{
							//			qDebug() << "RenderMarker: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										QString qsImage = qVar.toString();

										qVar = m_treeviewModel->data(idxChild,Qt::UserRole);
										int iMarkerType = qVar.toInt();

										if(!qsImage.isEmpty())
										{
											if(iMarkerType==0)//Circle
											{
											/*	QPixmap qPix;
												qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);
												qsImage = qVar.toString();
												if(!qsImage.isEmpty())
													qPix = m_treeviewModel->pixmap(qsImage);
												if(!qPix.isNull())
												{
													painter->drawPixmap(pos,qPix);
													region = painter->regionFromRect( pos,qPix.width(),qPix.height() );
													m_placemarks.push_front( ModelRegion(idxChild, region ) );
												}
												else
												{*/
													//painter->setPen( QColor(255, 0,0, 255 ) );
													//painter->setBrush( QColor( 255,0, 0, 80 ) );
													if(m_viewportChanged)
													{
														region = painter->regionFromEllipse(pos, 20, 20 );
														if(!region.isEmpty())
														{
															if(m_bEdit || m_bMoveObj)
																m_placemarks.push_front( ModelRegion( idxChild, region ) );
														//	painter->setPen(m_lineColor);
														//	painter->setBrush(m_fillColor);
															 
														}
													}
													painter->drawEllipse(pos,m_markerSize,m_markerSize);
											//	}
											}
											if(iMarkerType==6)//Direction
											{
												const GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
	
												if(targetPosition.longitude() <= 360.0 )//!= targetPosition.latitude())
												{
													qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);
													int iDirection = qVar.toInt();
													if(iDirection<=36000)//39900 defined as invalid 
													{ 
														if(m_marbleWidget->heading()!=0)
														{
															if(iDirection!=0)
																iDirection=0;
															else iDirection = m_marbleWidget->heading()*100*-1;
														}
														int ang1 = 0;
														if(iDirection>0)
															ang1 = 90-( (iDirection/100)+10);
														else ang1=80;
														int ang2 = 20*16;//ang1+3;//90*16;//ang1-5;// (ang1+10)* -1;
														
														//layerPen.setWidth(1);
														//painter->setPen(layerPen);
														painter->setPen( QColor( 255,0,0, 255 ) );
														painter->setBrush( QColor(255,0,0, 80 ) );
														painter->drawPie(pos,ang1*16,ang2);
														//layerPen.setWidth( m_lineWidth );
														painter->setPen(layerPen);
														painter->setBrush(m_fillColor);
													}
												}
												
												
											}
										}
									}//if Marker
									else if ( cType == GIS::Poly )
									{
							//			qDebug() << "RenderPoly: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

									/*	qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
										int iXoff = qVar.toInt();
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
										int iYoff = qVar.toInt();*/

										if(colorRef>0)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									
											//painter->setPen(color);
											if(1)//colorRef!=0 )//|| m_lineColor.rgb()!=0)
											{
												QPen lPen(color);
												lPen.setWidth( m_lineWidth );
												lPen.setStyle(m_penStyle);
												painter->setPen(lPen);
											}
									
										}
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												if(iTrans!=0)
													color = QColor( red, green, blue, iTrans);
												else
												{
													iTrans  = ( colorRef>>24 ) & 255;
													color = QColor( red, green, blue, iTrans);
												}
											}
											painter->setBrush(color);
										}

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										/*if(iXoff != 0 || iYoff != 0)
										{

										}
										else*/
											painter->drawPolyline(*gdl);
								
										//if(!qsImage.isEmpty())
										if(m_bEdit || m_bMoveObj)
										{
										//	GeoDataLineString gdl2;
											if(m_viewportChanged)
											{
												int step=1;
												if(gdl->size()>5000)
													step=(gdl->size()/100)*2;
												else if(gdl->size()>1000)
													step=(gdl->size()/100)*5;
												else if(gdl->size()>500)
													step=(gdl->size()/100)*10;
													
												for(int ix=0;ix<gdl->size();ix+=step)
												{
											//		gdl2.append(gdl->at(ix));
												//	m_polySkeleton->append(gdl->at(ix));
													if(ix>=gdl->size())
														ix=gdl->size()-1;
													region = painter->regionFromEllipse(gdl->at(ix), 20, 20 );
													m_selpoint.push_front( ModelRegion( idxChild, region ) );
												}

												region = painter->regionFromPolyline(*gdl, 8 );
												if(!region.isEmpty())
												{
													m_routeRegion.push_front( ModelRegion( idxChild, region ) );
												}
											}
										}

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										

									}
									else if ( cType == GIS::Polygone )
									{
							//			qDebug() << "RenderPolygone: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									
											//painter->setPen(color);
											QPen lPen(color);
											lPen.setWidth( m_lineWidth );
											lPen.setStyle(m_penStyle);
											painter->setPen(lPen);
									
										}

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												if(iTrans!=0)
													color = QColor( red, green, blue, iTrans);
												else
												{
													iTrans  = ( colorRef>>24 ) & 255;
													color = QColor( red, green, blue, iTrans);
												}
											}
											painter->setBrush(color);
										}
										else//use object trans settings
										{
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
											{
												color = m_fillColor;
												color.setAlpha(iTrans);
												painter->setBrush(color);
											}
										}

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										if(gdl->size()==0)return;
										GeoDataPolygon poly( Tessellate );
										poly.setOuterBoundary( GeoDataLinearRing(*gdl) );

										painter->drawPolygon(poly);
								
										//if(!qsImage.isEmpty())
										GeoDataLineString gdl2;
										if(m_bEdit || m_bMoveObj)
										{
											if(m_viewportChanged)
											{
												for(int ix=0;ix<gdl->size();ix++)
												{
													gdl2.append(gdl->at(ix));
												//	m_polySkeleton->append(gdl->at(ix));
													region = painter->regionFromEllipse(gdl->at(ix), 20, 20 );
													m_selpoint.push_front( ModelRegion( idxChild, region ) );
												}

												//to be able to select last line also
												gdl2.append(gdl->at(0));
												region = painter->regionFromPolyline(gdl2, 8 );
												//region = painter->regionFromPolygon(poly,Qt::WindingFill);
												if(!region.isEmpty())
												{
													m_routeRegion.push_front( ModelRegion( idxChild, region ) );
												}
											}
										}

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										

									}
									else if ( cType == GIS::Arrow )
									{
							//			qDebug() << "RenderArrow: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::DataRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();
										red      = colorRef & 255;
										green    = ( colorRef>>8 ) & 255;
										blue     = ( colorRef>>16 ) & 255;
										color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									//	painter->setPen( QColor( 255,255,0, 255 ) );
									//	painter->setPen(color);
									//	painter->setBrush( QColor( 99, 198, 99, 80 ) );
									//	painter->setPen(m_lineColor);
									//	painter->setBrush(m_fillColor);

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
										int dirc1 = qVar.toInt();
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
										int dirc2 = qVar.toInt();

									//	logging(QString("1"),dirc1,dirc2,0,0);

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										if(gdl->size()<2)break;

										GeoDataCoordinates pos1 = gdl->at(0);
										GeoDataCoordinates pos2 = gdl->at(1);

									//	logging(QString("2"),(long)pos1.longitude(GeoDataCoordinates::Degree),(long)pos1.latitude(GeoDataCoordinates::Degree),(long)pos2.longitude(GeoDataCoordinates::Degree),(long)pos2.latitude(GeoDataCoordinates::Degree));
										
										qreal lx1=0;
										qreal ly1=0;
										m_marbleWidget->screenCoordinates(pos1.longitude(GeoDataCoordinates::Degree),pos1.latitude(GeoDataCoordinates::Degree),lx1,ly1);
										qreal lx2=0;
										qreal ly2=0;
										m_marbleWidget->screenCoordinates(pos2.longitude(GeoDataCoordinates::Degree),pos2.latitude(GeoDataCoordinates::Degree),lx2,ly2);

									//	logging(QString("3"),(long)lx1,(long)ly1,(long)lx2,(long)ly2);

										QPoint pt0((int)lx1,(int)ly1);
										QPoint pt1((int)lx2,(int)ly2);

										double y=(double)ly2-(double)ly1;
										double x=(double)lx2-(double)lx1;
										double c=sqrt(pow(x,2)+pow(y,2));
										if (c>=70) { //fits to the 40 distance, so draw it
											double angle=atan2(y,x); //in radians
											angle+=PI/2;
											if (dirc1>0) {
												DrawArrow(painter,pt0,angle, dirc1,color);
											}
											if (dirc2>0) {
												angle -=PI;
												DrawArrow(painter,pt1,angle, dirc2,color);
											}
										}
										//painter->drawPolyline(*gdl);
								
										//painter->dra
										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										

									}
									else if (cType == GIS::Azimuth )
									{
									//	qDebug() << "RenderAzimuth: " << idxChild.internalId() << ic; 
										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										//if(!qsImage.isEmpty())
										painter->drawPolyline(*gdl);
									}//if Azimuth
									else if ( cType == GIS::Rect )
									{
					//					qDebug() << "RenderRect: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									
											//painter->setPen(color);
											QPen lPen(color);
											lPen.setWidth( m_lineWidth );
											lPen.setStyle(m_penStyle);
											painter->setPen(lPen);
										}

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												if(iTrans!=0)
													color = QColor( red, green, blue, iTrans);
												else
												{
													iTrans  = ( colorRef>>24 ) & 255;
													color = QColor( red, green, blue, iTrans);
												}
											}
											painter->setBrush(color);
										}
										else
										{
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
											{
												color = m_fillColor;
												color.setAlpha(iTrans);
												painter->setBrush(color);
											}
										}

										const GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										//targetPosition.
										//qVar = m_treeviewModel->data(idxChild,MarblePlacemarkModel::CoordinateRole);//Qt::UserRole);
										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);

										GeoDataCoordinates lt = gdl->at(0);
										GeoDataCoordinates rb = gdl->at(1);
										
										qreal width  = rb.longitude(GeoDataCoordinates::Degree)-lt.longitude(GeoDataCoordinates::Degree);
										qreal height = lt.latitude(GeoDataCoordinates::Degree)- rb.latitude(GeoDataCoordinates::Degree);

										//painter->drawRoundRect(targetPosition,width,height,25,25,true); 

										painter->drawRect(targetPosition,width,height,true);

										if(m_bEdit || m_bMoveObj)
										{
											if(m_viewportChanged)
											{
												GeoDataLineString gdlr;
												gdlr.append(lt);//left-top
												width  = lt.longitude(GeoDataCoordinates::Degree);
												height = lt.latitude(GeoDataCoordinates::Degree);
												lt.setLongitude(rb.longitude(GeoDataCoordinates::Degree),GeoDataCoordinates::Degree);
												gdlr.append(lt);//right-top
												gdlr.append(rb);//right-bottom
												rb.setLongitude(width,GeoDataCoordinates::Degree);
												gdlr.append(rb);//left-bottom

												rb.setLatitude(height,GeoDataCoordinates::Degree);
												gdlr.append(rb);
												region = painter->regionFromEllipse(targetPosition, 28, 28 );

												for(int ix=0;ix<gdlr.size();ix++)
												{
													region = painter->regionFromEllipse(gdlr.at(ix), 20, 20 );
													m_selpoint.push_front( ModelRegion( idxChild, region ) );
												}
											
												region = painter->regionFromPolyline(gdlr, 8 );
												m_routeRegion.push_front( ModelRegion( idxChild, region) );
											}
										}
										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
									}//if Rect 
								}
							}
						}
					}//child count
				}
			}
		}
    }

	m_viewportChanged=false;
	m_contentChanged=false;
//	qDebug() << "RenderLayerOut: " << idxParent.internalId() << iRowCount; 

}

void MapViewTSLayerPrivate::logging(QString& str1,long val1,long val2,long val3,long val4)
{
	QString qsFile("C://mvcontrol.txt");
	QFile file( qsFile );

	if ( !file.open( QIODevice::WriteOnly | QIODevice::Append) ) {
        qCritical() << "Cannot write to " << file.fileName();
    }

	QString qsMsg1 = ("%1");
	qsMsg1 = qsMsg1.arg(val1);
	QString qsMsg2 = ("%1");
	qsMsg2 = qsMsg2.arg(val2);
	QString qsMsg3 = ("%1");
	qsMsg3 = qsMsg3.arg(val3);
	QString qsMsg4 = ("%1");
	qsMsg4 = qsMsg4.arg(val4);

	QTextStream out(&file);

	//if(str2.isEmpty())
	out << str1 << " : " << qsMsg1 << " : " << qsMsg2 << " : " << qsMsg3 << " : " << qsMsg4 ; 
	//else out << str1 << " : " << qsMsg << " : " << str2 << "\n";

	//file.write(qsMsg.toLatin1(),qsMsg.length());
	QString qs("\n");
	file.write(qs.toAscii(),qs.length()); 

	file.close();
}

void MapViewTSLayerPrivate::renderLegendLayer( GeoPainter *painter )
{
	if(!m_update)return;

	long colorRef = -1;
	int red      = 0;
	int green    = 0;
	int blue     = 0;
	QColor color;

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);

	if(m_viewportChanged)
	{
		m_placemarks.clear();
		m_routeRegion.clear();
		m_selpoint.clear();
	}

	QVariant qVar;

	painter->setFont(m_layerFont);

	QPen layerPen(m_lineColor);
    layerPen.setWidth(m_lineWidth );
	layerPen.setStyle(m_penStyle);
	painter->setPen(layerPen);
	painter->setBrush(m_fillColor);
	//painter->setBrush(Qt::BrushStyle::NoBrush);//m_fillColor);

	//group item count
	int iRowCount = m_treeviewModel->rowCount(idxParent);
	int iReg=0;

	//for legend drawing relative start point
	int iMapHeight = m_marbleWidget->height();
	int iMapWidth  = m_marbleWidget->width();
	int oW=m_legendRect.right();
	int oH=m_legendRect.bottom();
	int iSX = 0;//iMapWidth-oW; 
	int iSY = 0;//iMapHeight-oH; 

//	qDebug() << "RenderLayer: " << idxParent.internalId() << iRowCount; 
	//m_iLegendAlign=2;
	switch(m_iLegendAlign)
	{
		case 0:		 //left-top
			iSX+=10;
			iSY+=10;
			break;
		case 1:		//right-top
			iSX = iMapWidth-oW; 
			iSX-=10;
			iSY+=10;
			break;
		case 2:		//left-bottom
			iSY = iMapHeight-oH; 
			iSX+=10;
			iSY-=10;
			break;
		case 3:		//right-bottom
			iSX = iMapWidth-oW; 
			iSY = iMapHeight-oH; 
			iSX-=10;
			iSY-=10;
			break;
	}

	if(iSX < 0 || iSX > iMapWidth)
		iSX=0;
	if(iSY < 0 || iSY > iMapHeight)
		iSY=0;

    for ( int i = 0; i < iRowCount; ++i ) 
	{

        QModelIndex index = m_treeviewModel->index( i, 0,idxParent );

		if ( index.isValid() )
		{
			qVar = m_treeviewModel->data(index,Qt::CheckStateRole);
			if(qVar.toBool())
			{
				//group child count
				int iChildCount = m_treeviewModel->rowCount(index);

				//QVariant qVar;

				GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
				//GIS::TreeViewItemType type = qVariantValue<Model::GIS::TreeViewItemType>( index.data( TreeViewModel::TypeRole ) );
				//int type = (int)index.data( TreeViewModel::TypeRole );
				GIS::TreeViewItemType type = (GIS::TreeViewItemType) qVariantValue<unsigned int>( index.data( TreeViewModel::TypeRole ) );
				if ( type == GIS::Group ) 
				{
					if(iChildCount>0)//m_treeviewModel->rowCount(index)>0)
					{
						for ( int ic = 0; ic < iChildCount; ++ic ) 
						{
							QModelIndex idxChild = m_treeviewModel->index( ic, 0,index );
							if(idxChild.isValid())
							{
								qVar = m_treeviewModel->data(idxChild,Qt::CheckStateRole);
								if(qVar.toBool())
								{
									GIS::TreeViewItemType cType = (GIS::TreeViewItemType) qVariantValue<unsigned int>( idxChild.data( TreeViewModel::TypeRole ) );
									if ( cType == GIS::Icon )
									{

									}//Icon
									else if ( cType == GIS::Marker )
									{
							//			qDebug() << "RenderMarker: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										QString qsImage = qVar.toString();

										qVar = m_treeviewModel->data(idxChild,Qt::UserRole);
										int iMarkerType = qVar.toInt();

										if(!qsImage.isEmpty())
										{
											if(iMarkerType==0)//Circle
											{
												painter->drawEllipse(pos,m_markerSize,m_markerSize);
											}
											else if(iMarkerType==1)//Rectangle
											{
												qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
												colorRef = (long)qVar.toLongLong();

												if(colorRef!=-1)
												{
													red      = colorRef & 255;
													green    = ( colorRef>>8 ) & 255;
													blue     = ( colorRef>>16 ) & 255;
													color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
											//	painter->setPen( QColor( 255,255,0, 255 ) );
													painter->setPen(color);
											//	painter->setPen(m_penStyle);
												}

												qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
												colorRef = (long)qVar.toLongLong();

												if(colorRef!=-1)
												{
													red      = colorRef & 255;
													green    = ( colorRef>>8 ) & 255;
													blue     = ( colorRef>>16 ) & 255;
													qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
													int iTrans = qVar.toInt();
													if(iTrans!=-1)
														color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
													else 
													{
														iTrans = m_fillColor.alpha();
														color = QColor( red, green, blue, iTrans);
													}
													painter->setBrush(color);
												}
												else
												{
													qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
													int iTrans = qVar.toInt();
													if(iTrans!=-1)
													{
														color = m_fillColor;
														color.setAlpha(iTrans);
														painter->setBrush(color);
													}
												}

												//painter->drawRoundRect(targetPosition,width,height,25,25,true); 
												const GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
												QRectF rect;
												rect.setLeft(targetPosition.longitude(GeoDataCoordinates::Degree));
												rect.setTop(targetPosition.latitude(GeoDataCoordinates::Degree));
												rect.setRight(targetPosition.longitude(GeoDataCoordinates::Degree)+m_markerSize);
												rect.setBottom(targetPosition.latitude(GeoDataCoordinates::Degree)+m_markerSize);

												rect.setRect(rect.left()+iSX,rect.top()+iSY,rect.width(),rect.height());
												painter->drawRect(rect);
												
												//painter->drawRect(targetPosition,m_markerSize,m_markerSize,true);

												painter->setPen(layerPen);
												painter->setBrush(m_fillColor);
											}//type rectangle
										}//!empty
									}//Marker
									else if ( cType == GIS::Rect )
									{
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									//	painter->setPen( QColor( 255,255,0, 255 ) );
											painter->setPen(color);
									//	painter->setPen(m_penStyle);
										}

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												color = QColor( red, green, blue, iTrans);
											}
											painter->setBrush(color);
										}
										else
										{
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
											{
												color = m_fillColor;
												color.setAlpha(iTrans);
												painter->setBrush(color);
											}
										}

										const GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										//targetPosition.
										//qVar = m_treeviewModel->data(idxChild,MarblePlacemarkModel::CoordinateRole);//Qt::UserRole);
										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);

										GeoDataCoordinates lt = gdl->at(0);
										GeoDataCoordinates rb = gdl->at(1);
										
										qreal width  = rb.longitude(GeoDataCoordinates::Degree)-lt.longitude(GeoDataCoordinates::Degree);
										qreal height = lt.latitude(GeoDataCoordinates::Degree)- rb.latitude(GeoDataCoordinates::Degree);

										//painter->drawRoundRect(targetPosition,width,height,25,25,true); 
										//painter->drawRect(targetPosition,width,height,true);
										QRectF rect;
										rect.setLeft(lt.longitude(GeoDataCoordinates::Degree));
										rect.setTop(lt.latitude(GeoDataCoordinates::Degree));
										rect.setRight(rb.longitude(GeoDataCoordinates::Degree));
										rect.setBottom(rb.latitude(GeoDataCoordinates::Degree));

										rect.setRect(rect.left()+iSX,rect.top()+iSY,rect.width(),rect.height());

										/*
										if(m_iLegendAlign!=0)//not left-top
										{
											switch(m_iLegendAlign)
											{
												case 1:		//right-top
													rect.setRect(rect.left()+iSX,rect.top(),rect.width(),rect.height());
													break;
												case 2:		//left-bottom
													rect.setRect(rect.left(),rect.top()+iSY,rect.width(),rect.height());
													break;
												case 3:		//right-bottom
													rect.setRect(rect.left()+iSX,rect.top()+iSY,rect.width(),rect.height());
													break;
											}
										}*/

										painter->drawRect(rect);

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);

									}//Rect
									else if ( cType == GIS::Poly )
									{
							//			qDebug() << "RenderPoly: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									//	painter->setPen( QColor( 255,255,0, 255 ) );
											painter->setPen(color);
									//	painter->setPen(m_penStyle);
										}
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else color = QColor( red, green, blue);
											painter->setBrush(color);
										}

									//	painter->setBrush( QColor( 99, 198, 99, 80 ) );
									//	painter->setPen(m_lineColor);
									//	painter->setBrush(m_fillColor);

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										QPolygonF qPoly;
										QPointF pt;
										GeoDataCoordinates pPos;
										
										for(int i=0;i<gdl->size();i++)
										{
											pPos = gdl->at(i);
											pt.setX(pPos.longitude(GeoDataCoordinates::Degree));
											pt.setY(pPos.latitude(GeoDataCoordinates::Degree));											
											qPoly << pt;
										}

										QPolygonF qTemp;
										for(int x=0;x<qPoly.size();x++)
										{
											pt = qPoly.at(x);
											pt.setX(pt.x()+iSX);
											pt.setY(pt.y()+iSY);
											qTemp << pt;
										}
										qPoly.clear();
										qPoly << qTemp;

										/*
										if(m_iLegendAlign!=0)//not left-top
										{
											switch(m_iLegendAlign)
											{
												case 1:			//right-top
													{
														QPolygonF qTemp;
														for(int x=0;x<qPoly.size();x++)
														{
															pt = qPoly.at(x);
															pt.setX(pt.x()+iSX);
															qTemp << pt;
														}
														qPoly.clear();
														qPoly << qTemp;

													}
													break;
												case 2:			//left-bottom
													{
														QPolygonF qTemp;
														for(int x=0;x<qPoly.size();x++)
														{
															pt = qPoly.at(x);
													//		pt.setX(pt.x()+iSX);
															pt.setY(pt.y()+iSY);
															qTemp << pt;
														}
														qPoly.clear();
														qPoly << qTemp;

													}
													break;
												case 3:			//right-bottom
													{
														QPolygonF qTemp;
														for(int x=0;x<qPoly.size();x++)
														{
															pt = qPoly.at(x);
															pt.setX(pt.x()+iSX);
															pt.setY(pt.y()+iSY);
															qTemp << pt;
														}
														qPoly.clear();
														qPoly << qTemp;

													}
													break;
											}


										}*/

										painter->drawPolygon(qPoly);
										//painter->drawPolyline(qPoly);//draws empty polygones
										//painter->drawPolyline(*gdl);
								
										//if(!qsImage.isEmpty())
									/*	if(m_bEdit || m_bMoveObj)
										{
										//	GeoDataLineString gdl2;
											if(m_viewportChanged)
											{
												for(int ix=0;ix<gdl->size();ix++)
												{
											//		gdl2.append(gdl->at(ix));
												//	m_polySkeleton->append(gdl->at(ix));
													region = painter->regionFromEllipse(gdl->at(ix), 20, 20 );
													m_selpoint.push_front( ModelRegion( idxChild, region ) );
												}

												region = painter->regionFromPolyline(*gdl, 8 );
												if(!region.isEmpty())
												{
													m_routeRegion.push_front( ModelRegion( idxChild, region ) );
												}
											}
										}*/

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										

									}//poly
									else if ( cType == GIS::Polygone )
									{
							//			qDebug() << "RenderPolygone: " << idxChild.internalId() << ic; 
										qVar = m_treeviewModel->data(idxChild,TreeViewModel::LineColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											color = QColor( red, green, blue );// = QColormap::instance().colorAt(ref));
									
											painter->setPen(color);
										}

										qVar = m_treeviewModel->data(idxChild,TreeViewModel::FillColorRole);//Qt::UserRole);
										colorRef = (long)qVar.toLongLong();

										if(colorRef!=-1)
										{
											red      = colorRef & 255;
											green    = ( colorRef>>8 ) & 255;
											blue     = ( colorRef>>16 ) & 255;
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
												color = QColor( red, green, blue,iTrans );// = QColormap::instance().colorAt(ref));
											else 
											{
												iTrans = m_fillColor.alpha();
												if(iTrans!=0)
													color = QColor( red, green, blue, iTrans);
												else
												{
													iTrans  = ( colorRef>>24 ) & 255;
													color = QColor( red, green, blue, iTrans);
												}
											}
											painter->setBrush(color);
										}
										else//use object trans settings
										{
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::TransRole);
											int iTrans = qVar.toInt();
											if(iTrans!=-1)
											{
												color = m_fillColor;
												color.setAlpha(iTrans);
												painter->setBrush(color);
											}
										}

										GeoDataLineString* gdl = m_treeviewModel->getPoly(idxChild);
										if(gdl->size()==0)return;
										
										QPolygonF qPoly;
										QPointF pt;
										GeoDataCoordinates pPos;
										
										for(int i=0;i<gdl->size();i++)
										{
											pPos = gdl->at(i);
											pt.setX(pPos.longitude(GeoDataCoordinates::Degree));
											pt.setY(pPos.latitude(GeoDataCoordinates::Degree));											
											qPoly << pt;
										}

										QPolygonF qTemp;
										for(int x=0;x<qPoly.size();x++)
										{
											pt = qPoly.at(x);
											pt.setX(pt.x()+iSX);
											pt.setY(pt.y()+iSY);
											qTemp << pt;
										}
										qPoly.clear();
										qPoly << qTemp;

										painter->drawPolygon(qPoly);

										painter->setPen(layerPen);
										painter->setBrush(m_fillColor);
										

									}
									else if ( cType == GIS::HotSpot )
									{
							//			qDebug() << "RenderSvg: " << idxChild.internalId() << ic; 
										GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										QPixmap qPix;
										qVar = m_treeviewModel->data(idxChild,Qt::UserRole);
										QString qsImage = qVar.toString();
										if(!qsImage.isEmpty())
										{
											qPix = m_treeviewModel->pixmap(qsImage);
											if(!qPix.isNull())
											{
												int x = targetPosition.longitude(GeoDataCoordinates::Degree);
												int y = targetPosition.latitude(GeoDataCoordinates::Degree);
												painter->drawPixmap(x,y,qPix); 
											}
										}
									}
									else if ( cType == GIS::Desc )
									{
							//			qDebug() << "RenderDesc: " << idxChild.internalId() << ic; 
										const GeoDataCoordinates targetPosition = idxChild.data( MarblePlacemarkModel::CoordinateRole ).value<GeoDataCoordinates>();
										qVar = m_treeviewModel->data(idxChild,Qt::DisplayRole);//Qt::UserRole);
										QString qsDesc = qVar.toString();
										if(!qsDesc.isEmpty())
										{
										//	painter->setPen( QColor(255, 0,0, 255 ) );
											int iW = 0;//m_pixmapSize.width();
											int iH = 0;//m_pixmapSize.height();

											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffXRole);
											int iXoff = qVar.toInt();
											qVar = m_treeviewModel->data(idxChild,TreeViewModel::OffYRole);
											int iYoff = qVar.toInt();

											qreal xpos= pos.longitude(GeoDataCoordinates::Degree);
											qreal ypos= pos.latitude(GeoDataCoordinates::Degree);
											//m_marbleWidget->screenCoordinates(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),xpos,ypos);

										/*	if(iXoff!=0)
												xpos += (qreal)iXoff;
											if(iYoff!=0)
												ypos += (qreal)iYoff;*/

											QSizeF textSize( painter->fontMetrics().width( qsDesc ), painter->fontMetrics().height() );  
											qreal rH = textSize.height();
											qreal rW = textSize.width();

											if(qsDesc.contains('\n'))
											{
												rH *= (qsDesc.count('\n')+1);
											}
											if(qsDesc.at(0)==':')
												xpos+=2;//qsDesc.insert(0,' ');

											//left/top
										/*	xpos = xpos - (iW/2);
											ypos = ypos - (iH/2);
											ypos = ypos - rH;
											xpos = xpos - (rW/2);*/
											//left/bottom 
											//rW = xpos + iX;
											xpos += 0.5;
											QRect rect((int)xpos,(int)ypos,iW+rW,iH+rH+rH);
											rect.setRect(xpos+iSX,ypos+iSY,iW+rW,iH+rH+rH);

											/*
											if(m_iLegendAlign!=0)//not left-top
											{
												switch(m_iLegendAlign)
												{
													case 1:		//right-top
														rect.setRect(xpos+iSX,ypos,iW+rW,iH+rH+rH);
														break;
													case 2:		//left-bottom
														rect.setRect(xpos,ypos+iSY,iW+rW,iH+rH+rH);
														break;
													case 3:		//right-bottom
														rect.setRect(xpos+iSX,ypos+iSY,iW+rW,iH+rH+rH);
														break;
												}
											}*/

											if(0)//m_iDimmer>50)
											{
												if(m_fontColor.red()==0 && m_fontColor.green()==0 && m_fontColor.blue()==0)
													painter->setPen( QColor(255,255,255) );
												else painter->setPen(m_fontColor);
											}
											else painter->setPen(m_fontColor);

											
											if(m_fontBackColor.red()!=255 || m_fontBackColor.green()!=255 || m_fontBackColor.blue()!=255)
											{
												QRadialGradient radialGrad(QPointF(100, 100), 100);
												radialGrad.setColorAt(0,m_fontBackColor );
												radialGrad.setColorAt(0.5, QColor( 160, 0, 0, 200  ) );
												radialGrad.setColorAt( 1,m_fontBackColor );
												radialGrad.setSpread( QGradient::ReflectSpread );
												QBrush gradientBrush( radialGrad );
											//	painter->setBrush(gradientBrush);
											/*	QBrush brush;
												brush.setColor(m_fontBackColor);
												brush.setStyle(Qt::SolidPattern);*/
												//painter->setBrush(brush);
												QRect fillRect(rect);
												fillRect.setTop(rect.top()-5);
												fillRect.setLeft(rect.left()-5);
												fillRect.setRight(rect.right()+5);
												painter->fillRect(fillRect,gradientBrush);
											}

											//painter->drawText(targetPosition, qsDesc );
											
											painter->drawText(rect,Qt::AlignHCenter,qsDesc);
											painter->setPen(layerPen);
										}
									}//if Desc
								}//check state
							}//idx valiid
						}//for iChildCount
					}//if iChildCount>0
				}//if type == Group
			}//check state
		}//if index valid
	}//for row cont

}

void MapViewTSLayerPrivate::renderAnnotations( GeoPainter *painter )
{
	if(m_treeviewModel!=NULL&&m_infoID>0)
	{
		//TreeNode* pNode = static_cast<TreeNode *>((void*)m_infoID);
		TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_infoID);
		if(pNode==NULL || pNode->parent==NULL)
		{
			m_infoID=0;
			return;
		}

		if(pNode->ItemType<4 || pNode->ItemType>18)
		{
			m_infoID=0;
			return;
		}

		try{

			if(pNode->parent->m_qsToolTip.isEmpty())
			{
				m_infoID=0;
				return;
			}
		}
		catch(...)
		{
			return;
		}

	//	QString tmp1;
		QString qsStr = pNode->parent->m_qsName;
		qsStr += "\n";
		int iLen=0;
		int lCount = pNode->parent->m_qsToolTip.count(";");
		if(lCount<=1)
		{
			qsStr += pNode->parent->m_qsToolTip;
			iLen = pNode->parent->m_qsToolTip.length();
		//	tmp1 = pNode->parent->m_qsToolTip;
			if(iLen < pNode->parent->m_qsName.length())
			{
				iLen = pNode->parent->m_qsName.length();
		/*		tmp1 = pNode->parent->m_qsName;
				if( tmp1.contains( QRegExp( "^[0-9]\\..+" ) ) )
					iLen*=2;*/
			}
		}
		else
		{
			++lCount;
			QString tmp;
			iLen = pNode->parent->m_qsName.length();
		//	tmp1 = pNode->parent->m_qsName;
			for(int i=0;i<lCount;i++)
			{
				tmp = pNode->parent->m_qsToolTip.section(';',i,i);
				qsStr += tmp;
				if(i+1<lCount)
					qsStr += "\n";
				if(iLen < tmp.length())
				{
			//		tmp1 = tmp;
					iLen = tmp.length();
			/*		if( tmp1.contains( QRegExp( "^[0-9]\\..+" ) ) )
						iLen*=2;*/
				}
			}
		}

		 qreal lon( 0.0 ), lat( 0.0 );
		 if ( m_marbleWidget->geoCoordinates( m_posInfo.x(), m_posInfo.y(),
                                         lon, lat, GeoDataCoordinates::Radian ) )
		 {
				GeoDataCoordinates pos;
				pos.set(lon,lat);
                painter->setPen( QColor( Qt::black ) );
				QRadialGradient radialGrad(QPointF(100, 100), 100);
				radialGrad.setColorAt(0, QColor( 198, 198, 198, 200 ) );
				radialGrad.setColorAt(0.5, QColor( 199, 198, 99, 200  ) );
				radialGrad.setColorAt( 1, Qt::white );
				radialGrad.setSpread( QGradient::ReflectSpread );

				QBrush gradientBrush( radialGrad );
				painter->setBrush( gradientBrush );

				//QSizeF textSize( painter->fontMetrics().width( tmp1 )+8, 0);
				QSizeF textSize( iLen*6, 0);
				//if(lCount>6)
				//	textSize.setHeight(5*lCount*painter->fontMetrics().height());
				painter->drawAnnotation( pos,qsStr, textSize, -10, -20, 5, 5 );
				m_annoRect = QRect(m_posInfo.x(),m_posInfo.y(),textSize.width(),textSize.height());
				m_annoRect.setLeft(m_annoRect.left()-textSize.width());
				m_annoRect.setTop(m_annoRect.top()-100);
				m_annoRect.setRight(m_annoRect.left()+2*textSize.width());
				m_annoRect.setBottom(m_posInfo.y());
				//m_annoRect.setHeight(100);
				//m_annoRect.setLeft(m_annoRect.left()-sToolTip.width());

				m_marbleWidget->update();
		 }

	}

}
/*
void MapViewTSLayerPrivate::renderAnnotations( GeoPainter *painter )
{
    if ( !m_selectionModel || m_selectionModel->selection().isEmpty() ) {
        // nothing to do
        return;
    }

	QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
	if(!idxParent.isValid())return;
	QModelIndex index = m_treeviewModel->index( 0, 0, idxParent );
	if(!index.isValid())return;

	int iChildCount = m_treeviewModel->rowCount(index);

    for ( int i = 0; i < iChildCount; ++i ) {
        
		QModelIndex idxChild = m_treeviewModel->index( i, 0,index );

        if ( m_treeviewModel && m_selectionModel ) {
            if ( m_selectionModel->selection().contains( idxChild ) ) {
                //bool const smallScreen = MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen;
				GeoDataCoordinates pos = qVariantValue<GeoDataCoordinates>( index.data( TreeViewModel::CoordinateRole ) );
                painter->setPen( QColor( Qt::black ) );
                //painter->setBrush( QBrush( oxygenSunYellow6 ) );
				QRadialGradient radialGrad(QPointF(100, 100), 100);
				radialGrad.setColorAt(0, QColor( 198, 198, 198, 200 ) );
				radialGrad.setColorAt(0.5, QColor( 199, 198, 99, 200  ) );
				radialGrad.setColorAt( 1, Qt::white );
				radialGrad.setSpread( QGradient::ReflectSpread );

				QBrush gradientBrush( radialGrad );
				painter->setBrush( gradientBrush );

                painter->drawAnnotation( pos, idxChild.data().toString(), QSize( 120, 0 ), 10, 30, 5, 5 );
            }
        }
    }
}*/

void MapViewTSLayerPrivate::renderRequest( GeoPainter *painter )
{
	if(!m_selIdx.isValid())return;

	QVariant qVar = m_treeviewModel->data(m_selIdx,Qt::CheckStateRole);
	if(!qVar.toBool())return;

	Model::TreeNode* pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
	if(pNode==NULL)return;

	if(pNode->ItemType==GIS::Overlay ||pNode->ItemType==GIS::Layer || pNode->ItemType==GIS::Group)
		return;

	if(m_selectionModel->isSelected( m_selIdx )&&m_treeviewModel)
	{
		//qDebug() << "RenderRequest: " << m_selIdx.internalPointer();
		m_regions.clear();
		//Model::TreeNode* pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
		if(pNode!=NULL&& (pNode->ItemType==GIS::Elipse ||pNode->ItemType==GIS::Polygone || pNode->ItemType==GIS::Poly || pNode->ItemType==GIS::Rect) )
		{
			QString qsPix = m_selTypePixmaps[ GIS::PolySelect ];
			QPixmap pixmap(qsPix);
			GeoDataCoordinates pos;
			int i=0;
			if(pNode->ItemType!=GIS::Elipse)
			{
				for ( i = 0; i < pNode->m_poly.size(); ++i ) 
				{
					pos = pNode->m_poly.at( i );
					if ( pos.longitude() != 0.0 && pos.latitude() != 0.0 ) 
					{
						if(!pixmap.isNull())
							painter->drawPixmap( pos, pixmap );
						QRegion region = painter->regionFromRect( pos, pixmap.width(), pixmap.height() );
						m_regions.push_front( RequestRegion( i, region ) );
					}
				}
			}
			else
			{
				QRegion region;
				int x = pNode->m_poly.size(); 
				x = x/4;
			
				for ( i = 0; i < pNode->m_poly.size(); i+=x ) 
				{
					pos = pNode->m_poly.at( i );
					if ( pos.longitude() != 0.0 && pos.latitude() != 0.0 ) 
					{
						if(!pixmap.isNull())
							painter->drawPixmap( pos, pixmap );
						region = painter->regionFromRect( pos, pixmap.width(), pixmap.height() );
						m_regions.push_front( RequestRegion( i, region ) );
					}
					pos = pNode->parent->m_position;
					region = painter->regionFromRect( pos, pixmap.width(), pixmap.height() );
					m_regions.push_front( RequestRegion( 5, region ) );
				}
			}
			if(pNode->ItemType==GIS::Rect)
			{
				pos = pNode->m_poly.at( 0 );
				qreal dLeft = pos.longitude(GeoDataCoordinates::Degree);
				pos = pNode->m_poly.at( 1 );
				pos.setLongitude(dLeft,GeoDataCoordinates::Degree);
				qsPix = m_selTypePixmaps[ GIS::RectCorner ];
				pixmap.load(qsPix);
				if(!pixmap.isNull())
						painter->drawPixmap( pos, pixmap );
				QRegion region = painter->regionFromRect( pos, pixmap.width(), pixmap.height() );
				m_regions.push_front( RequestRegion( i, region ) );
			}
		}
	}
	else m_regions.clear();
}


void MapViewTSLayerPrivate::storeDragPosition( const QPoint &pos )
{
    m_dragStopOver = pos;
    m_dragStopOverRightIndex = -1;

    qreal lon( 0.0 ), lat( 0.0 );
	
 /*   if ( m_polySkeleton && !pos.isNull()
        && m_marbleWidget->geoCoordinates( pos.x(), pos.y(), lon, lat, GeoDataCoordinates::Radian ) ) {
        GeoDataCoordinates waypoint( lon, lat );
     //   m_dragStopOverRightIndex = m_routingModel->rightNeighbor( waypoint, m_routeSkeleton );
    }*/
}

bool MapViewTSLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
	if(!m_update)
		return false;
	if(m_bInputEnabled)
		return false;

    if ( m_pointSelection ) {
        return e->button() == Qt::LeftButton;
    }

	//reset selection
	if(m_selectionModel->isSelected( m_selIdx ) )
	{
		QItemSelectionModel::SelectionFlag command = QItemSelectionModel::Clear;
		m_selectionModel->select( m_selIdx, command );
		if(m_bGeoProjected)
		{
			emit q->placemarkSelected(m_selIdx,q->getInterfaceID(),false );
		}
	}

	m_selIdx = QModelIndex();
	m_bIsPoly=false;
	m_bObjId=false;
	m_movingID = 0;
    m_movingIndex = -1;
	if(!m_bEdit)return false;

	QPoint pt = e->pos();
	m_mousePos.setX(pt.rx());
	m_mousePos.setY(pt.ry());

	//free multi selection
	if(m_multiItemSelect.count()>1 && e->button() != Qt::RightButton )
	{
		QItemSelectionModel::SelectionFlag command =  QItemSelectionModel::Clear;
		int ic=m_multiItemSelect.count();
		for(int i=0;i<ic;i++)
		{
			QModelIndex idx = m_multiItemSelect.value(i);
			m_selectionModel->select( idx, command );
			if(m_bGeoProjected)
			{
				emit q->placemarkSelected(idx,q->getInterfaceID(),false );
			}
		}
		m_multiItemSelect.clear();
	}

	/* 
	if enabled you can not pan the map after a object is selected !!??
	if(m_selIdx.isValid())
	{
		m_selIdx = QModelIndex();
		m_bObjId = false;
		m_movingIndex = -1;
	}*/

	//the poly line via points
    foreach( const RequestRegion &region, m_regions ) {
        if ( region.region.contains( e->pos() ) ) 
		{
			 foreach( const ModelRegion &regionm, m_selpoint ) {
				if ( regionm.region.contains( e->pos() ) ) 
				{
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
					if ( m_selectionModel->isSelected( regionm.index ) ) {
						command = QItemSelectionModel::Clear;
						sel=false;
					}
					m_selIdx = regionm.index;
					m_bObjId = true;
					m_selectionModel->select( regionm.index, command );
					if(m_bGeoProjected)
						emit q->placemarkSelected( regionm.index,q->getInterfaceID(),sel );
					// old regions are dirty, large region
					m_marbleWidget->update();
					break;
				
				}
			}
            if ( e->button() == Qt::LeftButton ) {
                m_movingIndex = region.index;
                m_dropStopOver = QPoint();
                m_dragStopOver = QPoint();
                return true;
            } else if ( e->button() == Qt::RightButton ) {
				m_bObjId = false;
                m_removeViaPointAction->setEnabled( true );
				m_insertViaPointAction->setEnabled( true );
                m_activeMenuIndex = region.index;
                showContextMenu( e->pos() );
                return true;
            } else
                return false;
        }
    }

	int i=0;
	if(m_multiSelect.size()>1 && m_selIdx.isValid())
	{
		QModelIndex idx;
		QVariant qVar;
		bool bFound=false;
		for(int iX=0;iX<m_multiSelect.size();iX++)
		{
			idx = m_multiSelect.value(iX);
			if(idx.isValid())
			{
				qVar = m_treeviewModel->data(idx,TreeViewModel::TransRole);
				if(!qVar.toBool())
				{
					if(m_selIdx.internalId() == idx.internalId())
					{
						bFound=true;
					//	iX=m_multiSelect.size();
						i=iX;
					}
				}
				else
				{
					m_pNode = m_treeviewModel->getNodeFromIndex(idx);
					m_pNode->m_bMultiSelect=false;
					m_pNode->m_iXoff=0;
					m_pNode->m_iYoff=0;
				}
			}
		}
		idx = m_multiSelect.value(i);
		i=0;

		if(bFound)
		{
			foreach( const ModelRegion &region, m_placemarks ) 
			{
				if ( region.region.contains( e->pos() ) && m_selectionModel ) 
				{
					if ( e->button() == Qt::LeftButton && m_treeviewModel ) 
					{
						if(region.index.internalId() == idx.internalId())
						{
							m_selIdx = region.index;
							m_bObjId = true;
							m_movingIndex = -1;
						}
					}
					else if ( e->button() == Qt::RightButton && m_treeviewModel ) 
					{
						if(region.index.internalId() == idx.internalId())
						{
							m_selIdx = region.index;
							m_bObjId = false;
							showContextMenu( e->pos() );
							m_multiSelect.clear();
							return true;
						}
					}

				}
			}
		}
		m_multiSelect.clear();
		return true;
	}

	m_multiSelect.clear();
	long lastGroupID=0;
	foreach( const ModelRegion &region, m_placemarks ) 
	{
        if ( region.region.contains( e->pos() ) && m_selectionModel ) 
		{
            if ( e->button() == Qt::LeftButton && m_treeviewModel ) 
			{
				TreeNode* pNode = m_treeviewModel->getNodeFromIndex(region.index);
				if(pNode!=NULL && lastGroupID != pNode->parent->m_lId)
				{
					lastGroupID = pNode->parent->m_lId;
					m_multiSelect.insert(i,region.index);
					i++;
					if(i>15)
					{
						m_bObjId = false;
						showMultiSelectMenu( e->pos()  );
						return true;
					}
				}
			}

		}

	}

	if(i>1)
	{
		m_bObjId = false;
		showMultiSelectMenu( e->pos()  );
		return true;
	}
	else m_multiSelect.clear();

    foreach( const ModelRegion &region, m_placemarks ) 
	{
        if ( region.region.contains( e->pos() ) && m_selectionModel ) 
		{
            if ( e->button() == Qt::LeftButton && m_treeviewModel ) 
			{
				bool sel=true;
                QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;

				if(m_selIdx.isValid() && m_selectionModel->isSelected( m_selIdx ) ) {
                    command = QItemSelectionModel::Clear;
					//sel=false;
					if(m_bGeoProjected)
						emit q->placemarkSelected( m_selIdx,q->getInterfaceID(),false);
					if(m_selIdx.internalId() == region.index.internalId() )
						sel=false;
                }
                else if ( m_selectionModel->isSelected( region.index ) ) {
                    command = QItemSelectionModel::Clear;
					sel=false;
                }

				m_selIdx = region.index;
				m_bObjId = true;
				m_movingIndex = -1;
                m_selectionModel->select( region.index, command );
                m_dropStopOver = e->pos();
                storeDragPosition( e->pos() );
				if(m_bGeoProjected)
					emit q->placemarkSelected( region.index,q->getInterfaceID(),sel);
                // annotation and old annotation are dirty, large region
                m_marbleWidget->update();
                return true;
            } else if ( e->button() == Qt::RightButton ) {
                m_removeViaPointAction->setEnabled( false );
				m_insertViaPointAction->setEnabled( false );
				m_deleteObjectAction->setEnabled( true );
				m_selIdx = region.index;
				m_bObjId = false;
                showContextMenu( e->pos() );
                return true;
            } else
                return false;
        }//if
    }//for each

	foreach( const ModelRegion &region, m_selpoint ) 
	{
		if ( region.region.contains( e->pos() ) ) 
		{
			bool sel=true;
			QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
			if ( m_selectionModel->isSelected( region.index ) ) {
				command = QItemSelectionModel::Clear;
				sel=false;
			}
			m_selIdx = region.index;
			m_bObjId = true;
			m_selectionModel->select( region.index, command );
			if(m_bGeoProjected)
				emit q->placemarkSelected( region.index,q->getInterfaceID(),sel);
			return true;//if returned false, object moves after mouse button release
		}
	}


	//clicke over poly line
	foreach( const ModelRegion &region, m_routeRegion ) 
	{
        if ( region.region.contains( e->pos() ) && m_selectionModel ) 
		{

			foreach( const ModelRegion &regions, m_selpoint ) 
			 {
				if(regions.region.intersects(region.region))
			    //if ( region.region.contains( e->pos() )
				{
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
					if ( m_selectionModel->isSelected( region.index ) ) {
						command = QItemSelectionModel::Clear;
						sel=false;
					}
					m_selectionModel->select( region.index, command );

					if(m_bGeoProjected)
						emit q->polygoneSelected( region.index,q->getInterfaceID(),sel );
					/*qreal lon( 0.0 ), lat( 0.0 );
					if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
										lon, lat, GeoDataCoordinates::Radian ) ) 
					{
						emit q->pointSelected( GeoDataCoordinates( lon, lat ) );
					}*/
					m_selIdx = region.index;
					m_bObjId = true;
					m_bIsPoly= true;
					 // old regions are dirty, large region
					m_marbleWidget->update();
					break;
				}
			}

			if(!m_bObjId && !m_bIsPoly)
			{
				m_pNode = m_treeviewModel->getNodeFromIndex(region.index);
				if(m_pNode->ItemType == Plot || m_pNode->ItemType == PlotEx)
				{
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
					if ( m_selectionModel->isSelected( region.index ) ) {
						command = QItemSelectionModel::Clear;
						sel=false;
					}
					m_selectionModel->select( region.index, command );

					if(m_bGeoProjected)
						emit q->polygoneSelected( region.index,q->getInterfaceID(),sel );

					m_selIdx = region.index;
					m_bObjId = true;
					m_bIsPoly= true;
				}
				else m_pNode=NULL;
			}

			if ( e->button() == Qt::LeftButton ) {
				/** @todo: Determine the neighbored via points and insert in order */
				m_dropStopOver = e->pos();
				storeDragPosition( e->pos() );
				return true;
			} else if ( e->button() == Qt::RightButton ) {
				m_bObjId = false;
				m_removeViaPointAction->setEnabled( false );
				m_insertViaPointAction->setEnabled( true );
				m_deleteObjectAction->setEnabled( true );
				showContextMenu( e->pos() );
				return true;
			} else
				return false;
		}
    }

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }
	

 /*   foreach( const RequestRegion &region, m_alternativeRouteRegions ) {
        if ( region.region.contains( e->pos() ) ) {
            m_alternativeRoutesView->setCurrentIndex( region.index );
            return true;
        }
    }*/

	if (m_selectionModel->isSelected( m_selIdx) ) 
	{
		m_bIsPoly=false;
		m_bObjId =false;
        QItemSelectionModel::SelectionFlag command = QItemSelectionModel::Clear;
		m_selectionModel->select( m_selIdx, command );

		if(m_bGeoProjected)
		{
			if(m_movingIndex<0)
				emit q->placemarkSelected( m_selIdx,q->getInterfaceID(),false);
			else if(m_movingIndex>=0)
				emit q->polygoneSelected( m_selIdx,q->getInterfaceID(),false);
		}
    }

    return false;
}

/*
bool MapViewTSLayerPrivate::handleMouseButtonPress( QMouseEvent *e )
{
    if ( m_pointSelection ) {
        return e->button() == Qt::LeftButton;
    }

	
    foreach( const RequestRegion &region, m_regions ) {
        if ( region.region.contains( e->pos() ) ) {
            if ( e->button() == Qt::LeftButton ) {
                m_movingIndex = region.index;
                m_dropStopOver = QPoint();
                m_dragStopOver = QPoint();
                return true;
            } else if ( e->button() == Qt::RightButton ) {
                m_removeViaPointAction->setEnabled( true );
                m_activeMenuIndex = region.index;
                showContextMenu( e->pos() );
                return true;
            } else
                return false;
        }
    }

	
    foreach( const ModelRegion &region, m_placemarks ) {
        if ( region.region.contains( e->pos() ) ) {
            if ( e->button() == Qt::LeftButton ) {
                QModelIndex index = region.index;//m_proxyModel->mapFromSource( region.index );
                QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
                if ( m_selectionModel->isSelected( index ) ) {
                    command = QItemSelectionModel::Clear;
                }
                m_selectionModel->select( index, command );
                m_dropStopOver = e->pos();
                storeDragPosition( e->pos() );

				m_movingIndex=2;
				
               // m_dropStopOver = QPoint();
               // m_dragStopOver = QPoint();

                // annotation and old annotation are dirty, large region
                m_marbleWidget->repaint();
                return true;
            } else if ( e->button() == Qt::RightButton ) {
                m_removeViaPointAction->setEnabled( false );
                showContextMenu( e->pos() );
                return true;
            } else
                return false;
        }
    }

	if ( m_layerRegion.contains( e->pos() ) ) {
        if ( e->button() == Qt::LeftButton ) { 
            
         //   m_dropStopOver = e->pos();
         //   storeDragPosition( e->pos() );
            return false;
        } else if ( e->button() == Qt::RightButton ) {
            m_removeViaPointAction->setEnabled( false );
            showContextMenu( e->pos() );
            return true;
        } else
            return false;
    }

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

    foreach( const ModelRegion &region, m_placemarks ) {
        if ( region.region.contains( e->pos() ) ) {
            emit q->placemarkSelected( region.index );
            return true;
        }
    }

    return false;
}*/

bool MapViewTSLayerPrivate::handleMouseButtonRelease( QMouseEvent *e )
{
	if(!m_update)
		return false;
	if(m_bInputEnabled)
		return false;

	m_mousePos.setX(-1);
	m_mousePos.setY(-1);

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

    if ( m_pointSelection ) 
	{
        if ( e->button() == Qt::LeftButton ) {
            qreal lon( 0.0 ), lat( 0.0 );
            if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                                 lon, lat, GeoDataCoordinates::Radian ) ) 
			{
				if(m_bGeoProjected)
					emit q->pointSelected( GeoDataCoordinates( lon, lat ) );
                return true;
            }
        } else {
            return false;
        }
    }

	if(m_bObjId && m_movingIndex<0)
	{
		if(m_movingID==2)
		{
			m_bIsPoly=false;
			emit q->placemarkMoved(m_selIdx,q->getInterfaceID());
		}

		m_bObjId=false;
		m_movingID = 0;
		m_movingIndex = -1;
		m_pNode=NULL;
        clearStopOver();
    //    emit q->layerDirty();
        return true;
    }
    else if ( m_movingIndex >= 0 ) 
	{
		if(m_movingID==1)
			emit q->polygoneMoved(m_selIdx,q->getInterfaceID());
		m_bIsPoly=false;
		m_bObjId=false;
		m_movingID = 0;
        m_movingIndex = -1;
		m_pNode=NULL;
        clearStopOver();
    //    emit q->layerDirty();
        return true;
    }
/*	else if(m_selIdx.isValid()) //if(m_bObjId )//|| m_movingIndex >=0)//deselect
	{
        QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
        if ( m_selectionModel->isSelected( m_selIdx) ) 
		{
			m_bIsPoly=false;
            command = QItemSelectionModel::Clear;
			m_selectionModel->select( m_selIdx, command );
        }

		//m_selIdx = region.index;
		//m_bObjId = false;
		//m_movingIndex = -1;
	}*/


    if ( !m_dropStopOver.isNull() && !m_dragStopOver.isNull() ) {
        QPoint moved = e->pos() - m_dragStopOver;
        if ( moved.manhattanLength() < 10 ) {
            return false;
        }

		QModelIndex idxParent = m_treeviewModel->getIndexFromIndexId(m_indexID);
		int iRowCount = m_treeviewModel->rowCount(idxParent);
        qreal lon( 0.0 ), lat( 0.0 );
		
        if ( m_dragStopOverRightIndex >= 0 && m_dragStopOverRightIndex <= iRowCount
                && m_marbleWidget->geoCoordinates( m_dropStopOver.x(), m_dropStopOver.y(), lon, lat, GeoDataCoordinates::Radian ) ) {
            GeoDataCoordinates position( lon, lat );
            m_dragStopOverRightIndex = viaInsertPosition( e->modifiers() );
//            m_routeSkeleton->insert( m_dragStopOverRightIndex, position );
            clearStopOver();
       //     emit q->layerDirty();
            return true;
        }
    }

    return false;
}

bool MapViewTSLayerPrivate::handleMouseDblClick( QMouseEvent *e )
{
	if(!m_update)
		return false;
	if(m_bInputEnabled)
		return false;

	QPoint pt = e->pos();
	//m_mousePos.setX(-1);
	//m_mousePos.setY(-1);

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

	foreach( const ModelRegion &region, m_placemarks ) 
	{
        if ( region.region.contains( e->pos() ) && m_selectionModel ) 
		{
            if ( e->button() == Qt::LeftButton && m_treeviewModel ) 
			{
				bool sel=true;
                QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;

				if(m_selIdx.isValid() && m_selectionModel->isSelected( m_selIdx ) ) {
                    command = QItemSelectionModel::Clear;
					//sel=false;
					if(m_bGeoProjected)
						emit q->placemarkDblClick( m_selIdx,q->getInterfaceID(),pt,false);
					if(m_selIdx.internalId() == region.index.internalId() )
						sel=false;
                }
                else if ( m_selectionModel->isSelected( region.index ) ) {
                    command = QItemSelectionModel::Clear;
					sel=false;
                }

				m_selIdx = region.index;
				m_bObjId = true;
				m_movingIndex = -1;
                m_selectionModel->select( region.index, command );
                m_dropStopOver = e->pos();
                storeDragPosition( e->pos() );
				if(m_bGeoProjected)
					emit q->placemarkDblClick( region.index,q->getInterfaceID(),pt,sel);
                // annotation and old annotation are dirty, large region
                m_marbleWidget->update();
                return true;
            } 
			else
                return false;
        }//if
    }//for each

	//clicke over poly line
	foreach( const ModelRegion &region, m_routeRegion ) 
	{
        if ( region.region.contains( e->pos() ) && m_selectionModel ) 
		{

			foreach( const ModelRegion &regions, m_selpoint ) 
			 {
				if(regions.region.intersects(region.region))
			    //if ( region.region.contains( e->pos() )
				{
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
					if ( m_selectionModel->isSelected( region.index ) ) {
						command = QItemSelectionModel::Clear;
						sel=false;
					}
					m_selectionModel->select( region.index, command );

					if(m_bGeoProjected)
						emit q->polygoneDblClick( region.index,q->getInterfaceID(),pt,sel );
					/*qreal lon( 0.0 ), lat( 0.0 );
					if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
										lon, lat, GeoDataCoordinates::Radian ) ) 
					{
						emit q->pointSelected( GeoDataCoordinates( lon, lat ) );
					}*/
					m_selIdx = region.index;
					m_bObjId = true;
					m_bIsPoly= true;
					 // old regions are dirty, large region
					m_marbleWidget->update();
					break;
				}
			}

			if(!m_bObjId && !m_bIsPoly)
			{
				m_pNode = m_treeviewModel->getNodeFromIndex(region.index);
				if(m_pNode->ItemType == Plot || m_pNode->ItemType == PlotEx)
				{
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;
					if ( m_selectionModel->isSelected( region.index ) ) {
						command = QItemSelectionModel::Clear;
						sel=false;
					}
					m_selectionModel->select( region.index, command );

					if(m_bGeoProjected)
						emit q->polygoneDblClick( region.index,q->getInterfaceID(),pt,sel );

					m_selIdx = region.index;
					m_bObjId = true;
					m_bIsPoly= true;
				}
				else m_pNode=NULL;
			}

			if ( e->button() == Qt::LeftButton ) {
				/** @todo: Determine the neighbored via points and insert in order */
				m_dropStopOver = e->pos();
				storeDragPosition( e->pos() );
				return true;
			} else
				return false;
		}
    }

    if ( e->button() != Qt::LeftButton ) {
        return false;
    }

	return false;

}

bool MapViewTSLayerPrivate::handleMouseMove( QMouseEvent *e )
{
	if(!m_update)
		return false;
	if(m_bInputEnabled)
		return false;

    if ( m_pointSelection ) {
        m_marbleWidget->setCursor( Qt::CrossCursor );
        return true;
    }

    if ( !m_treeviewModel ) {
        return false;
    }

	//m_infoID=0;

    qreal lon( 0.0 ), lat( 0.0 );
    if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                         lon, lat, GeoDataCoordinates::Radian ) ) 
	{

	/*	QRegion rg;
		if(m_routeRegion.size()>0)
		{
			foreach( const ModelRegion &region, m_routeRegion ) 
			{
				if ( region.region.contains( e->pos() )  )
				{
					rg = region.region;
					break;
				}
			}
		}*/

        if ( (m_movingIndex >= 0 || m_bIsPoly) && (m_bEdit && m_bMoveObj) ) //polygone, rect, ..
		{
			m_viewportChanged=true;
			m_infoID=0;
            GeoDataCoordinates moved( lon, lat );
			const QPoint ePos = e->pos();
         //   m_polySkeleton->setPosition( m_movingIndex, moved );
			if(m_bObjId && m_selIdx.isValid())
			{
				m_pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
				TreeNode* pGroup= m_pNode->parent;
				TreeNode* pItem=NULL;
				int iGC = pGroup->children.count();
				for(int ic=0;ic<iGC;ic++)
				{
					pItem=pGroup->children.at(ic);
					moveObjects(moved,pItem,ePos);

				}

				m_mousePos.setX(e->pos().x());
				m_mousePos.setY(e->pos().y());
				m_movingID=1;
				m_marbleWidget->setCursor( Qt::ArrowCursor );
				m_marbleWidget->update();
				
			}//objId
        }//moving index > -1
		else if(m_bObjId && m_selIdx.isValid() && (m_bEdit && m_bMoveObj)&& !m_bIsPoly )//Icons
		{
			m_viewportChanged=true;
			m_infoID=0;
			GeoDataCoordinates moved( lon, lat );
			m_pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
			if(m_pNode!=NULL && m_pNode->parent!=NULL)
			{
				m_pNode->m_position.set(moved.longitude(),moved.latitude(),0);
				if(m_pNode->ItemType != GIS::Group)
					m_pNode->parent->m_position.set(moved.longitude(),moved.latitude(),0);
			}

			m_movingID=2;
			//qDebug() << "MouseMove: " << m_movingID;
			m_marbleWidget->setCursor( Qt::ArrowCursor );

		/*	QRect dirty = QRect( e->pos(), QSize( 100, 100 ) );
			dirty.setLeft(dirty.left() - 100);
			dirty.setTop(dirty.top() - 100);
			m_marbleWidget->update( dirty );*/
			m_marbleWidget->update();

		}
		else if ( !m_dragStopOver.isNull())// && !rg.isEmpty() ) 
		{
            // Repaint only that region of the map that is affected by the change
            m_dragStopOverRightIndex = viaInsertPosition( e->modifiers() );
			QRect dirty = m_polyRegion.boundingRect();
			dirty |= QRect( m_dropStopOver, m_pixmapSize );
			dirty |= QRect( e->pos(), m_pixmapSize );
			if ( e->buttons() & Qt::LeftButton ) {
				m_dropStopOver = e->pos();
			} else {
				m_dragStopOver = QPoint();
				m_dropStopOver = QPoint();
			}
			m_marbleWidget->update( dirty );
			m_marbleWidget->setCursor( Qt::ArrowCursor );
			m_infoID=0;
				
        } else if ( isInfoPoint( e->pos() ) ) {
            clearStopOver();
            m_marbleWidget->setCursor( Qt::ArrowCursor );

		//	paintStopOver(m_annoRect);// QRect( e->pos(), m_pixmapSize ) );
		//	 m_marbleWidget->repaint( m_annoRect );
			if(m_infoID)
			{
				int iW = m_annoRect.width();
				int iH = m_annoRect.height();
				if(iW>0 && iH>0)
				{
					QRect dirty = QRect( e->pos(), QSize( iW,iH));
					dirty.setLeft(dirty.left() - iW);
					dirty.setTop(dirty.bottom() - iH);
					m_marbleWidget->update( dirty );
				}
			}
        } 
		else if ( 0)//!rg.isEmpty() ) 
		{
            m_dropStopOver = e->pos();
            m_marbleWidget->setCursor( Qt::ArrowCursor );
        } else if ( !m_dropStopOver.isNull() ) 
		{
			m_infoID=0;
            clearStopOver();
        } 
        else {
			//enz m_marbleWidget->setCursor( Qt::OpenHandCursor ); 
			if(m_infoID)
			{
				m_infoID=0;
				int iW = m_annoRect.width();
				int iH = m_annoRect.height();
				if(iW>0 && iH>0)
				{
					QRect dirty = QRect(m_posInfo, QSize(iW,iH));
					dirty.setLeft(dirty.left() - iW);
					dirty.setTop(dirty.top() - iH);
					m_marbleWidget->update( dirty );
				}
			}
			
            return false;
        }

        // Update pixmap in the map (old and new position needs repaint)
		//if ( m_movingIndex < 0 )
	//	if(!m_infoID)
	//	paintStopOver( QRect(m_posInfo, m_pixmapSize ) );
		paintStopOver( QRect( e->pos(), m_pixmapSize ) );
	//	else paintStopOver( m_dirtyRect);
        return true;
    }

    return false;
}

int MapViewTSLayerPrivate::getIntersectionObjects(QModelIndex& idx,QPoint pt,bool flag,int pos)
{
	int count=-1;
	if(flag)
	{
		count=0;
		m_multiItemSelect.clear();
		foreach( const ModelRegion &region, m_placemarks ) 
		{
			if ( region.region.contains( pt ) ) 
			{
				m_multiItemSelect.insert(count,region.index);
				count++;
			}
		}
		return count;
	}
	else
	{
		if(pos < m_multiItemSelect.size() )
		{
			count = pos;
			idx = m_multiItemSelect.value(pos);
		}
		return count;
	}

}

int MapViewTSLayerPrivate::getIntersectionRegionObjects(QModelIndex& idx,const QRect& rect,bool flag,int pos)
{
	int count=0;
	m_multiItemSelect.clear();
//	QRegion reg;
//	reg.setRects(&rect,1);
	QRect boundingRect;
	foreach( const ModelRegion &region, m_placemarks ) 
	{
		//if ( region.region.contains( pt ) ) 
		boundingRect = region.region.boundingRect();
		if(rect.contains(boundingRect))
		{
			m_multiItemSelect.insert(count,region.index);
			count++;
		}
	}

	return count;
}

bool MapViewTSLayerPrivate::calcEllipseCoords(qreal longitude, qreal latitude, qreal minor, qreal major,GeoDataLineString& ellipse)
{
	// Initialize variables
	qreal centerLon = longitude;
	qreal centerLat = latitude;
	qreal altitude = 0.0;//centerPosition.altitude();
	//centerPosition.geoCoordinates( centerLon, centerLat,
	//							   GeoDataCoordinates::Degree );

	qreal width = minor; 
	qreal height= major;

//					qreal M_PI    = 3.14159265358979323846;
	qreal DEG2RAD = M_PI / 180.0;
	qreal RAD2DEG = 180.0 / M_PI;

	// Ensure a valid latitude range: 
	if ( centerLat + 0.5 * height > 90.0 || centerLat - 0.5 * height < -90.0 ) {
		return false;
	}

	// Don't show the ellipse if it's too small:
	GeoDataLatLonBox ellipseBox( centerLat + 0.5 * height, centerLat - 0.5 * height,
								 centerLon + 0.5 * width,  centerLon - 0.5 * width, 
								 GeoDataCoordinates::Degree );
	
	if ( !m_marbleWidget->viewport()->viewLatLonAltBox().intersects( ellipseBox ) ||
		 !m_marbleWidget->viewport()->resolves( ellipseBox ) ) return false;

	qreal lon = 0.0;
	qreal lat = 0.0;

	// Optimizing the precision by determining the size which the 
	// ellipse covers on the screen:
	qreal degreeResolution = m_marbleWidget->viewport()->angularResolution() * RAD2DEG;
	// To create a circle shape even for very small precision we require uneven numbers:
	int precision = width / degreeResolution / 8 + 1; 
	if ( precision > 81 ) precision = 81;
	precision = 180;

	// Calculate the shape of the upper half of the ellipse:
	for ( int i = 0; i < precision; ++i ) {
		qreal t = 1.0 - 2.0 * (qreal)(i) / (qreal)(precision);
		lat = centerLat + 0.5 * height * sqrt( 1.0 - t * t );
		lon = centerLon + 0.5 * width * t;
		ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
	}
	// Calculate the shape of the lower half of the ellipse:
	for ( int i = 0; i < precision; ++i ) {
		qreal t = 2.0 * (qreal)(i) / (qreal)(precision) -  1.0;
		lat = centerLat - 0.5 * height * sqrt( 1.0 - t * t );
		lon = centerLon + 0.5 * width * t;
		ellipse << GeoDataCoordinates( lon, lat, altitude, GeoDataCoordinates::Degree );
	}

	return true;
}

bool MapViewTSLayerPrivate::calcPlotCoords(const QModelIndex& idx,const GeoDataCoordinates& src,const GeoDataCoordinates& dest)
{
	if(idx.isValid())
	{
		Model::TreeNode* pNode = m_treeviewModel->getNodeFromIndex(idx);
		if(pNode!=NULL && pNode->parent!=NULL)
		{
			if(pNode->ItemType==GIS::Plot||pNode->ItemType==GIS::PlotEx)
			{
				GeoDataLineString gdl;
				gdl << pNode->m_poly;
				pNode->m_poly.clear();
				
				//move the polygone
				GeoDataCoordinates pos;
				qreal mLon = src.longitude();
				qreal mLat = src.latitude();
				
				//pos.set(mLon,mLat,0);//mouse pos
				qreal dLon = mLon-dest.longitude();
				qreal dLat = mLat-dest.latitude();
				for(int i=0;i<gdl.size();i++)
				{
					pos.set(gdl.at(i).longitude()-dLon,gdl.at(i).latitude()-dLat,0);
					pNode->m_poly.append(pos);
				}

				pNode->m_position.set(dest.longitude(),dest.latitude(),0);

			}//poly
		}//pNode
	}//index

	return true;
}
/*
bool MapViewTSLayerPrivate::handleMouseMove( QMouseEvent *e )
{
    if ( m_pointSelection ) {
        m_marbleWidget->setCursor( Qt::CrossCursor );
        return true;
    }

    if ( !m_treeviewModel ) {
        return false;
    }


	bool bDel=true;
    qreal lon( 0.0 ), lat( 0.0 );

	
    if ( m_marbleWidget->geoCoordinates( e->pos().x(), e->pos().y(),
                                         lon, lat, GeoDataCoordinates::Radian ) ) {

        if ( m_movingIndex == 2 ) 
		{
            GeoDataCoordinates moved( lon, lat );
      //      m_routeSkeleton->setPosition( m_movingIndex, moved );
			QModelIndex idx;// = m_selectionModel->currentIndex();
			QRect dirty;

			foreach( const ModelRegion &region, m_placemarks ) 
			{
				if ( region.region.contains( e->pos() ) ) 
				{
					idx = region.index;
					dirty = region.region.boundingRect();
				//	m_layerRegion = region.region;
					break;
				}
			}

			if(!idx.isValid())
				return false;

			m_pNode = m_treeviewModel->getNodeFromIndex(idx);
			GeoDataCoordinates position = m_pNode->m_position;//.at(0); //.set(lon,lat,0,GeoDataCoordinates::Degree);

			if(m_pNode->ItemType != GIS::Rect)
			{
				//position.setLongitude(lon,GeoDataCoordinates::Radian);
				//position.setLatitude(lat,GeoDataCoordinates::Radian);
//				m_pNode->m_position.clear();
				m_pNode->m_position.set(moved.longitude(),moved.latitude(),0);

				dirty |= QRect( e->pos(), QSize( 100, 30 ) );
				dirty.setLeft(dirty.left() - 10);
				dirty.setTop(dirty.top() - 10);

			}
			else
			{
				GeoDataCoordinates lt = m_pNode->m_poly.at(0);
				GeoDataCoordinates rb = m_pNode->m_poly.at(1);
				m_pNode->m_poly.clear();

				qreal mLon = position.longitude()-moved.longitude();
				qreal mLat = position.latitude()-moved.latitude();

				if(position.longitude()>moved.longitude())
				{
					mLon = position.longitude()-moved.longitude();
					lt.setLongitude(lt.longitude()-mLon);
					rb.setLongitude(rb.longitude()-mLon);
				}
				else 
				{
					mLon = moved.longitude()-position.longitude();
					lt.setLongitude(lt.longitude()+mLon);
					rb.setLongitude(rb.longitude()+mLon);
				}

				if(position.latitude()>moved.latitude())
				{
					mLat = position.latitude()-moved.latitude();
					lt.setLatitude(lt.latitude()-mLat);
					rb.setLatitude(rb.latitude()-mLat);
				}
				else 
				{
					mLat = moved.latitude()-position.latitude();
					lt.setLatitude(lt.latitude()+mLat);
					rb.setLatitude(rb.latitude()+mLat);
				}

				m_pNode->m_poly.append(lt);
				m_pNode->m_poly.append(rb);

//				m_pNode->m_position.clear();
				m_pNode->m_position.set(moved.longitude(),moved.latitude(),0);

				dirty = m_layerRegion.boundingRect();
				dirty.setLeft(dirty.left() - 10);
				dirty.setTop(dirty.top() - 10);
				dirty.setRight(dirty.right() + 10);
				dirty.setBottom(dirty.bottom() + 10);
				bDel=false;
			}
		
            //dirty |= QRect( m_dropStopOver, QSize( 30, 30 ) );
			
            
            if ( e->buttons() & Qt::LeftButton ) {
                m_dropStopOver = e->pos();
            } else {
                m_dragStopOver = QPoint();
                m_dropStopOver = QPoint();
            }
            m_marbleWidget->repaint( dirty );
            m_marbleWidget->setCursor( Qt::ArrowCursor );
		//	m_movingIndex = 3;
        } 
		else if ( !m_dragStopOver.isNull()&& m_movingIndex == 3 ) 
		{
            // Repaint only that region of the map that is affected by the change
            //m_dragStopOverRightIndex = viaInsertPosition( e->modifiers() );

			GeoDataCoordinates position = m_pNode->m_position;//.at(0); //.set(lon,lat,0,GeoDataCoordinates::Degree);
			position.setLongitude(lon,GeoDataCoordinates::Radian);
			position.setLatitude(lat,GeoDataCoordinates::Radian);
//			m_pNode->m_position.clear();
			m_pNode->m_position.set(position.longitude(),position.latitude(),0);

            QRect dirty = m_layerRegion.boundingRect(); 
            dirty |= QRect( m_dropStopOver, QSize( 30, 30 ) );
			dirty.setLeft(dirty.left() - 10);
			dirty.setTop(dirty.top() - 10);
            //dirty |= QRect( e->pos(), QSize( 30, 30 ) );
            if ( e->buttons() & Qt::LeftButton ) {
                m_dropStopOver = e->pos();
            } else {
                m_dragStopOver = QPoint();
                m_dropStopOver = QPoint();
            }
            m_marbleWidget->repaint( dirty );
            m_marbleWidget->setCursor( Qt::ArrowCursor );
        } 
		else if ( isInfoPoint( e->pos() ) ) {
            clearStopOver();
			m_movingIndex=1;
            m_marbleWidget->setCursor( Qt::ArrowCursor );
        } 
		else if ( m_layerRegion.contains( e->pos() ) ) {
            m_dropStopOver = e->pos();
			m_movingIndex=-1;
		//	return false;
            m_marbleWidget->setCursor( Qt::OpenHandCursor );
        } 
		else if ( m_polyRegion.contains( e->pos() ) ) {
            clearStopOver();
			m_movingIndex=1;
		//	return false;
            m_marbleWidget->setCursor( Qt::ArrowCursor );
        } 
		else if ( !m_dropStopOver.isNull() ) {
            clearStopOver();
        } 
        else {
			m_movingIndex=-1;
            return false;
        }

        // Update pixmap in the map (old and new position needs repaint)
		if(bDel)
			paintStopOver( QRect( e->pos(), m_pixmapSize ) );
        return true;
    }

    return false;
}*/

bool MapViewTSLayerPrivate::handleKeyEvent( QKeyEvent *e )
{
    if ( m_pointSelection && e->key() == Qt::Key_Escape ) {
        m_pointSelection = false;
        emit q->pointSelectionAborted();
        return true;
    }

	if(m_selIdx.isValid() && (m_bEdit && m_bMoveObj) )//Icons
	{
		if ( e->key() == Qt::Key_Right ) 
		{
			if(m_rotation<0)m_rotation=0;
			++m_rotation;
			return rotatePoly(1.0);
		}
		else if ( e->key() == Qt::Key_Left ) 
		{	
			if(m_rotation>0)m_rotation=0;
			--m_rotation;
			return rotatePoly(-1.0);
		}
		else if ( e->key() == Qt::Key_Up ) 
		{
			if(m_rotation<0)m_rotation=0;
			++m_rotation;
			return resizePoly(1.0);
		}
		else if ( e->key() == Qt::Key_Down ) 
		{	
			if(m_rotation>0)m_rotation=0;
			--m_rotation;
			return resizePoly(-1.0);
		}
	}
	

    return false;
}

bool MapViewTSLayerPrivate::rotatePoly(qreal angle)
{
	m_pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
	if(m_pNode==NULL)return false;

	TreeNode* pGroup = m_pNode->parent;
	int iGroupCount = pGroup->children.count();
	TreeNode* pItem = NULL;

	int iSize = m_pNode->m_poly.size();
	GeoDataLineString gdl;
	GeoDataCoordinates cor;
	GeoDataCoordinates center;
	GeoDataLatLonBox box;

	qreal lon=0;
	qreal lat=0;
	qreal xpos=0;
	qreal ypos=0;
	qreal arc=0;
	qreal cosa = 0;
	qreal sina = 0;

	qreal left=180.0;
	qreal right=-180.0;
	qreal top=-90.0;
	qreal bot=90.0;

	//if(iSize<2)return false;
	int iVia=0;

	for(int item=0;item<iGroupCount;item++)
	{
		pItem = pGroup->children.at(item);
		if(pItem==NULL)continue;
		iSize = pItem->m_poly.size();
		if(iSize<2)continue;

		//turn over a specific via point
		iVia = pItem->m_qsName.toInt();
		if(iVia==1010)
			center = pItem->m_poly.at(0);
		else if(iVia==1011 || iVia==1012)
		{
			if(pItem->ItemType==Polygone)
			{
				box = pItem->m_poly.latLonAltBox();
				center = box.center();
			}
		}
		else center = pItem->m_poly.at(1);

		right = center.longitude();//(GeoDataCoordinates::Degree);
		bot = center.latitude();//(GeoDataCoordinates::Degree);

		arc = angle * PI/180.0;
		cosa = cos(arc);
		sina = sin(arc);

		for(int i=0;i<iSize;i++)
		{
			cor = pItem->m_poly.at(i);
			xpos = cor.longitude();//GeoDataCoordinates::Degree);
			ypos = cor.latitude();//GeoDataCoordinates::Degree);
			
			lon = ((xpos-right) * cosa) - ((ypos-bot) * sina);
			lat = ((xpos-right) * sina) + ((ypos-bot) * cosa);
			xpos = lon+right;
			ypos = lat+bot;

			cor.set(xpos,ypos,0);//,GeoDataCoordinates::Degree);
			gdl << cor;
		}
		pItem->m_poly.clear();
		pItem->m_poly << gdl;
		gdl.clear();

	}//for item

	m_bObjId=false;
	m_movingID = 0;
    m_movingIndex = -1;
	m_pNode=NULL;
    clearStopOver();

	m_viewportChanged=true;
	m_marbleWidget->update();
	return true;
}

bool MapViewTSLayerPrivate::resizePoly(qreal angle)
{
	m_pNode = m_treeviewModel->getNodeFromIndex(m_selIdx);
	if(m_pNode==NULL)return false;

	TreeNode* pGroup = m_pNode->parent;
	int iGroupCount = pGroup->children.count();
	TreeNode* pItem = NULL;

	int iSize = m_pNode->m_poly.size();
	GeoDataLineString gdl;
	GeoDataCoordinates cor;
	GeoDataCoordinates center;
	GeoDataLatLonBox box;
	qreal lon=0;
	qreal lat=0;
	qreal xpos=0;
	qreal ypos=0;
	qreal arc=0;
	qreal cosa = 0;
	qreal sina = 0;

	qreal left=180.0;
	qreal right=-180.0;
	qreal top=-90.0;
	qreal bot=90.0;

//	if(iSize<2)return false;
	int iVia=0;
	for(int item=0;item<iGroupCount;item++)
	{
		pItem = pGroup->children.at(item);
		if(pItem==NULL)continue;
		iSize = pItem->m_poly.size();
		if(iSize<2)continue;

		iVia = pItem->m_qsName.toInt();
		if(iVia==1010)
		{
			box = pItem->m_poly.latLonAltBox();
			center = box.center();
		}
		else if(iVia==1011 || iVia==1012)
		{
			if(pItem->ItemType==Polygone)
			{
				box = pItem->m_poly.latLonAltBox();
				center = box.center();
			}
		}
		else if(iVia==1013)
		{
			//box = pItem->m_poly.latLonAltBox();
			center = pItem->m_poly.at(1);;//box.center();
		}
		else
		{
			box = pItem->m_poly.latLonAltBox();
			center = box.center();
		}

		left = center.longitude(GeoDataCoordinates::Degree);
		top = center.latitude(GeoDataCoordinates::Degree);

		m_marbleWidget->screenCoordinates(left,top,right,bot);

		for(int i=0;i<iSize;i++)
		{
			cor = pItem->m_poly.at(i);
			lon = cor.longitude(GeoDataCoordinates::Degree);
			lat = cor.latitude(GeoDataCoordinates::Degree);
			m_marbleWidget->screenCoordinates(lon,lat,xpos,ypos);

			arc = 0 * PI/180.0;
			
			//resize
			if(angle==1.0)
			{
				cosa = acos(arc);
				sina = asin(arc);
			}
			else
			{
				cosa = acos(arc)/2;
				sina = asin(arc)/2;
			}

			lon = ((xpos-right) * cosa) - ((ypos-bot) * sina);
			lat = ((xpos-right) * sina) + ((ypos-bot) * cosa);
			xpos = lon+right;
			ypos = lat+bot;

			m_marbleWidget->geoCoordinates(xpos,ypos,lon,lat);
			cor.set(lon,lat,0,GeoDataCoordinates::Degree);
			gdl << cor;
		}
		pItem->m_poly.clear();
		pItem->m_poly << gdl;
		gdl.clear();

	}//for item

	m_bObjId=false;
	m_movingID = 0;
    m_movingIndex = -1;
	m_pNode=NULL;
    clearStopOver();
	m_viewportChanged=true;

	m_marbleWidget->update();
	return true;
}

bool MapViewTSLayerPrivate::moveObjects(GeoDataCoordinates& moved,TreeNode* pNode,const QPoint& pt)
{
	if(pNode==NULL)return false;

		if(pNode->parent!=NULL)
		{
			if(pNode->ItemType!=GIS::Rect&&pNode->ItemType!=GIS::Elipse)
			{
				GeoDataLineString gdl;
				gdl << pNode->m_poly;
				pNode->m_poly.clear();
				
				if(0)//m_movingIndex>=0 && m_movingIndex<gdl.size())
				{
					for(int i=0;i<gdl.size();i++)
					{
						if(i!=m_movingIndex)
							pNode->m_poly.append(gdl.at(i));
						else pNode->m_poly.append(moved);
					}
				}
				else //move the polygone
				{
					GeoDataCoordinates pos;
					qreal mLon = 0;
					qreal mLat = 0;
					m_marbleWidget->geoCoordinates(m_mousePos.rx(),m_mousePos.ry(),
                                         mLon, mLat, GeoDataCoordinates::Radian );
					//pos.set(mLon,mLat,0);//mouse pos
					qreal dLon = mLon-moved.longitude();
					qreal dLat = mLat-moved.latitude();
					for(int i=0;i<gdl.size();i++)
					{
						pos.set(gdl.at(i).longitude()-dLon,gdl.at(i).latitude()-dLat,0);
						pNode->m_poly.append(pos);
					}

					//m_mousePos.setX(pt.x());
					//m_mousePos.setY(pt.y());

					
					if(pNode->ItemType == Plot || pNode->ItemType == PlotEx)
					{
						//pos = m_pNode->parent->m_position;
						pos.set(pNode->parent->m_position.longitude()-dLon,pNode->parent->m_position.latitude()-dLat,0);
						pNode->m_position.set(pos.longitude(),pos.latitude(),0);
						pNode->parent->m_position.set(pos.longitude(),pos.latitude(),0);
					}
					else
					{
						pNode->m_position.set(moved.longitude(),moved.latitude(),0);
						pNode->parent->m_position.set(moved.longitude(),moved.latitude(),0);
					}

					m_movingIndex=gdl.size()+1;//set to 5 to correct handle mouse release
				

				}
			}
			else if(pNode->ItemType==GIS::Elipse)
			{
				GeoDataCoordinates pos1;// = m_pNode->parent->m_position;
				if(0)//m_movingIndex>=0)
				{
					pos1 = pNode->m_poly.at(m_movingIndex);
				}
				else pos1 = pNode->parent->m_position;

				GeoDataCoordinates pos2;// = moved;

				double qWidth  = moved.longitude(GeoDataCoordinates::Degree)-pos1.longitude(GeoDataCoordinates::Degree);
				double qHeigth = pos1.latitude(GeoDataCoordinates::Degree)-moved.latitude(GeoDataCoordinates::Degree);

				if(m_movingIndex==0 || m_movingIndex==180)//right/left
				{
					pos2 = pNode->m_poly.at(90);
					qHeigth = pos1.latitude(GeoDataCoordinates::Degree)-pos2.latitude(GeoDataCoordinates::Degree);
				}
				if(m_movingIndex==90 || m_movingIndex==270)//top/bottom
				{
					pos2 = pNode->m_poly.at(0);
					qWidth  = pos2.longitude(GeoDataCoordinates::Degree)-pos1.longitude(GeoDataCoordinates::Degree);
				}
				if(m_movingIndex==5)//center point selected
				{
					pos2 = pNode->m_poly.at(90);
					qHeigth = pos1.latitude(GeoDataCoordinates::Degree)-pos2.latitude(GeoDataCoordinates::Degree);
					pos2 = pNode->m_poly.at(0);
					qWidth  = pos2.longitude(GeoDataCoordinates::Degree)-pos1.longitude(GeoDataCoordinates::Degree);
				
					pos1 = moved;
					pNode->m_position = pos1;
					pNode->parent->m_position = pos1;
				}

				if(m_movingIndex<0)m_movingIndex=1;

				if(qWidth<0)qWidth*=-1;
				if(qHeigth<0)qHeigth*=-1;

				//radius * 2
				qWidth*=2;
				qHeigth*=2;

				qreal lat  = pos1.latitude(GeoDataCoordinates::Degree);
				qreal lon  = pos1.longitude(GeoDataCoordinates::Degree);

				GeoDataLineString gdl;
				if(calcEllipseCoords(lon,lat, qWidth, qHeigth,gdl))
				{
					pNode->m_poly.clear();
					pNode->m_poly << gdl;
				}

			}
			else //rect
			{
				GeoDataCoordinates pos;
				
				if(m_movingIndex==0)
				{
					pos = m_pNode->m_poly.at(1);
					pNode->m_poly.clear();
					pNode->m_poly.append(moved);
					pNode->m_poly.append(pos);

					pNode->m_position.set(moved.longitude()+( (pos.longitude()-moved.longitude())/2),moved.latitude()-( (moved.latitude()-pos.latitude())/2),0);
					pNode->parent->m_position.set(moved.longitude()+( (pos.longitude()-moved.longitude())/2),moved.latitude()-( (moved.latitude()-pos.latitude())/2),0);
				}//else moving index 0
				else if(m_movingIndex==1)
				{
					pos = pNode->m_poly.at(0);
					pNode->m_poly.clear();
					pNode->m_poly.append(pos);
					pNode->m_poly.append(moved);

					pNode->m_position.set(pos.longitude()+( (moved.longitude()-pos.longitude())/2),pos.latitude()-( (pos.latitude()-moved.latitude())/2),0);
					pNode->parent->m_position.set(pos.longitude()+( (moved.longitude()-pos.longitude())/2),pos.latitude()-( (pos.latitude()-moved.latitude())/2),0);
				}//else moving index 1
				else if(m_movingIndex==2)
				{
					GeoDataCoordinates position;// = m_pNode->m_position;
					GeoDataCoordinates lt = pNode->m_poly.at(0);
					GeoDataCoordinates rb = pNode->m_poly.at(1);
					pNode->m_poly.clear();

					position.set(lt.longitude(),rb.latitude());

					qreal mLon = position.longitude()-moved.longitude();
					qreal mLat = position.latitude()-moved.latitude();

					if(position.longitude()>moved.longitude())
					{
						mLon = position.longitude()-moved.longitude();
						lt.setLongitude(lt.longitude()-mLon);
						rb.setLongitude(rb.longitude()-mLon);
					}
					else 
					{
						mLon = moved.longitude()-position.longitude();
						lt.setLongitude(lt.longitude()+mLon);
						rb.setLongitude(rb.longitude()+mLon);
					}

					if(position.latitude()>moved.latitude())
					{
						mLat = position.latitude()-moved.latitude();
						lt.setLatitude(lt.latitude()-mLat);
						rb.setLatitude(rb.latitude()-mLat);
					}
					else 
					{
						mLat = moved.latitude()-position.latitude();
						lt.setLatitude(lt.latitude()+mLat);
						rb.setLatitude(rb.latitude()+mLat);
					}

					pNode->m_poly.append(lt);
					pNode->m_poly.append(rb);

					mLon = lt.longitude() + ( (rb.longitude()-lt.longitude())/2);
					mLat = lt.latitude() - ( (lt.latitude()-rb.latitude())/2);

					pNode->m_position.set(mLon,mLat,0);
					pNode->parent->m_position.set(mLon,mLat,0);
				}//else moving index 2
				else if( (m_movingIndex<0 || m_movingIndex==5) && m_mousePos.rx()>=0)
				{
					GeoDataCoordinates position;// = m_pNode->m_position;
					GeoDataCoordinates lt = pNode->m_poly.at(0);
					GeoDataCoordinates rb = pNode->m_poly.at(1);
					pNode->m_poly.clear();

					qreal mLon = 0;
					qreal mLat = 0;
					m_marbleWidget->geoCoordinates(m_mousePos.rx(),m_mousePos.ry(),
                                         mLon, mLat, GeoDataCoordinates::Radian );
					position.set(mLon,mLat,0);//mouse pos

					qreal dLon = position.longitude()-moved.longitude();
					qreal dLat = position.latitude()-moved.latitude();

					lt.set(lt.longitude()-dLon,lt.latitude()-dLat,0);
					rb.set(rb.longitude()-dLon,rb.latitude()-dLat,0);
					pNode->m_poly.append(lt);
					pNode->m_poly.append(rb);

					mLon = lt.longitude() + ( (rb.longitude()-lt.longitude())/2);
					mLat = lt.latitude() - ( (lt.latitude()-rb.latitude())/2);

					pNode->m_position.set(mLon,mLat,0);
					pNode->parent->m_position.set(mLon,mLat,0);

					m_mousePos.setX(pt.x());
					m_mousePos.setY(pt.y());

					m_movingIndex=5;//set to 5 to correct handle mouse release

				}
		}// if !rect

	}//pNode!=NULL

	return true;
	//qDebug() << "MouseMove: " << m_movingID;
}

bool MapViewTSLayerPrivate::isInfoPoint( const QPoint &point )
{
/*	if(m_infoID>0 && m_annoRect.width() !=0)
	{
		m_marbleWidget->repaint( m_annoRect );
		m_annoRect.setWidth(0);
	}*/
	//m_infoID=0;
	
    foreach( const RequestRegion &region, m_regions ) 
	{
        if ( region.region.contains( point ) ) {
			//m_infoID = region.index;
            return true;
        }
    }

    foreach( const ModelRegion &region, m_placemarks ) 
	{
        if ( region.region.contains( point ) ) {
			m_infoID = region.index.internalId();
			m_posInfo = point;
			//m_marbleWidget->repaint( m_annoRect );
			//m_annoRect.setWidth(0);
			emit q->mouseHoversObject(region.index,m_posInfo,q->getInterfaceID());
            return true;
        }
    }

	foreach( const ModelRegion &region, m_routeRegion ) 
	{
		if ( region.region.contains( point )  )
		{
			m_infoID = region.index.internalId();
			m_posInfo = point;
			//m_marbleWidget->repaint( m_annoRect );
			//m_annoRect.setWidth(0);
			return true;
		}
	}

    return false;
}

/*
 bool RoutingLayerPrivate::isAlternativeRoutePoint( const QPoint &point )
 {
     foreach( const SkeletonRegion &region, m_alternativeRouteRegions ) {
         if ( region.region.contains( point ) ) {
             return true;
         }
     }

     return false;
 }
 */

void MapViewTSLayerPrivate::paintStopOver( QRect dirty )
{
    m_marbleWidget->update( m_dirtyRect );
    int dx = 1 + m_pixmapSize.width() / 2;
    int dy = 1 + m_pixmapSize.height() / 2;
    dirty.adjust( -dx, -dy, -dx, -dy );
    m_marbleWidget->update( dirty );
    m_dirtyRect = dirty;
}

void MapViewTSLayerPrivate::clearStopOver()
{
	if(m_annoRect.width()>0)
	{
		 m_marbleWidget->update( m_annoRect );
		 m_annoRect.setWidth(0);
	}
    m_dropStopOver = QPoint();
    m_dragStopOver = QPoint();
    m_marbleWidget->update( m_dirtyRect );

/*	if(m_infoID)
	{
		int iW = m_annoRect.width();
		int iH = m_annoRect.height();
		QRect dirty = QRect(m_posInfo, QSize( 100,100));
		dirty.setLeft(dirty.left() - 100);
		dirty.setTop(dirty.bottom() - 100);
		m_marbleWidget->repaint( dirty );
	}*/
}
/*
void  MapViewTSLayerPrivate::exportOverlay2kml(GeoDataDocument* document)
{
	if(m_indexID==0)return;
	if(document==NULL)return;

	TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_indexID);
	if(pNode==NULL) return;
	if(pNode->ItemType != Layer) return;

	QString qsOverlay = pNode->parent->m_qsName;
	QString qsLayer   = pNode->m_qsName;
	int iGroupCount   = pNode->children.count();

	//document->setName(qsOverlay);

	TreeNode* pGroup=NULL;
	TreeNode* pChild=NULL;
	int iChildCount=0;

	for(int iG=0;iG<iGroupCount;iG++)//count the group
	{
		pGroup = pNode->children.at(iG);
		if(pGroup!=NULL)
		{
			iChildCount = pGroup->children.count();
			for(int iC=0;iC<iChildCount;iC++)
			{
				pChild =  pGroup->children.at(iC);
				if(pChild != NULL)
				{
					switch(pChild->ItemType)
					{
						case Icon:	
							 addIconToKml(pGroup,pChild,document);
							 break;
					    case Text:
						case Desc:
							 addLabelToKml(pGroup,pChild,document);
							 break;
						case Rect:	
							addRectToKml(pGroup,pChild,document);
							break;
						case Polygone:	
							addPolygoneToKml(pGroup,pChild,document);
							break;
						case Poly:	
							addPolyToKml(pGroup,pChild,document);
							break;
						case Elipse:	
							addEllipseToKml(pGroup,pChild,document);
							break;
						case Azimuth:	
							addAzimuthToKml(pGroup,pChild,document);
							break;
						case Plot:	
							addPlotToKml(pGroup,pChild,document);
							break;
						case PlotEx:	
							addPlotExToKml(pGroup,pChild,document);
							break;
					}

				}//for pChild
			}//

		}//if pGroup

	}//for GroupCount

}*/

void  MapViewTSLayerPrivate::exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder)
{
	if(m_indexID==0)return;
	if(document==NULL)return;
	if(folder==NULL)return;

	TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_indexID);
	if(pNode==NULL) return;
	if(pNode->ItemType != Layer) return;

//	document->setDescription("Taktical");
	folder->setDescription("RSMOG");

	QString qsOverlay = pNode->parent->m_qsName;
	QString qsLayer   = pNode->m_qsName;
	int iGroupCount   = pNode->children.count();

	//document->setName(qsOverlay);

	TreeNode* pGroup=NULL;
	TreeNode* pChild=NULL;
	int iChildCount=0;

	for(int iG=0;iG<iGroupCount;iG++)//count the group
	{
		pGroup = pNode->children.at(iG);
		if(pGroup!=NULL)
		{
			iChildCount = pGroup->children.count();
			//for(int iC=0;iC<iChildCount;iC++)
			if(iChildCount==1)//group with one child
			{
				pChild =  pGroup->children.at(0);
				if(pChild != NULL)
				{
					switch(pChild->ItemType)
					{
						case Icon:	
							 addIconToKml(pGroup,pChild,document,folder);
							 break;
					    case Text:
						case Desc:
							 addLabelToKml(pGroup,pChild,document,folder);
							 break;
						case Rect:	
							addRectToKml(pGroup,pChild,document,folder);
							break;
						case Polygone:	
							addPolygoneToKml(pGroup,pChild,document,folder);
							break;
						case Poly:	
							addPolyToKml(pGroup,pChild,document,folder);
							break;
						case Elipse:	
							addEllipseToKml(pGroup,pChild,document,folder);
							break;
						case Azimuth:	
							addAzimuthToKml(pGroup,pChild,document,folder);
							break;
						case Plot:	
							addPlotToKml(pGroup,pChild,document,folder);
							break;
						case PlotEx:	
							addPlotExToKml(pGroup,pChild,document,folder);
							break;
					}

				}//if one Child
				
			}//
			else	//MultiGeometry
				addMultiGeometryToKml(pGroup,document,folder);

		}//if pGroup

	}//for GroupCount

}

void MapViewTSLayerPrivate::export2kml(QString& filename)
{
	if(m_indexID==0)return;

	TreeNode* pNode = m_treeviewModel->getNodeFromIndexId(m_indexID);
	if(pNode==NULL) return;
	if(pNode->ItemType != Layer) return;

	QString qsOverlay = pNode->parent->m_qsName;
	QString qsLayer   = pNode->m_qsName;
	int iGroupCount   = pNode->children.count();

	GeoDataDocument* document = new GeoDataDocument;
/*	QList<GeoDataFeature> features;

	GeoDataStyle style;
	// GeoDataStyle styleIco;
    GeoDataLineStyle	lineStyle;
	GeoDataIconStyle	iconStyle;
	GeoDataLabelStyle  labelStyle;*/

	GeoDataDocument container;
	GeoDataFolder* result = new GeoDataFolder;
	result->setName(qsOverlay);
	if ( result ) {
        container.append( result );
    }
	/*
	TreeNode* pGroup=NULL;
	TreeNode* pChild=NULL;
	int iChildCount=0;

	for(int iG=0;iG<iGroupCount;iG++)//count the group
	{
		pGroup = pNode->children.at(iG);
		if(pGroup!=NULL)
		{
			iChildCount = pGroup->children.count();
			for(int iC=0;iC<iChildCount;iC++)
			{
				pChild =  pGroup->children.at(iC);
				if(pChild != NULL)
				{
					switch(pChild->ItemType)
					{
						case Icon:	
							 addIconToKml(pGroup,pChild,document);
							 break;
						case Rect:	
							addRectToKml(pGroup,pChild,document);
							break;
						case Polygone:	
							addPolygoneToKml(pGroup,pChild,document);
							break;
						case Poly:	
							addPolyToKml(pGroup,pChild,document);
							break;
						case Elipse:	
							addEllipseToKml(pGroup,pChild,document);
							break;
						case Azimuth:	
							addAzimuthToKml(pGroup,pChild,document);
							break;
						case Plot:	
							addPlotToKml(pGroup,pChild,document);
							break;
						case PlotEx:	
							addPlotExToKml(pGroup,pChild,document);
							break;
					}

				}//for pChild
			}//

		}//if pGroup

	}//for GroupCount

	container.append( new GeoDataDocument( *document ) );
	GeoWriter writer;
    writer.setDocumentType( "http://earth.google.com/kml/2.2" );

   // QString const filename = "test.kml";
    QFile file( filename );
    if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
        qCritical() << "Cannot write to " << file.fileName();
    }

    if ( !writer.write( &file, &container ) ) {
        qCritical() << "Can not write to " << file.fileName();
    }

	//writer.writeElement(
    file.close();
*/
	delete document;
}
/*
void MapViewTSLayerPrivate::addIconToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark = new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	lineStyle.setColor(m_fillColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	//style.setStyleId( m_lineColor.name().replace( '#', 'f' ) );
	style.setStyleId( pGroup->m_qsName );

	QPixmap qPix = m_treeviewModel->pixmap(pChild->m_qsImage);
	if(!qPix.isNull())
	{
		//to export ToolTip as <description>
		if(!pGroup->m_qsToolTip.isEmpty())
			placemark->setDescription(pGroup->m_qsToolTip);

		iconStyle.setIcon(qPix.toImage());
		iconStyle.setScale(1.0);
		iconStyle.setIconPath(pChild->m_qsImage);
	
		style.setIconStyle(iconStyle);
		document->addStyle( style );
		placemark->setStyleUrl(  QString( "#" ).append(pGroup->m_qsName ));

		GeoDataCoordinates pos = pGroup->m_position;
		placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		document->append( placemark );
	}

}*/

void MapViewTSLayerPrivate::addMultiGeometryToKml(TreeNode* pGroup,GeoDataDocument* document,GeoDataFolder* folder)
{
	if(pGroup==NULL || document==NULL || folder==NULL)
		return;

	GeoDataPlacemark* placemark = new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	m_bMultiGeoStyle=false;

	//to export ToolTip as <description>
	if(!pGroup->m_qsToolTip.isEmpty())
		placemark->setDescription(pGroup->m_qsToolTip);

	TreeNode* pChild=NULL;
	GeoDataMultiGeometry*	pMultiGeo = new GeoDataMultiGeometry();

	int iChildCount = pGroup->children.count();
	for(int iC=0;iC<iChildCount;iC++)
	{
		pChild =  pGroup->children.at(iC);
		if(pChild != NULL)
		{
			switch(pChild->ItemType)
			{
				case Icon:
					 addIconToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					 break;
			    case Poly:
					addPolyToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case Polygone:	
					addPolygonToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case Plot:	
					addPlotToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case Elipse:	
					addEllipseToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case Azimuth:	
					addAzimuthToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case PlotEx:	
					addPlotExToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					break;
				case Text:
				case Desc:
					 addLabelToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
					 break;
				case Rect:
					 addRectToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
			    case Arrow:
					 addArrowToMultiGeometry(pGroup,pChild,document,placemark,pMultiGeo);
				default: break;
			}
		}
	}

	placemark->setGeometry(pMultiGeo);
	folder->append(placemark);
	
}

void MapViewTSLayerPrivate::addIconToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pGroup->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_fillColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	//style.setStyleId( m_lineColor.name().replace( '#', 'f' ) );
	style.setStyleId(styleID);

	QPixmap qPix = m_treeviewModel->pixmap(pChild->m_qsImage);
	if(!qPix.isNull())
	{
		//to export ToolTip as <description>
		if(!pGroup->m_qsToolTip.isEmpty())
			placemark->setDescription(pGroup->m_qsToolTip);

		iconStyle.setIcon(qPix.toImage());
		iconStyle.setScale(1.0);
		iconStyle.setIconPath(pChild->m_qsImage);
	
		style.setIconStyle(iconStyle);
		document->addStyle( style );
		placemark->setStyleUrl(  QString( "#" ).append(styleID ));

		GeoDataCoordinates pos = pGroup->m_position;
		placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		folder->append( placemark );
	}

}

void MapViewTSLayerPrivate::addLabelToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString qsLabel; 
	if(pChild->ItemType==Desc)
	{
		qsLabel = pChild->m_qsDesc;
		if(!qsLabel.isEmpty())
		{
			placemark->setName( qsLabel );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			folder->append( placemark );
		}
	}
	else if(pChild->ItemType==Text)
	{
		qsLabel = pChild->m_qsName;
		if(!qsLabel.isEmpty())
		{
			placemark->setName( qsLabel );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			folder->append( placemark );
		}
	}

}
void MapViewTSLayerPrivate::addRectToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) ); 

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS; 
	GeoDataCoordinates pos2;
	
	pos  = pChild->m_poly.at(0);
	pos2 = pChild->m_poly.at(1);

	gdLS << pos;
	pos.setLongitude(pos2.longitude());
	gdLS << pos;
	pos.setLatitude(pos2.latitude());
	gdLS << pos;
	pos  = pChild->m_poly.at(0);
	pos.setLatitude(pos2.latitude());
	gdLS << pos;
	pos  = pChild->m_poly.at(0);
	gdLS << pos;
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewTSLayerPrivate::addPolygoneToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;
	int alpha=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		alpha     = ( colorRef>>24 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(alpha);
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewTSLayerPrivate::addPolyToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	//gdLS << pNode->m_poly.at(0);
	placemark->setGeometry( new GeoDataLineString( gdLS) );

	folder->append( placemark );

}
void MapViewTSLayerPrivate::addEllipseToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}
void MapViewTSLayerPrivate::addAzimuthToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	//gdLS << pNode->m_poly.at(0);
	placemark->setGeometry( new GeoDataLineString( gdLS) );

	folder->append( placemark );

}

void MapViewTSLayerPrivate::addPlotToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}

void MapViewTSLayerPrivate::addPlotExToKml(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataFolder* folder)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataPlacemark* placemark=new GeoDataPlacemark;
	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString gdLS;
	gdLS << pChild->m_poly;
	gdLS << pChild->m_poly.at(0);
	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	folder->append( placemark );

}

void MapViewTSLayerPrivate::addIconToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	if(pMG==NULL)return;

	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	GeoDataCoordinates pos = pGroup->m_position;
	GeoDataPoint* pPoint = new GeoDataPoint(pos); 
	if(pPoint==NULL)return;

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	//style.setStyleId( m_lineColor.name().replace( '#', 'f' ) );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pGroup->m_qsName).arg(pChild->m_lId);
	style.setStyleId( styleID);

	QPixmap qPix = m_treeviewModel->pixmap(pChild->m_qsImage);
	if(!qPix.isNull())
	{
		iconStyle.setIcon(qPix.toImage());
		iconStyle.setScale(1.0);
		iconStyle.setIconPath(pChild->m_qsImage);
	
		style.setIconStyle(iconStyle);
		document->addStyle( style );
		placemark->setStyleUrl(  QString( "#" ).append(styleID ));
		m_bMultiGeoStyle=true;

		GeoDataCoordinates pos = pGroup->m_position;
		placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

		pMG->append( pPoint );
	}

}

void MapViewTSLayerPrivate::addLabelToMultiGeometry(TreeNode* pGroup,TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
//	GeoDataLabelStyle	labelStyle;
	if(pMG==NULL)return;

//	GeoDataCoordinates pos = pGroup->m_position;
//	GeoDataPoint* pPoint = new GeoDataPoint(pos); 
//	if(pPoint==NULL)return;

	GeoDataExtendedData extData;
	GeoDataData turnType;
        
    //turnType.setValue( qVariantFromValue<int>( int( directions[i].turnType() ) ) );
        

	QString qsLabel; 
	if(pChild->ItemType==Desc)
	{
		qsLabel = pChild->m_qsDesc;
		if(!qsLabel.isEmpty())
		{
			turnType.setName(pGroup->m_qsName);
			turnType.setValue( qsLabel );
			
			extData.addValue( turnType );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			placemark->setExtendedData(extData);
		//	pMG->append( pPoint );
		}
	}
	else if(pChild->ItemType==Text)
	{
		qsLabel = pChild->m_qsName;
		if(!qsLabel.isEmpty())
		{
			turnType.setName(pGroup->m_qsName);
			turnType.setValue( qsLabel );
			extData.addValue( turnType );
			//to export ToolTip as <description>
			if(!pGroup->m_qsToolTip.isEmpty())
				placemark->setDescription(pGroup->m_qsToolTip);

			GeoDataCoordinates pos = pGroup->m_position;
			placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

			placemark->setExtendedData(extData);
		//	pMG->append( pPoint );
		}
	}

}

void MapViewTSLayerPrivate::addRectToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

//	GeoDataPlacemark* placemark=new GeoDataPlacemark;
//	placemark->setName( pGroup->m_qsName );
	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) ); 

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing* gdLR = new GeoDataLinearRing();
	GeoDataCoordinates pos2;
	
	pos  = pChild->m_poly.at(0);
	pos2 = pChild->m_poly.at(1);

	gdLR->append(pos);
	pos.setLongitude(pos2.longitude());
	gdLR->append(pos);
	pos.setLatitude(pos2.latitude());
	gdLR->append(pos);
	pos  = pChild->m_poly.at(0);
	pos.setLatitude(pos2.latitude());
	gdLR->append(pos);
	pos  = pChild->m_poly.at(0);
	gdLR->append(pos);

	pMG->append(gdLR);
//	placemark->setGeometry( new GeoDataLinearRing( gdLS) );

//	folder->append( placemark );

}
void MapViewTSLayerPrivate::addPolygonToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	//GeoDataPlacemark* placemark=new GeoDataPlacemark;
	//placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;
	int alpha=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		alpha     = ( colorRef>>24 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(alpha);
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );
	m_bMultiGeoStyle=true;//use polygon style also for line style

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing* gdLR = new GeoDataLinearRing();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLR->append(pChild->m_poly.at(i));

	pMG->append(gdLR);

}
void MapViewTSLayerPrivate::addPolyToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

//	GeoDataPlacemark* placemark=new GeoDataPlacemark;
//	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString* gdLS = new GeoDataLineString();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLS->append(pChild->m_poly.at(i));

	pMG->append(gdLS);

}

void MapViewTSLayerPrivate::addEllipseToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
	GeoDataPolyStyle	polyStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

//	GeoDataPlacemark* placemark=new GeoDataPlacemark;
//	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	if(pChild->m_lFillCol != -1)
	{
		colorRef = pChild->m_lFillCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		if(pChild->m_iTrans != -1)
			col.setAlpha(pChild->m_iTrans);
		else col.setAlpha(m_fillColor.alpha());
		polyStyle.setColor(col);
	}
	else polyStyle.setColor(m_fillColor);
	style.setPolyStyle(polyStyle);

	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLinearRing* gdLR = new GeoDataLinearRing();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLR->append(pChild->m_poly.at(i));
	gdLR->append(pChild->m_poly.at(0));

	pMG->append(gdLR);

}

void MapViewTSLayerPrivate::addAzimuthToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

//	GeoDataPlacemark* placemark=new GeoDataPlacemark;
//	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString* gdLS = new GeoDataLineString();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLS->append(pChild->m_poly.at(i));

	pMG->append(gdLS);

}

void MapViewTSLayerPrivate::addPlotToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);
	style.setStyleId( styleID);

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

//	GeoDataCoordinates pos = pGroup->m_position;
//	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString* gdLS = new GeoDataLineString();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLS->append(pChild->m_poly.at(i));
	gdLS->append(pChild->m_poly.at(0));
	pMG->append(gdLS);

	//folder->append( placemark );

}

void MapViewTSLayerPrivate::addPlotExToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

	lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) ); 

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	//GeoDataCoordinates pos = pGroup->m_position;
	//placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString* gdLS = new GeoDataLineString();
	for(int i=0; i<pChild->m_poly.size();i++)
		gdLS->append(pChild->m_poly.at(i));
	gdLS->append(pChild->m_poly.at(0));
	pMG->append(gdLS);
	//placemark->setGeometry( new GeoDataLinearRing( gdLS) );

	//folder->append( placemark );
	//pMG->append(gdLS);

}

void MapViewTSLayerPrivate::addArrowToMultiGeometry(TreeNode* pGroup, TreeNode* pChild,GeoDataDocument* document,GeoDataPlacemark* placemark,GeoDataMultiGeometry* pMG)
{
	GeoDataStyle		style;
    GeoDataLineStyle	lineStyle;
//	GeoDataIconStyle	iconStyle;
//	GeoDataLabelStyle	labelStyle;

//	GeoDataPlacemark* placemark=new GeoDataPlacemark;
//	placemark->setName( pGroup->m_qsName );

	QString styleID=("%1-%2");
	styleID = styleID.arg(pChild->m_qsName).arg(pChild->m_lId);

	GeoDataCoordinates pos1 = pChild->m_poly.at(0);
	GeoDataCoordinates pos2 = pChild->m_poly.at(1);

	qreal lx1=0;
	qreal ly1=0;
	m_marbleWidget->screenCoordinates(pos1.longitude(GeoDataCoordinates::Degree),pos1.latitude(GeoDataCoordinates::Degree),lx1,ly1);
	qreal lx2=0;
	qreal ly2=0;
	m_marbleWidget->screenCoordinates(pos2.longitude(GeoDataCoordinates::Degree),pos2.latitude(GeoDataCoordinates::Degree),lx2,ly2);

	int dirc1 = pChild->m_iXoff;
	int dirc2 = pChild->m_iYoff;

	qreal lon1=0;
	qreal lat1=0;
	qreal lon2=0;
	qreal lat2=0;
	qreal lon3=0;
	qreal lat3=0;

	QPoint ptDir1((int)lx1,(int)ly1);
	QPoint ptDir2((int)lx2,(int)ly2);

	double y=(double)ly2-(double)ly1;
	double x=(double)lx2-(double)lx1;
	double c=sqrt(pow(x,2)+pow(y,2));
	if (c>=70) 
	{ //fits to the 40 distance, so draw it
		double angle=atan2(y,x); //in radians
		angle+=PI/2;
		if (dirc1>0) 
		{
			QPoint pt0;
			QPoint pt1;

			CycToPoint(ptDir1,pt0,angle,40);
			CycToPoint(pt0,pt1,angle+150.0*PI/180,13);
			m_marbleWidget->geoCoordinates(pt0.x(),pt0.y(),lon1,lat1);
			m_marbleWidget->geoCoordinates(pt1.x(),pt1.y(),lon2,lat2);
			
			CycToPoint(pt0,pt1,angle+210.0*PI/180,13);
			m_marbleWidget->geoCoordinates(pt1.x(),pt1.y(),lon3,lat3);
		}
		if (dirc2>0) 
		{
			angle -=PI;
			QPoint pt0;
			QPoint pt1;

			CycToPoint(ptDir2,pt0,angle,40);
			CycToPoint(pt0,pt1,angle+150.0*PI/180,13);
			m_marbleWidget->geoCoordinates(pt0.x(),pt0.y(),lon1,lat1);
			m_marbleWidget->geoCoordinates(pt1.x(),pt1.y(),lon2,lat2);
			
			CycToPoint(pt0,pt1,angle+210.0*PI/180,13);
			m_marbleWidget->geoCoordinates(pt1.x(),pt1.y(),lon3,lat3);
		}
	}

	QColor col;
	long colorRef=0;
	int red=0;
	int green=0;
	int blue=0;

	if(pChild->m_lLineCol != -1)
	{
		colorRef = pChild->m_lLineCol;
		red      = colorRef & 255;
		green    = ( colorRef>>8 ) & 255;
		blue     = ( colorRef>>16 ) & 255;
		col = QColor( red, green, blue );
		lineStyle.setColor(col);
	}
	else lineStyle.setColor(m_lineColor);
	lineStyle.setWidth(m_lineWidth);
	style.setLineStyle( lineStyle );
	style.setStyleId( styleID );

//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

	GeoDataStyleMap styleMap;
//	styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
	styleMap.setStyleId(styleID);
	styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
	document->addStyle( style );

	if(!m_bMultiGeoStyle)
		placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

	GeoDataCoordinates pos = pGroup->m_position;
	placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

	GeoDataLineString* gdLS = new GeoDataLineString();
	GeoDataCoordinates cor;
	cor.set(lon1,lat1,0,GeoDataCoordinates::Degree);
	gdLS->append(cor);
	cor.set(lon2,lat2,0,GeoDataCoordinates::Degree);
	gdLS->append(cor);
	cor.set(lon3,lat3,0,GeoDataCoordinates::Degree);
	gdLS->append(cor);
	
	pMG->append(gdLS);

}

/*
void MapViewTSLayerPrivate::export2kml(QString& filename)
{
	 GeoDataDocument* document = new GeoDataDocument;

	 QList<GeoDataFeature> features;

	 GeoDataStyle style;
	// GeoDataStyle styleIco;
     GeoDataLineStyle lineStyle;
	 GeoDataIconStyle iconStyle;
	 GeoDataLabelStyle  labelStyle;
	 TreeNode* pNode=NULL;

	 GeoDataDocument container;
	 GeoDataFolder* result = new GeoDataFolder;
	 result->setName( "Overlay" );
	 if ( result ) {
        container.append( result );
    }

	 if(!m_placemarks.isEmpty())
	 {
		  foreach( const ModelRegion &region, m_placemarks ) 
		  {
			pNode = m_treeviewModel->getNodeFromIndex(region.index);
			if(pNode!=NULL)
			{
				GeoDataPlacemark* placemark = new GeoDataPlacemark;
				placemark->setName( pNode->parent->m_qsName );
		        
				lineStyle.setColor(m_lineColor);
				lineStyle.setWidth(m_lineWidth);
				style.setLineStyle( lineStyle );
				//style.setStyleId( m_lineColor.name().replace( '#', 'f' ) );
				style.setStyleId( pNode->parent->m_qsName );
				
				if(!pNode->m_qsImage.isEmpty())
				{
					QPixmap qPix = m_treeviewModel->pixmap(pNode->m_qsImage);
					if(!qPix.isNull())
					{
					
						iconStyle.setIcon(qPix);
						iconStyle.setScale(1.0);
						iconStyle.setIconPath(pNode->m_qsImage);
					
						style.setIconStyle(iconStyle);
					//	style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
					}
				}
				

				//GeoDataStyleMap styleMap;
				//style map id is different from style id and must be set to placemark url as well
				//styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
				//styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
				document->addStyle( style );
			//	document->addStyle( styleIco );

				//placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );
				placemark->setStyleUrl( pNode->parent->m_qsName );
				

				GeoDataCoordinates pos = pNode->parent->m_position;

				placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

				//placemark->setStyle(&style);
				//features.append( *placemark );
				//features.at(0).setStyle(styleIco);

				//placemark->setGeometry( new GeoDataLinearRing( region.region.geometry().outerBoundary() ) );
				//document->append(iconStyle);
				document->append( placemark );
				//document->addStyleMap( styleMap );
			}//ifpNode

		  }//foreach
	 }

	 long id=0;

	 if(!m_routeRegion.isEmpty())
	 {
		foreach( const ModelRegion &region, m_selpoint )
		{
			pNode = m_treeviewModel->getNodeFromIndex(region.index);
			if(pNode!=NULL && id != pNode->m_lId)
			{
				id = pNode->m_lId;
				GeoDataPlacemark* placemark = new GeoDataPlacemark;
				placemark->setName( pNode->m_qsName );
		        
				lineStyle.setColor(m_fillColor);
				lineStyle.setWidth(m_lineWidth);
				
				style.setLineStyle( lineStyle );
				style.setStyleId( m_fillColor.name().replace( '#', 'f' ) );

				GeoDataStyleMap styleMap;
				styleMap.setStyleId( m_fillColor.name().replace( '#', 'f' ) );
				styleMap.insert( "normal", QString( "#" ).append( style.styleId() ) );
				document->addStyle( style );

				placemark->setStyleUrl( QString( "#" ).append( styleMap.styleId() ) );

				GeoDataCoordinates pos = pNode->m_position;

				placemark->setCoordinate(pos.longitude(GeoDataCoordinates::Degree),pos.latitude(GeoDataCoordinates::Degree),0,GeoDataCoordinates::Degree);

				if(pNode->ItemType == Rect)
				{
					GeoDataLinearRing gdLR;
					GeoDataCoordinates pos2;
					
					pos  = pNode->m_poly.at(0);
					pos2 = pNode->m_poly.at(1);
	
					gdLR << pos;
					pos.setLongitude(pos2.longitude());
					gdLR << pos;
					pos.setLatitude(pos2.latitude());
					gdLR << pos;
					pos  = pNode->m_poly.at(0);
					pos.setLatitude(pos2.latitude());
					gdLR << pos;
					pos  = pNode->m_poly.at(0);
					gdLR << pos;
					placemark->setGeometry( new GeoDataLinearRing( gdLR) );
					//placemark->setGeometry( &gdLR);

				}
				else if(pNode->ItemType == Polygone)
				{
					GeoDataLineString gdLS;
					gdLS << pNode->m_poly;
					gdLS << pNode->m_poly.at(0);
					placemark->setGeometry( new GeoDataLineString( gdLS) );

				}
				else if(pNode->ItemType == Poly)
				{
					GeoDataLineString gdLS;
					gdLS << pNode->m_poly;
					//gdLS << pNode->m_poly.at(0);
					placemark->setGeometry( new GeoDataLineString( gdLS) );

				}
				else if (pNode->ItemType == Elipse )
				{
					GeoDataLineString gdLS = pNode->m_poly;
					gdLS << pNode->m_poly.at(0);
					placemark->setGeometry( new GeoDataLineString( gdLS) );
				}
				else if(pNode->ItemType == Azimuth)
				{
					GeoDataLineString gdLS;
					gdLS << pNode->m_poly;
					//gdLS << pNode->m_poly.at(0);
					placemark->setGeometry( new GeoDataLineString( gdLS) );

				}
				document->append( placemark );
			//	document->addStyleMap( styleMap );

			}//ifpNode

		  }//foreach
		
	 }

    container.append( new GeoDataDocument( *document ) );
	GeoWriter writer;
    writer.setDocumentType( "http://earth.google.com/kml/2.2" );

   // QString const filename = "test.kml";
    QFile file( filename );
    if ( !file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
        qCritical() << "Cannot write to " << file.fileName();
    }

    if ( !writer.write( &file, &container ) ) {
        qCritical() << "Can not write to " << file.fileName();
    }

	//writer.writeElement(
    file.close();

	delete document;

}*/

void MapViewTSLayerPrivate::DrawArrow( GeoPainter *painter, QPoint& pt, double angle, short dir_count,QColor& color)
{
	QPoint pt0;
	QPoint pt1;
	CycToPoint(pt,pt0,angle,40);
	CycToPoint(pt0,pt1,angle+150.0*PI/180,13);
	painter->setPen(color);
	painter->drawLine(pt0.x(),pt0.y(),pt1.x(),pt1.y());
//	logging(QString("a"),pt0.x(),pt0.y(),pt1.x(),pt1.y());
	CycToPoint(pt0,pt1,angle+210.0*PI/180,13);
	painter->drawLine(pt0.x(),pt0.y(),pt1.x(),pt1.y());
//	logging(QString("b"),pt0.x(),pt0.y(),pt1.x(),pt1.y());

	//pt1 = CycToPoint(pt0,angle+220.0*PI/180,20);
	pt1.setX(pt0.x()+2);
	pt1.setY(pt0.y()+2);
	qreal lon=0;
	qreal lat=0;
	m_marbleWidget->geoCoordinates(pt1.x(),pt1.y(),lon,lat);
//	logging(QString("b"),pt1.x(),pt1.y(),(long)lon,(long)lat);
	GeoDataCoordinates pos;
	pos.set(lon,lat,0,GeoDataCoordinates::Degree);

	QString qsText = "%1";
	qsText = qsText.arg(dir_count);
	if(m_iDimmer>50)
		painter->setPen( QColor(255,255,255) );
	else painter->setPen( QColor(0,0,0) );
	painter->drawText(pos,qsText);

}

/** angle in radians*/
bool MapViewTSLayerPrivate::CycToPoint(QPoint& center,QPoint& ret, double angle, double relativeDistanceToCenter)
{
	double x, y;

	while (angle > 2*PI)
		angle -= 2*PI;

	while (angle < 0)
		angle += 2*PI;

	//double angle_arc = (double(angle) * _PI_) / (double(1800));
	//int angle = int (angle_arc+0.5);
//	double angle_arc = (double(angle) * PI) / (double(1800));
	x = sin(angle);
	y = cos(angle);

	x *= relativeDistanceToCenter;
	y *= relativeDistanceToCenter;

	ret.setX(x+center.x());
	ret.setY(center.y()-y);
   
	return true;
}

void MapViewTSLayerPrivate::removePlacemark(qint64 id)
{
	if(id>0)
	{
		if(m_selIdx.isValid() && m_selIdx.internalId()==id)
		{
			//m_placemarks.removeOne(m_selIdx);
			m_selIdx = QModelIndex();
			m_bIsPoly=false;
			m_bObjId=false;
			m_activeMenuIndex = -1;
		//	return;
		//	d->m_marbleWidget->update();
		}
		if(m_placemarks.size()>0)
		{
			int i=0;
			foreach( const ModelRegion &region, m_placemarks ) 
			{
				if ( region.index.internalId() == id)
				{
					//m_placemarks.removeOne(region);
					m_placemarks.removeAt(i);
					return;
				}
				i++;
			}
		}//if placemarks
		if(m_routeRegion.size()>0)
		{
			int i=0;
			foreach( const ModelRegion &region, m_routeRegion ) 
			{
				if ( region.index.internalId() == id)
				{
					//m_placemarks.removeOne(region);
					m_routeRegion.removeAt(i);
					if(m_selpoint.size()>0)
					{
						i=0;
						foreach( const ModelRegion &region, m_selpoint ) 
						{
							if ( region.index.internalId() == id)
							{
								m_selpoint.removeAt(i);
							}
							i++;
						}//foreach selpoint
					}//if selpoint
					return;
				}//if region == routeregion
				i++;
			}//for each route region

		}//if route region
	
	}//if id

}

/*****			MapViewTSLayer		******************************************/

MapViewTSLayer::MapViewTSLayer( MarbleWidget *widget, QWidget *parent, qint64 index,long interfaceID ) :
        QObject( parent ), d( new MapViewTSLayerPrivate( this, widget,index ) )
{
    widget->installEventFilter( this );

	connect( widget, SIGNAL( visibleLatLonAltBoxChanged( GeoDataLatLonAltBox ) ),
            this, SLOT( setViewportChanged() ) );

	d->m_selTypePixmaps[GIS::Unknown] = ":/bitmaps/unmanned_hard_landing.png";
 //   d->m_selTypePixmaps[GIS::RectSelect] = MarbleDirs::path("bitmaps/unmanned_soft_landing.png");
	d->m_selTypePixmaps[GIS::PolySelect] = ":/bitmaps/nation.png"; 
	d->m_selTypePixmaps[GIS::RectCorner] = ":/bitmaps/coordinate.png"; 

	d->m_parentWidget = widget;
	m_InterfaceID = interfaceID; 
	m_zValue=1.0;
	m_cmID=-1;
}

MapViewTSLayer::~MapViewTSLayer()
{
//	if(d->m_polySkeleton!=NULL)
//		delete d->m_polySkeleton;

    delete d;
}

QStringList MapViewTSLayer::renderPosition() const
{
    //return QStringList() << "HOVERS_ABOVE_SURFACE";
	return QStringList() << "USER_TOOLS";
}

bool MapViewTSLayer::render( GeoPainter *painter, ViewportParams *viewport,
                           const QString& renderPos, GeoSceneLayer *layer )
{
    if ( d->m_marbleWidget->mapTheme()->head()->target() != "earth" ) {
        return false;
    }

	if ( renderPos != "USER_TOOLS" ) {
        return true;
    }

/*	GeoDataLatLonBox box = viewport->viewLatLonAltBox();
	qreal north,south,east,west;
	box.boundaries(north,south,east,west,GeoDataCoordinates::Degree);
	if(d->m_minLon != 399.0)
	{
		GeoDataLatLonBox ellipseBox(d->m_maxLat,d->m_minLat,d->m_maxLon,d->m_minLon, GeoDataCoordinates::Degree );
		if(!box.intersects(ellipseBox) )
			return true;
	}*/

//	qDebug() << "RenderIn: " ; 

    Q_UNUSED( viewport )
    Q_UNUSED( renderPos )
    Q_UNUSED( layer )

	d->m_update=true;
	QModelIndex idxLayer = d->m_treeviewModel->getIndexFromIndexId(d->m_indexID);
	if(!idxLayer.isValid())
		return false;
	QVariant qVar = d->m_treeviewModel->data(idxLayer,Qt::CheckStateRole);
	if(!qVar.toBool())
	{
		d->m_update=false;
		return false;
	}
	QModelIndex idxOverlay = d->m_treeviewModel->parent(idxLayer);
	if(!idxOverlay.isValid())
		return false;
	qVar = d->m_treeviewModel->data(idxOverlay,Qt::CheckStateRole);
	if(!qVar.toBool())
	{
		d->m_update=false;
		return false;
	}

    painter->save();
    painter->autoMapQuality();

/*    if ( d->m_placemarkModel) {
        d->renderPlacemarks( painter );
    }*/

	//Legend layer ?
	if(!d->m_bGeoProjected)
	{
		d->renderLegendLayer( painter );
		painter->restore();
		return true;
	}

	if ( d->m_treeviewModel) {
//		qDebug() << "RenderReqIn: " ; 
        d->renderRequest( painter );
//		qDebug() << "RenderReqOut: " ; 
    }

    if ( d->m_treeviewModel) {
//		qDebug() << "RenderLayerIn: " ; 
        d->renderLayer( painter );
//		qDebug() << "RenderLayerOut: " ; 
    }

	//qDebug() << "RenderIn " ; 
	if ( d->m_treeviewModel) { 
//		qDebug() << "RenderAnnoIn: " ; 
        d->renderAnnotations( painter );
//		qDebug() << "RenderAnnoOut: " ; 
    }
	//qDebug() << "RenderOut " ; 

    painter->restore();
	// d->m_viewportChanged = false;
//	qDebug() << "RenderOut: " ; 
    return true;
}

void MapViewTSLayer::setDimmerValue(int dValue)
{
	if ( d!=NULL)
		d->m_iDimmer=dValue;
}

void MapViewTSLayer::setLineColor(long col)
{
	if ( d!=NULL)
	{
		int red			= col & 255;
		int green		= ( col>>8 ) & 255;
		int blue		= ( col>>16 ) & 255;
		d->m_lineColor	= QColor( red, green, blue );
	}
		
}
void MapViewTSLayer::setLineColor(QColor& col)
{
	if ( d!=NULL)
	{
		d->m_lineColor	= col;
	}
		
}
QColor& MapViewTSLayer::getLineColor()
{
	return d->m_lineColor;
}

void MapViewTSLayer::setDefaultFont( const QFont& font )
{
	if ( d!=NULL)
		d->m_layerFont = font;
}

void MapViewTSLayer::setFontColor(long col)
{
	if ( d!=NULL)
	{
		int red			= col & 255;
		int green		= ( col>>8 ) & 255;
		int blue		= ( col>>16 ) & 255;
		d->m_fontColor	= QColor( red, green, blue);
	}
}
QColor& MapViewTSLayer::getFontColor()
{
	return d->m_fontColor;
}
void MapViewTSLayer::setFontBackColor(long col)
{
	if ( d!=NULL)
	{
		int red			= col & 255;
		int green		= ( col>>8 ) & 255;
		int blue		= ( col>>16 ) & 255;
		d->m_fontBackColor	= QColor( red, green, blue);
	}
}
QColor& MapViewTSLayer::getFontBackColor()
{
	return d->m_fontBackColor;
}
void MapViewTSLayer::setFillColor(long col)
{
	if ( d!=NULL)
	{
		int red			= col & 255;
		int green		= ( col>>8 ) & 255;
		int blue		= ( col>>16 ) & 255;
		d->m_fillColor	= QColor( red, green, blue,80);
	}
}
void MapViewTSLayer::setFillColor(QColor& col)
{
	if ( d!=NULL)
	{
		d->m_fillColor	= QColor( col.red(), col.green(), col.blue(),80);
	}
}
QColor& MapViewTSLayer::getFillColor()
{
	return d->m_fillColor;
}

void MapViewTSLayer::setTransparency(int trans)
{
	if ( d!=NULL)
	{
	/*	int red			= col & 255;
		int green		= ( col>>8 ) & 255;
		int blue		= ( col>>16 ) & 255;
		d->m_fillColor	= QColor( red, green, blue,80);*/
		if(trans>=0 && trans <=255)
			d->m_fillColor.setAlpha(trans);
	}
}

int MapViewTSLayer::getTransparency()
{
	if ( d!=NULL)
	{
		return d->m_fillColor.alpha();
	}
	else return 80;//default
}
	
void MapViewTSLayer::setLineWidth(int width)
{
	if ( d!=NULL)
		d->m_lineWidth = width;
}

int MapViewTSLayer::getLineWidth()
{
	if ( d!=NULL)
		return d->m_lineWidth;
	else return 1;
}

void MapViewTSLayer::setPenStyleValue(Qt::PenStyle style)
{
	if ( d!=NULL)
		d->m_penStyle = style;
}
	
Qt::PenStyle MapViewTSLayer::getPenStyleValue()
{
	if ( d!=NULL)
		return d->m_penStyle;
	else return Qt::SolidLine;

}

void MapViewTSLayer::setMarkerSize(int size)
{
	if ( d!=NULL)
		d->m_markerSize=size;
}
	
void MapViewTSLayer::setEdit(bool edit)
{
	if ( d!=NULL)
		d->m_bEdit=edit;
}

bool MapViewTSLayer::getEdit()
{
	if ( d!=NULL)
		return d->m_bEdit;
	else return false;
}
	
void MapViewTSLayer::setDelete(bool del)
{
	if ( d!=NULL)
		d->m_bDelete=del;
}

bool MapViewTSLayer::getDelete()
{
	if ( d!=NULL)
		return d->m_bDelete;
	else return false;
}

void MapViewTSLayer::setObjectEdit(bool move)
{
	if ( d!=NULL)//also set by O_EDIT which affects both
	{
		d->m_bMoveObj=move;
		d->m_bDeleteObj=move;
	}
}
	
void MapViewTSLayer::setObjectMove(bool move)
{
	if ( d!=NULL)
		d->m_bMoveObj=move;
}
	
void MapViewTSLayer::setObjectDelete(bool del)
{
	if ( d!=NULL)
		d->m_bDeleteObj=del;
}

bool MapViewTSLayer::getObjectDelete()
{
	if ( d!=NULL)
		return d->m_bDeleteObj;
	else return false;
}

bool MapViewTSLayer::eventFilter( QObject *obj, QEvent *event )
{
    Q_UNUSED( obj )

    if ( event->type() == QEvent::KeyPress ) {
        QKeyEvent *e = static_cast<QKeyEvent*>( event );
        return d->handleKeyEvent( e );
    }

	if ( event->type() == QEvent::MouseButtonDblClick ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseDblClick( e );
    }

    if ( event->type() == QEvent::MouseButtonPress ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonPress( e );
    }

    if ( event->type() == QEvent::MouseButtonRelease ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseButtonRelease( e );
    }

    if ( event->type() == QEvent::MouseMove ) {
        QMouseEvent *e = static_cast<QMouseEvent*>( event );
        return d->handleMouseMove( e );
    }

    return false;
}

void MapViewTSLayer::setModel ( TreeViewModel *model )
{
    d->m_placemarkModel = 0;
    d->m_treeviewModel = model;
	setViewportChanged();
}

void MapViewTSLayer::setModel ( MarblePlacemarkModel *model )
{
    d->m_treeviewModel = 0;
    d->m_placemarkModel = model;
	setViewportChanged();
}

void MapViewTSLayer::synchronizeWith( QItemSelectionModel *selection )
{
    d->m_selectionModel = selection;
//    d->m_proxyModel = model;
}

void MapViewTSLayer::synchronizeWithTree(const QModelIndex& idx)
{
	if(idx.isValid() )
	{
		d->synchronizeWithTree(idx);
	}
}

void MapViewTSLayer::synchronizeWithExtern(const QModelIndex& idx, bool ext )
{
	if(idx.isValid() )
	{
		d->synchronizeWithExtern(idx,ext);
	}
}

void MapViewTSLayer::setContextMenu(QString qsMenu)
{
	if(qsMenu.isEmpty() || d==NULL)
		return;

	//first delete old menu entries
	d->delContextMenu();

	QStringList qsl;
	QString qsTag;
	QString qsVal;
	int iVal=0;
	int id=0;
	do{
		qsTag = qsMenu.section(',',id,id+1);
		if(!qsTag.isEmpty())
		{
			qsVal = qsTag.section(',',1,1);
			iVal = qsVal.toInt();
			qsVal = qsTag.section(',',0,0);
			d->addContextMenu(qsVal,iVal);
		}
		id+=2;
	}while(!qsTag.isEmpty());


}

/*
void RoutingLayer::synchronizeAlternativeRoutesWith( AlternativeRoutesModel* model, QComboBox *view )
{
    d->m_alternativeRoutesModel = model;
    d->m_alternativeRoutesView = view;

    connect( d->m_alternativeRoutesModel, SIGNAL( rowsInserted( QModelIndex, int, int) ),
             this, SLOT( showAlternativeRoads() ) );
}*/

void MapViewTSLayer::setPointSelectionEnabled( bool enabled )
{
    d->m_pointSelection = enabled;
}

void MapViewTSLayer::setLayerDirty( bool dirty )
{
    d->m_layerDirty = dirty;

    /** @todo: The full repaint can be avoided. The route however has changed
      * and the exact bounding box needs to be recalculated before doing
      * a partly repaint, otherwise we might end up repainting only parts of the route
      */
    // d->m_marbleWidget->repaint( d->m_routeRegion );
	d->m_viewportChanged=true;
    d->m_marbleWidget->update();
}
/*
void RoutingLayer::setRouteSkeleton( RouteSkeleton *skeleton )
{
    d->m_routeSkeleton = skeleton;
}*/

void MapViewTSLayer::removeViaPoint()
{
    if ( d->m_activeMenuIndex >= 0 ) 
	{
     //   d->m_polySkeleton->remove( d->m_activeMenuIndex );

		//if(d->m_bObjId && d->m_selIdx.isValid())
		if(d->m_selIdx.isValid())
		{
			d->m_pNode = d->m_treeviewModel->getNodeFromIndex(d->m_selIdx);
			if(d->m_pNode!=NULL && d->m_pNode->ItemType != Rect)
			{
				/*d->m_pNode->m_poly.clear();
				for(int i=0;i<d->m_polySkeleton->size();i++)
					d->m_pNode->m_poly.append(d->m_polySkeleton->at(i));*/
				if(d->m_activeMenuIndex >-1 && d->m_pNode->m_poly.size()>d->m_activeMenuIndex)
					d->m_pNode->m_poly.remove(d->m_activeMenuIndex );
			}

		}
		d->m_activeMenuIndex = -1;
        setLayerDirty( true );
        //emit layerDirty();
    }
}

void MapViewTSLayer::insertViaPoint()
{
	if(d->m_selIdx.isValid())
	{
		d->m_pNode = d->m_treeviewModel->getNodeFromIndex(d->m_selIdx);
		if(d->m_pNode!=NULL && d->m_pNode->ItemType != Rect)
		{
			int xpos = d->m_mousePos.x();
			int ypos = d->m_mousePos.y();
			if(xpos>=0 && ypos>=0)
			{
				qreal lon=0;
				qreal lat=0;
				d->m_marbleWidget->geoCoordinates(xpos,ypos,lon,lat);
				GeoDataCoordinates position;
				GeoDataCoordinates posAt;
				GeoDataLineString gdl;
				position.setLongitude(lon,GeoDataCoordinates::Degree);
				position.setLatitude(lat,GeoDataCoordinates::Degree);
				int iSize=d->m_pNode->m_poly.size();
				bool set=false;
				if(d->m_pNode->ItemType == Poly)
				{
					for(int i=0;i<iSize;i++)
					{
						posAt = d->m_pNode->m_poly.at(i);
						if(!set && posAt.longitude()>position.longitude())
						{
							gdl << position;//insert
							gdl << posAt;
							set=true;
						}
						else gdl << posAt;
					}
					d->m_pNode->m_poly.clear();
					d->m_pNode->m_poly << gdl;
				}
				else if(d->m_pNode->ItemType == Polygone)
				{
					for(int i=0;i<iSize;i++)
					{
						posAt = d->m_pNode->m_poly.at(i);
						if(!set && posAt.longitude()>position.longitude() && (position.latitude()-(position.latitude()/100) )<=posAt.latitude()  && (position.latitude()+(position.latitude()/100) )>=posAt.latitude())
						{
							gdl << position;//insert
							gdl << posAt;
							set=true;
						}
						else gdl << posAt;
					}
					d->m_pNode->m_poly.clear();
					d->m_pNode->m_poly << gdl;
				}
			}

			//if(d->m_activeMenuIndex>=0)
			//	d->m_pNode->m_poly.remove(d->m_activeMenuIndex );
		}

	}
	d->m_activeMenuIndex = -1;
    setLayerDirty( true );
    //emit layerDirty();
   
}

void MapViewTSLayer::deleteObject()
{
	if(d->m_selIdx.isValid()&&d->m_multiItemSelect.count()<2)
	{
		/*if(d->m_selectionModel->isSelected( d->m_selIdx ) ) 
		{
			QItemSelectionModel::SelectionFlag command = QItemSelectionModel::Clear;
			d->m_selectionModel->select( d->m_selIdx, command );
		}*/

		d->m_bIsPoly=false;
		d->m_bObjId=false;
		d->m_pNode = d->m_treeviewModel->getNodeFromIndex(d->m_selIdx);
		d->m_activeMenuIndex = -1;

		if(d->m_pNode->ItemType != Overlay || d->m_pNode->ItemType != Layer) 
		{
			d->m_viewportChanged = true;
		/*	if(d->m_pNode->ItemType != Group)
			{
				d->m_placemarks.removeOne(ModelRegion(d->m_selIdx));
			}*/
		}

		//d->removePlacemark(d->m_selIdx);
		if(d->m_pNode!=NULL)
			emit deleteItem (d->m_selIdx,getInterfaceID());
		d->m_selIdx = QModelIndex();
		
	//	d->m_marbleWidget->update();
	}
	else if(d->m_multiItemSelect.count()>0)//delete multi selected items
	{
		d->m_bIsPoly=false;
		d->m_bObjId=false;
		d->m_activeMenuIndex = -1;
		int ic = d->m_multiItemSelect.count();
		for(int i=0;i<ic;i++)
		{
			QModelIndex idx=d->m_multiItemSelect.value(i);
			if(idx.isValid())
			{
				d->m_pNode = d->m_treeviewModel->getNodeFromIndex(idx);
				
				if(d->m_pNode->ItemType != Overlay || d->m_pNode->ItemType != Layer) 
				{
					d->m_viewportChanged = true;
				}
	
				if(d->m_pNode!=NULL)
					emit deleteItem (idx,getInterfaceID());
			}
		}
		d->m_selIdx = QModelIndex();
		d->m_multiItemSelect.clear();
	}
	d->m_pNode=NULL;
}

void MapViewTSLayer::slotDeleteTreeItem(int type,long id)
{
	//reset all selection members
	if(0)//d->m_selIdx.isValid())
	{
		d->removePlacemark(d->m_selIdx.internalId());
		if(d->m_selIdx.internalId()==id)
		{
		/*	if(d->m_selectionModel->isSelected( d->m_selIdx ) ) 
			{
				QItemSelectionModel::SelectionFlag command = QItemSelectionModel::Clear;
				d->m_selectionModel->select( d->m_selIdx, command );
			}*/

			d->m_selIdx = QModelIndex();
			d->m_bIsPoly=false;
			d->m_bObjId=false;
			d->m_activeMenuIndex = -1;
		//	d->m_marbleWidget->update();
		}		
	}
	else if(type==173)//Shape
	{
		qint64 qID=(qint64)id;
		if(qID>0)
			d->removePlacemark(qID);
	}
}

void MapViewTSLayer::slotLayerContextMenuHovered(QAction* pAction)
{
	if(pAction != NULL)
	{
		QMenu* pMenu = pAction->menu();
		if(pMenu!=NULL)
		{
			QString qsMenu = pMenu->whatsThis();
			m_cmID = qsMenu.toInt();
		}
	}
}

void MapViewTSLayer::slotSelectActionMenu(QAction* pAction)
{
	if(pAction==NULL)
		return;

	QVariant qVar = pAction->data();
	int iVal = qVar.toInt();
	if(iVal<0)return;

	int iSize = d->m_multiSelect.size();
	if(iVal>=iSize)return;
	int iAng = 360/iSize;
	double dRad=0;
	int iDist=50;
	if(iSize>5)
		iDist=iSize*10;
	int iXval=0;
	int iYval=0;

	for(int iX=0;iX<iSize;iX++)
	{
		QModelIndex idx = d->m_multiSelect.value(iX);
		if(idx.isValid() && iX==iVal)
		{
			d->m_selIdx = idx;
		//	d->m_bObjId = true;
			TreeNode* pNode = d->m_treeviewModel->getNodeFromIndex(idx);
			if(pNode!=NULL && (pNode->ItemType == Icon || pNode->ItemType == Marker) )
			{
				bool sel=true;
				QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;

				if(d->m_selectionModel->isSelected( d->m_selIdx ) ) 
				{
					command = QItemSelectionModel::Clear;
					//sel=false;
					if(d->m_bGeoProjected)
						emit placemarkSelected( d->m_selIdx,getInterfaceID(),false);
					//if(m_selIdx.internalId() == region.index.internalId() )
					//	sel=false;
				}
			/*    else if ( m_selectionModel->isSelected( region.index ) ) {
					command = QItemSelectionModel::Clear;
					sel=false;
				}*/
				else
				{
					 d->m_selectionModel->select( d->m_selIdx, command );
			 //   m_dropStopOver = e->pos();
			 //   storeDragPosition( e->pos() );
					if(d->m_bGeoProjected)
						emit placemarkSelected(d->m_selIdx,getInterfaceID(),sel);
				}

				d->m_movingIndex = -1;
	           
				// annotation and old annotation are dirty, large region
				d->m_marbleWidget->update();
			}


			d->m_bMultiSelect=false;
			m_cmID=-1;
		}//if idx && iVal
		else
		{
			TreeNode* pNode = d->m_treeviewModel->getNodeFromIndex(idx);
			if(pNode!=NULL && (pNode->ItemType == Icon || pNode->ItemType == Marker) )
			{
				pNode->m_bMultiSelect=true;
				dRad = (2*PI*(iAng*iX))/360.0;
				iYval = (int)(sin(dRad)*iDist);
				iXval = (int)(cos(dRad)*iDist);
				pNode->m_iXoff = iXval;
				pNode->m_iYoff = iYval;

			}

		}
	}//for
}

void MapViewTSLayer::slotSelectContextMenu()
{
	if(d->m_bMultiSelect && d->m_selectionMenu!=NULL && m_cmID>=0)
	{
//		qsMenu = pAction->toolTip();
//		int iPos = qsMenu.toInt();
		QModelIndex idx = d->m_multiSelect.value(m_cmID);
		if(idx.isValid())
		{
			d->m_selIdx = idx;
		//	d->m_bObjId = true;
			TreeNode* pNode = d->m_treeviewModel->getNodeFromIndex(idx);
			if(pNode!=NULL && (pNode->ItemType == Icon || pNode->ItemType == Marker) )
			{
				bool sel=true;
                QItemSelectionModel::SelectionFlag command = QItemSelectionModel::ClearAndSelect;

				if(d->m_selectionModel->isSelected( d->m_selIdx ) ) 
				{
                    command = QItemSelectionModel::Clear;
					//sel=false;
					if(d->m_bGeoProjected)
						emit placemarkSelected( d->m_selIdx,getInterfaceID(),false);
					//if(m_selIdx.internalId() == region.index.internalId() )
					//	sel=false;
                }
            /*    else if ( m_selectionModel->isSelected( region.index ) ) {
                    command = QItemSelectionModel::Clear;
					sel=false;
                }*/
				else
				{
					 d->m_selectionModel->select( d->m_selIdx, command );
             //   m_dropStopOver = e->pos();
             //   storeDragPosition( e->pos() );
					 if(d->m_bGeoProjected)
						emit placemarkSelected(d->m_selIdx,getInterfaceID(),sel);
				}

				d->m_movingIndex = -1;
				d->m_bIsPoly=false;
               
                // annotation and old annotation are dirty, large region
                d->m_marbleWidget->update();
			}


			d->m_bMultiSelect=false;
			m_cmID=-1;
		}
	}
}

void MapViewTSLayer::slotLayerContextMenu(QAction* pAction)
{
	if(d!=NULL && pAction!=NULL)
	{
		if(d->pActionGroup!=NULL)
		{
			QAction* pMenuAction=NULL;
			QVariant qVar;
			int iVal=0;
			QList<QAction*> pList = d->pActionGroup->actions();
			int i=pList.size();
			for(int x=0;x<i;x++)
			{
				pMenuAction = pList.at(x);
				if(pMenuAction==pAction)
				{
					qVar = pAction->data();
					iVal = qVar.toInt();
				//	QString qsMenu;
					if(d->m_bMultiSelect && d->m_selectionMenu!=NULL && m_cmID>=0)
					{
				//		qsMenu = pAction->toolTip();
				//		int iPos = qsMenu.toInt();
						QModelIndex idx = d->m_multiSelect.value(m_cmID);
						if(idx.isValid())
						{
							d->m_selIdx = idx;
						/*	TreeNode* pNode = d->m_treeviewModel->getNodeFromIndex(idx);
							if(pNode!=NULL)
							{
								qint64 id = pNode->m_lId;
							}*/
						}
					}
					d->m_bMultiSelect=false;
					d->m_bIsPoly=false;
					m_cmID=-1;
					if(d->m_selIdx.isValid())
						emit contextMenuSelected(d->m_selIdx,getInterfaceID(),iVal );
				}
			}
		}
	}

}

int MapViewTSLayer::intersectionObjects(QModelIndex& idx,QPoint pt,bool flag,int pos)
{
	int ret = d->getIntersectionObjects(idx,pt,flag,pos);
	return ret;
}

int MapViewTSLayer::intersectionRegionObjects(QModelIndex& idx,const QRect& rect,bool flag,int pos)
{
	int ret = d->getIntersectionRegionObjects(idx,rect,flag,pos);

	if(ret>0)
	{
		QModelIndexList items;
	//	QItemSelection selection;
	/*	QModelIndex idx1 = d->m_multiItemSelect.value(0);
		QModelIndex idx2 = d->m_multiItemSelect.value(ret-1);
		QItemSelection selection(idx2,idx1);

		//d->m_selectionModel->select(selection, QItemSelectionModel::Select|QItemSelectionModel::Rows );
		d->m_selectionModel->select(idx1,QItemSelectionModel::Select);
		d->m_selectionModel->select(idx2,QItemSelectionModel::Select);
		items = d->m_selectionModel->selectedIndexes();
		int ic=items.count();
	
		bool sel=true;
		if(d->m_bGeoProjected)
			emit placemarkSelected(idx1,getInterfaceID(),sel);

		d->m_multiItemSelect.clear();*/

		for(int i=0;i<ret;i++)
		{
			if(i < d->m_multiItemSelect.size() )
			{
			
				idx = d->m_multiItemSelect.value(i);
				if(!idx.isValid())return i;
			
				TreeNode* pNode = d->m_treeviewModel->getNodeFromIndex(idx);
				if(pNode!=NULL && (pNode->ItemType == Icon || pNode->ItemType == Marker) )
				{
					items << idx;
					bool sel=true;
					QItemSelectionModel::SelectionFlag command = QItemSelectionModel::Select;

					QItemSelection selidx(idx,idx);
					//selection.merge(selidx, QItemSelectionModel::Select);

					d->m_selectionModel->select(selidx, QItemSelectionModel::Select);//|QItemSelectionModel::Rows );
					//d->m_selectionModel->select(idx, command );
					items = d->m_selectionModel->selectedIndexes();
					int ic=items.count();
				
					if(d->m_bGeoProjected)
						emit placemarkMultiSelected(idx,getInterfaceID(),sel);

					//d->m_multiItemSelect.clear();
				}
			}//i
		}//for 
	}//ret
//	d->m_multiItemSelect.clear();
	return ret;

}

void MapViewTSLayer::setViewportChanged()
{
	if(d!=NULL)
	{
		d->m_viewportChanged = true;
		d->m_contentChanged = true;
	}
}

void MapViewTSLayer::setLayerUpdate(bool val)
{
	if(d!=NULL)
	{
		d->m_viewportChanged = val;
		d->m_contentChanged = val;
	}

}

void MapViewTSLayer::setInputEnabled(bool val)
{
	if(d!=NULL)
		d->m_bInputEnabled = val;
}

void MapViewTSLayer::setLayerZValue(qreal zValue)
{
	m_zValue = zValue;
}

// Overriding LayerInterface to paint on top of the route
qreal MapViewTSLayer::zValue() const
{
	return m_zValue;
}

void MapViewTSLayer::setLegendLayer(bool legend)
{
	if(d!=NULL)
		d->m_bGeoProjected = legend;
}

bool MapViewTSLayer::getLegendLayer()
{
	if(d!=NULL)
		return d->m_bGeoProjected;
	else return true;
}

void MapViewTSLayer::setLegendLayerAlignment(int align)
{
	if(d!=NULL)
		d->m_iLegendAlign = align;
}
	
void MapViewTSLayer::setLegendRect(QRect& rect)
{
	if(d!=NULL)
	{
		if(rect.left()<d->m_legendRect.left())
			d->m_legendRect.setLeft(rect.left());
		if(rect.right()>d->m_legendRect.right())
			d->m_legendRect.setRight(rect.right());
		if(rect.top()<d->m_legendRect.top())
			d->m_legendRect.setTop(rect.top());
		if(rect.bottom()>d->m_legendRect.bottom())
			d->m_legendRect.setBottom(rect.bottom());
	}

}

void MapViewTSLayer::export2kml(QString& filename)
{
	if(d!=NULL)
		d->export2kml(filename);

}

void  MapViewTSLayer::exportOverlay2kml(GeoDataDocument* document,GeoDataFolder* folder)
{
	if(d!=NULL)
		d->exportOverlay2kml(document,folder);

}

} // namespace Marble

#include "moc_MapViewTSLayer.cpp"
